@@ -5,7 +5,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -34,37 +34,7 @@ import sys
 from cs_case_domain import *
 from cs_case import *
 import cs_exec_environment
-
-#-------------------------------------------------------------------------------
-# Extract a parameters file name from a shell run sript
-#-------------------------------------------------------------------------------
-
-def get_param(path):
-    """
-    Extract a parameters file name from a shell run sript
-    """
-    f = open(path, 'r')
-    lines = f.readlines()
-    f.close()
-
-    param = None
-
-    for line in lines:
-        if line[0] == '#':
-            continue
-        i = line.find('#')
-        if i > -1:
-            line = line[0:i]
-        line = line.strip()
-        if len(line) == 0:
-            continue
-        args = cs_exec_environment.separate_args(line)
-        if args.count('run') > 0:
-            if args.index('run') > 0: # "<package.name> run"
-                param = cs_exec_environment.get_command_single_value \
-                    (args, ('--param=', '--param', '-p'))
-
-    return param
+import cs_runcase
 
 #===============================================================================
 # Main function for code coupling execution
@@ -73,11 +43,15 @@ def get_param(path):
 def coupling(package,
              domains,
              casedir,
+             staging_dir = None,
+             verbose = True,
              package_compute = None):
 
     use_saturne = False
     use_syrthes = False
     use_neptune = False
+    use_cathare = False
+    use_py_code = False
 
     # Use alternate compute (back-end) package if defined
 
@@ -91,8 +65,10 @@ def coupling(package,
     sat_domains = []
     syr_domains = []
     nep_domains = []
-    ast_domain = None
+    cat_domains = []
+    ast_domain = []
     fsi_coupler = None
+    py_domains = []
 
     if domains == None:
         raise RunCaseError('No domains defined.')
@@ -105,18 +81,27 @@ def coupling(package,
             msg += 'script or domain key is missing.'
             raise RunCaseError(msg)
 
-        if (d.get('solver') == 'Code_Saturne' or d.get('solver') == 'Saturne'):
+        if (d.get('solver') == 'Code_Saturne' or d.get('solver') == 'NEPTUNE_CFD'):
 
-            try:
-                runcase = os.path.join(os.getcwd(),
-                                       d.get('domain'),
-                                       'SCRIPTS',
-                                       d.get('script'))
-                param = get_param(runcase)
+            script = d.get('script')
 
-            except Exception:
-                err_str = 'Cannot read Code_Saturne script: ' + runcase
-                raise RunCaseError(err_str)
+            if script[-4:] == '.xml':
+                param = script
+
+            else:
+                runcase_path = os.path.join(os.getcwd(),
+                                            d.get('domain'),
+                                            'SCRIPTS',
+                                            script)
+                try:
+                    runcase = cs_runcase.runcase(runcase_path)
+                    param = runcase.get_parameters()
+
+                except Exception:
+                    err_str = 'Cannot read ' + d.get('solver') \
+                              + ' script: ' + runcase_path
+                    raise RunCaseError(err_str)
+                d['script'] = param
 
             dom = domain(package,
                          package_compute = package_compute,
@@ -126,8 +111,12 @@ def coupling(package,
                          n_procs_min = d.get('n_procs_min'),
                          n_procs_max = d.get('n_procs_max'))
 
-            use_saturne = True
-            sat_domains.append(dom)
+            if (d.get('solver')) == 'Code_Saturne':
+                use_saturne = True
+                sat_domains.append(dom)
+            elif (d.get('solver')) == 'NEPTUNE_CFD':
+                use_neptune = True
+                nep_domains.append(dom)
 
         elif (d.get('solver') == 'SYRTHES'):
 
@@ -150,33 +139,9 @@ def coupling(package,
             use_syrthes = True
             syr_domains.append(dom)
 
-        elif (d.get('solver') == 'NEPTUNE_CFD'):
-
-            try:
-                runcase = os.path.join(os.getcwd(),
-                                       d.get('domain'),
-                                       'SCRIPTS',
-                                       d.get('script'))
-                param = get_param(runcase)
-
-            except Exception:
-                err_str = 'Cannot read NEPTUNE_CFD script: ' + runcase
-                raise RunCaseError(err_str)
-
-            dom = domain(package,
-                         package_compute = package_compute,
-                         name = d.get('domain'),
-                         param = param,
-                         n_procs_weight = d.get('n_procs_weight'),
-                         n_procs_min = d.get('n_procs_min'),
-                         n_procs_max = d.get('n_procs_max'))
-
-            use_neptune = True
-            nep_domains.append(dom)
-
         elif (d.get('solver') == 'Code_Aster' or d.get('solver') == 'Aster'):
 
-            if ast_domain:
+            if len(ast_domain) > 0:
                 err_str = 'Only 1 Code_Aster domain is currently handled\n'
                 raise RunCaseError(err_str)
 
@@ -191,7 +156,7 @@ def coupling(package,
                 err_str += ' script = ' + d.get('script') + '\n'
                 raise RunCaseError(err_str)
 
-            ast_domain = dom
+            ast_domain.append(dom)
 
         elif (d.get('coupler') == 'FSI_coupler'):
 
@@ -215,31 +180,127 @@ def coupling(package,
                 err_str += '  epsilon = ' + d.get('epsilon') + '\n'
                 raise RunCaseError(err_str)
 
+        elif (d.get('solver') == 'CATHARE'):
+            # Current version using Cathare2: the cathare case is converted to a
+            # .so library which is opened and launched by a NEPTUNE_CFD executable
+
+            script = d.get('script')
+
+            if script[-4:] == '.xml':
+                param = script
+
+            else:
+                runcase_path = os.path.join(os.getcwd(),
+                                            d.get('domain'),
+                                            'SCRIPTS',
+                                            script)
+                try:
+                    runcase = cs_runcase.runcase(runcase_path)
+                    param = runcase.get_parameters()
+
+                except Exception:
+                    err_str = 'Cannot read ' + d.get('solver') \
+                              + ' script: ' + runcase_path
+                    raise RunCaseError(err_str)
+                d['script'] = param
+
+            dom = cathare_domain(package,
+                                 package_compute = package_compute,
+                                 name = d.get('domain'),
+                                 param = d.get('paramfile'),
+                                 n_procs_weight = d.get('n_procs_weight'),
+                                 n_procs_min = d.get('n_procs_min'),
+                                 n_procs_max = d.get('n_procs_max'),
+                                 cathare_case_file=d.get('cathare_case_file'),
+                                 neptune_cfd_dom=d.get('neptune_cfd_domain'))
+
+            use_cathare = True
+            cat_domains.append(dom)
+
+        elif (d.get('solver') == 'PYTHON_CODE'):
+            # Generic Code_Saturne/Python Script coupling
+            # The python script can contain any MPI compatible code or supevisor
+
+            try:
+                dom = python_domain(package,
+                                    name = d.get('domain'),
+                                    cmd_line = d.get('command_line'),
+                                    script_name = d.get('py_code'))
+
+            except Exception:
+                err_str = 'Cannot create Python code domain.\n'
+                err_str += ' domain = ' + d.get('domain') + '\n'
+                err_str += ' script = ' + d.get('script') + '\n'
+                raise RunCaseError(err_str)
+
+            use_py_code = True
+            py_domains.append(dom)
+
         else:
             err_str = 'Unknown code type : ' + d.get('solver') + '.\n'
             raise RunCaseError(err_str)
 
     # Now handle case for the corresponding calculation domain(s).
 
+    coupling_parameters = \
+"""
+# -*- coding: utf-8 -*-"
+
+domains = [
+
+"""
+
+    d_first = True
+    for d in domains:
+        if not d_first:
+            coupling_parameters += ",\n\n"
+        d_first = False
+        k_first = True
+        for k in ('solver', 'domain', 'script', 'n_procs_weight', 'n_procs_min',
+                  'coupler', 'max_time_steps', 'n_sub_iterations', 'time_step',
+                  'start_time', 'epsilon'):
+            v = d.get(k)
+            if v:
+                if type(v) == str:
+                    s_v = "'" + v + "'"
+                else:
+                    s_v = str(v)
+                if k_first:
+                    coupling_parameters += "    {"
+                    k_first = False
+                else:
+                    coupling_parameters += ",\n     "
+                coupling_parameters += "'" + k + "': " + s_v
+        coupling_parameters += "}"
+    coupling_parameters += "\n\n    ]\n\n"
+
     c = case(package,
              package_compute = package_compute,
              case_dir = casedir,
-             domains = sat_domains + nep_domains,
+             staging_dir = staging_dir,
+             coupling_parameters = coupling_parameters,
+             domains = sat_domains + nep_domains + cat_domains,
              syr_domains = syr_domains,
              ast_domain = ast_domain,
-             fsi_coupler = fsi_coupler)
+             fsi_coupler = fsi_coupler,
+             py_domains = py_domains)
 
-    msg = ' Coupling execution between: \n'
-    if use_saturne == True:
-        msg += '   o Code_Saturne [' + str(len(sat_domains)) + ' domain(s)];\n'
-    if use_syrthes == True:
-        msg += '   o SYRTHES      [' + str(len(syr_domains)) + ' domain(s)];\n'
-    if use_neptune == True:
-        msg += '   o NEPTUNE_CFD  [' + str(len(nep_domains)) + ' domain(s)];\n'
-    if ast_domain or fsi_coupler:
-        msg += '   o Code_Aster   [1 domain(s)];\n'
-        msg += '                  [1 coupler(s)];\n'
-    sys.stdout.write(msg+'\n')
+    if verbose:
+        msg = ' Coupling execution between: \n'
+        if use_saturne == True:
+            msg += '   o Code_Saturne [' + str(len(sat_domains)) + ' domain(s)];\n'
+        if use_syrthes == True:
+            msg += '   o SYRTHES      [' + str(len(syr_domains)) + ' domain(s)];\n'
+        if use_neptune == True:
+            msg += '   o NEPTUNE_CFD  [' + str(len(nep_domains)) + ' domain(s)];\n'
+        if ast_domain or fsi_coupler:
+            msg += '   o Code_Aster   [1 domain(s)];\n'
+            msg += '                  [1 coupler(s)];\n'
+        if use_cathare == True:
+            msg += '   o CATHARE2     [' + str(len(cat_domains)) + ' domain(s)];\n'
+        if use_py_code == True:
+            msg += '   o Python Script  [' + str(len(py_domains)) + ' domain(s)];\n'
+        sys.stdout.write(msg+'\n')
 
     return c
 
