@@ -5,7 +5,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -29,17 +29,19 @@ This module describes the script used to create a study/case for Code_Saturne.
 This module defines the following functions:
 - usage
 - process_cmd_line
-- make_executable
+- set_executable
+- unset_executable
 and the following classes:
 - study
 - class
 """
 
-
 #-------------------------------------------------------------------------------
 # Library modules import
 #-------------------------------------------------------------------------------
 
+from __future__ import print_function
+
 import os, sys, shutil, stat
 import types, string, re, fnmatch
 from optparse import OptionParser
@@ -50,6 +52,7 @@ except Exception:
     import configparser  # Python3
 
 import cs_exec_environment
+import cs_runcase
 
 #-------------------------------------------------------------------------------
 # Process the passed command line arguments
@@ -79,6 +82,10 @@ def process_cmd_line(argv, pkg):
                       metavar="<case>",
                       help="create a case from another one")
 
+    parser.add_option("--import-only", dest="import_only",
+                      action="store_true",
+                      help="rebuild scripts of existing case")
+
     parser.add_option("--noref", dest="use_ref",
                       action="store_false",
                       help="don't copy references")
@@ -99,14 +106,25 @@ def process_cmd_line(argv, pkg):
                       metavar="<ast_case>",
                       help="create a new Code_Aster case.")
 
+    parser.add_option("--cathare", dest="cat_case_name", type="string",
+                      metavar="<cat_case>",
+                      help="create a new Cathare2 case.")
+
+    parser.add_option("--python", dest="py_case_name", type="string",
+                      metavar="<py_case>",
+                      help="create a new Python script case.")
+
     parser.set_defaults(use_ref=True)
     parser.set_defaults(study_name=os.path.basename(os.getcwd()))
     parser.set_defaults(case_names=[])
     parser.set_defaults(copy=None)
     parser.set_defaults(verbose=1)
+    parser.set_defaults(import_only=False)
     parser.set_defaults(n_sat=1)
     parser.set_defaults(syr_case_names=[])
     parser.set_defaults(ast_case_name=None)
+    parser.set_defaults(cat_case_name=None)
+    parser.set_defaults(py_case_name=None)
 
     (options, args) = parser.parse_args(argv)
 
@@ -114,34 +132,68 @@ def process_cmd_line(argv, pkg):
         if len(args) > 0:
             options.case_names = args
         else:
-            options.case_names = ["CASE1"]
+            if not options.import_only:
+                options.case_names = ["CASE1"]
+            else:
+                options.case_names = [""]
 
     return Study(pkg,
                  options.study_name,
                  options.case_names,
                  options.syr_case_names,
                  options.ast_case_name,
+                 options.cat_case_name,
+                 options.py_case_name,
                  options.copy,
+                 options.import_only,
                  options.use_ref,
                  options.verbose)
 
-
 #-------------------------------------------------------------------------------
 # Assign executable mode (chmod +x) to a file
 #-------------------------------------------------------------------------------
 
-def make_executable(filename):
+def set_executable(filename):
     """
     Give executable permission to a given file.
     Equivalent to `chmod +x` shell function.
     """
 
     st   = os.stat(filename)
-    mode = st[stat.ST_MODE]
-    os.chmod(filename, mode | stat.S_IEXEC)
+    mode = st[stat.ST_MODE] | stat.S_IXUSR
+    if mode & stat.S_IRGRP:
+        mode = mode | stat.S_IXGRP
+    if mode & stat.S_IROTH:
+        mode = mode | stat.S_IXOTH
+    os.chmod(filename, mode)
 
     return
 
+#-------------------------------------------------------------------------------
+# Remove executable mode (chmod -x) from a file or files inside a directory
+#-------------------------------------------------------------------------------
+
+def unset_executable(path):
+    """
+    Remove executable permission from a given file or files inside a directory.
+    Equivalent to `chmod -x` shell function.
+    """
+
+    if os.path.isfile(path):
+        try:
+            st   = os.stat(path)
+            mode = st[stat.ST_MODE] | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH
+            mode = mode ^ (stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)
+            os.chmod(path, mode)
+        except Exception:
+            pass
+
+    elif os.path.isdir(path):
+        l = os.listdir(path)
+        for p in l:
+            unset_executable(p)
+
+    return
 
 #-------------------------------------------------------------------------------
 # Build lines necessary to import SYRTHES packages
@@ -171,7 +223,8 @@ def syrthes_path_line(pkg):
 class Study:
 
     def __init__(self, package, name, cases, syr_case_names, ast_case_name,
-                 copy, use_ref, verbose):
+                 cat_case_name, py_case_name,
+                 copy, import_only, use_ref, verbose):
         """
         Initialize the structure for a study.
         """
@@ -184,6 +237,7 @@ class Study:
         self.copy = copy
         if self.copy is not None:
             self.copy = os.path.abspath(self.copy)
+        self.import_only = import_only
         self.use_ref = use_ref
         self.verbose = verbose
 
@@ -198,19 +252,18 @@ class Study:
 
         self.ast_case_name = ast_case_name
 
+        self.cat_case_name = cat_case_name
 
-    def get_syrthes_version(self):
-        """
-        Get available SYRTHES version.
-        """
-        syrthes_version = None
+        self.py_case_name = py_case_name
 
-        config = configparser.ConfigParser()
-        config.read(self.package.get_configfiles())
-        if config.has_option('install', 'syrthes'):
-            syrthes_version = 4
+        if self.import_only:
+            self.use_ref = False
+            self.copy = None
 
-        return syrthes_version
+            # Options is now deprecated, print info
+            sys.stdout.write("  o '%s create --import-only' is deprecated.\n" % \
+                    self.package.name)
+            sys.stdout.write("  o Use '%s update' instead\n\n" % self.package.name)
 
 
     def create(self):
@@ -220,13 +273,18 @@ class Study:
 
         if self.name != os.path.basename(os.getcwd()):
 
-            if self.verbose > 0:
-                sys.stdout.write("  o Creating study '%s'...\n" % self.name)
+            if not self.import_only:
+                if self.verbose > 0:
+                    sys.stdout.write("  o Creating study '%s'...\n" % self.name)
+                os.mkdir(self.name)
+                os.chdir(self.name)
+                os.mkdir('MESH')
+                os.mkdir('POST')
 
-            os.mkdir(self.name)
-            os.chdir(self.name)
-            os.mkdir('MESH')
-            os.mkdir('POST')
+            else:
+                if self.verbose > 0:
+                    sys.stdout.write("  o Importing study '%s'...\n" % self.name)
+                os.chdir(self.name)
 
         # Creating Code_Saturne cases
         repbase = os.getcwd()
@@ -236,12 +294,7 @@ class Study:
 
         # Creating SYRTHES cases
         if len(self.syr_case_names) > 0:
-            syrthes_version = self.get_syrthes_version()
-            if syrthes_version == 4:
-                self.create_syrthes_cases(repbase)
-            else:
-                sys.stderr.write("Cannot locate SYRTHES installation.")
-                sys.exit(1)
+            self.create_syrthes_cases(repbase)
 
         # Creating Code_Aster case
         if self.ast_case_name is not None:
@@ -254,8 +307,23 @@ class Study:
                 sys.stderr.write("Cannot locate Code_Aster installation.")
                 sys.exit(1)
 
+        # Creating Cathare case
+        if self.cat_case_name is not None:
+            config = configparser.ConfigParser()
+            config.read(self.package.get_configfiles())
+            if config.has_option('install', 'cathare'):
+                self.create_cathare_case(repbase, config.get('install', 'cathare'))
+            else:
+                sys.stderr.write("Cannot locate Cathare installation.")
+                sys.exit(1)
+
+        # Creating the Python script case
+        if self.py_case_name is not None:
+            self.create_python_case(repbase)
+
         # Creating coupling structure
-        if len(self.cases) + len(self.syr_case_names) > 1 or self.ast_case_name:
+        if len(self.cases) + len(self.syr_case_names) > 1 or self.ast_case_name \
+           or self.cat_case_name or self.py_case_name:
             self.create_coupling(repbase)
 
 
@@ -264,28 +332,23 @@ class Study:
         Create and initialize SYRTHES case directories.
         """
 
-        try:
-            config = configparser.ConfigParser()
-            config.read(self.package.get_configfiles())
-            syr_datapath = os.path.join(config.get('install', 'syrthes'),
-                                        os.path.join('share', 'syrthes'))
-            sys.path.insert(0, syr_datapath)
-            if not os.getenv('SYRTHES4_HOME'):
-                syr_profile = os.path.join(config.get('install', 'syrthes'),
-                                           'bin', 'syrthes.profile')
-                cs_exec_environment.source_shell_script(syr_profile)
-            import syrthes
-        except Exception:
-            sys.stderr.write("SYRTHES create case: Cannot locate SYRTHES installation.\n")
-            sys.exit(1)
+        os_env_save = os.environ
+
+        cs_exec_environment.source_syrthes_env(self.package)
+        import syrthes
 
         for s in self.syr_case_names:
             os.chdir(repbase)
-            retval = syrthes.create_syrcase(s)
+            if not self.import_only:
+                retval = syrthes.create_syrcase(s)
+            else:
+                retval = 0
             if retval > 0:
                 sys.stderr.write("Cannot create SYRTHES case: '%s'\n" % s)
                 sys.exit(1)
 
+        os_environ = os_env_save
+
 
     def create_aster_case(self, repbase):
         """
@@ -297,7 +360,9 @@ class Study:
                              self.ast_case_name)
 
         c = os.path.join(repbase, self.ast_case_name)
-        os.mkdir(c)
+
+        if not self.import_only:
+            os.mkdir(c)
 
         datadir = self.package.get_dir("pkgdatadir")
         try:
@@ -309,6 +374,60 @@ class Study:
             sys.exit(1)
 
 
+    def create_cathare_case(self, repbase, cathare_path):
+        """
+        Create and initialize Cathare case directory for coupling.
+        """
+
+        os.chdir(repbase)
+        self.create_case(self.cat_case_name)
+
+        # Copying the Cathare coupling necessary files for the cases
+        datadir = os.path.join(self.package.get_dir("datadir"),
+                               self.package.name)
+
+        if os.path.isdir(os.path.join(datadir, 'cathare_coupling')):
+
+            cathare_cpl_dir = os.path.join(datadir, 'cathare_coupling')
+
+            # Script which converts the Cathare JDD file to a .so library
+            fin  = open(os.path.join(cathare_cpl_dir, 'user_scripts', 'cathare_jdd_to_lib.sh'))
+            fout = open(os.path.join(repbase, self.cat_case_name, 'DATA', 'cathare_jdd_to_lib.sh'), 'wt')
+            for line in fin:
+                fout.write(line.replace('#path_to_v25_3=',
+                                        'export v25_3='+cathare_path))
+            fin.close()
+            fout.close()
+
+
+        else:
+            sys.stderr.write("Cannot find the directory: " + \
+                             os.path.join(datadir, 'cathare_coupling') + "\n")
+            sys.exit(1)
+
+    def create_python_case(self, repbase):
+        """
+        Create and initialize Python code case directory for coupling
+        """
+
+        os.chdir(repbase)
+
+        if self.verbose > 0:
+            sys.stdout.write("  o Creating Python code case  '%s'...\n" %
+                             self.py_case_name)
+
+        c = os.path.join(repbase, self.py_case_name)
+
+        if not os.path.isdir(c):
+            os.mkdir(c)
+
+        os.chdir(c)
+
+        for d in ["DATA", "SRC", "RESU"]:
+            if not os.path.isdir(d):
+                os.mkdir(d)
+
+
     def create_coupling(self, repbase):
         """
         Create structure to enable code coupling.
@@ -348,6 +467,9 @@ domains = [
 
         for c in self.cases:
 
+            c = os.path.normpath(c)
+            base_c = os.path.basename(c)
+
             template = sep + \
 """
     {'solver': 'PACKAGE',
@@ -358,7 +480,7 @@ domains = [
      'n_procs_max': None}
 """
             template = re.sub(e_pkg, solver_name, template)
-            template = re.sub(e_dom, c, template)
+            template = re.sub(e_dom, base_c, template)
 
             dict_str += template
 
@@ -370,6 +492,9 @@ domains = [
 
         for c in self.syr_case_names:
 
+            c = os.path.normpath(c)
+            base_c = os.path.basename(c)
+
             template = \
 """
     ,
@@ -382,7 +507,7 @@ domains = [
      'opt' : ''}               # Additional SYRTHES options
                                # (ex.: postprocessing with '-v ens' or '-v med')
 """
-            template = re.sub(e_dom, c, template)
+            template = re.sub(e_dom, base_c, template)
             dict_str += template
 
         if self.ast_case_name is not None:
@@ -409,6 +534,51 @@ domains = [
 """
             dict_str += template
 
+        if self.cat_case_name is not None:
+
+            c = os.path.normpath(self.cat_case_name)
+            base_c = os.path.basename(c)
+
+            template = \
+"""
+    ,
+    {'solver': 'CATHARE',
+     'domain': 'DOMAIN',
+     'script': 'runcase',
+     'paramfile': 'setup.xml',
+     'cathare_case_file': 'jdd_case.dat',
+     'neptune_cfd_domain': 'NEPTUNE',
+     'n_procs_weight': None,
+     'n_procs_min': 1,
+     'n_procs_max': 1}
+"""
+
+            template = re.sub(e_pkg, solver_name, template)
+            template = re.sub(e_dom, base_c, template)
+
+            dict_str += template
+
+
+        if self.py_case_name is not None:
+
+            c = os.path.normpath(self.py_case_name)
+            base_c = os.path.basename(c)
+
+            template = \
+"""
+    ,
+    {'solver': 'PYTHON_CODE',
+     'domain': 'DOMAIN',
+     'py_code': 'pycode.py',
+     'command_line': '',
+     'n_procs_weight': None,
+     'n_procs_min': 1,
+     'n_procs_max': 1}
+"""
+
+            template = re.sub(e_dom, self.py_case_name, template)
+            dict_str += template
+
         # Now finish building dictionnary string
 
         dict_str += \
@@ -419,7 +589,8 @@ domains = [
         # Result directory for coupling execution
 
         resu = os.path.join(repbase, 'RESU_COUPLING')
-        os.mkdir(resu)
+        if not self.import_only:
+            os.mkdir(resu)
 
         coupling_base = 'coupling_parameters.py'
         coupling = os.path.join(repbase, coupling_base)
@@ -435,9 +606,17 @@ domains = [
 
         fd.write(dict_str)
 
+        if self.cat_case_name is not None:
+            config = configparser.ConfigParser()
+            config.read(self.package.get_configfiles())
+            cathare_libpath=config.get('install', 'cathare')
+        else:
+            cathare_libpath=None
+
         self.build_batch_file(distrep = repbase,
                               casename = 'coupling',
-                              coupling = coupling_base)
+                              coupling = coupling_base,
+                              cathare_path = cathare_libpath)
 
 
     def create_case(self, casename):
@@ -445,26 +624,34 @@ domains = [
         Create a case for a Code_Saturne study.
         """
 
+        casedirname = casename
+
         if self.verbose > 0:
-            sys.stdout.write("  o Creating case  '%s'...\n" % casename)
+            if not self.import_only:
+                sys.stdout.write("  o Creating case  '%s'...\n" % casename)
+            else:
+                if not casename:
+                    casedirname = "."
+                    casename = os.path.basename(os.getcwd())
+                sys.stdout.write("  o Importing case  '%s'...\n" % casename)
 
         datadir = self.package.get_dir("pkgdatadir")
         data_distpath  = os.path.join(datadir, 'data')
-        user_distpath = os.path.join(datadir, 'user')
-        if self.package.name == 'code_saturne' :
-            user_examples_distpath = os.path.join(datadir, 'user_examples')
 
-        try:
-            os.mkdir(casename)
-        except:
-            sys.exit(1)
+        if not self.import_only:
+            try:
+                os.mkdir(casedirname)
+            except:
+                sys.exit(1)
 
-        os.chdir(casename)
+        os.chdir(casedirname)
 
         # Data directory
 
         data = 'DATA'
-        os.mkdir(data)
+
+        if not self.import_only:
+            os.mkdir(data)
 
         if self.use_ref:
 
@@ -472,14 +659,14 @@ domains = [
             ref           = os.path.join(data, 'REFERENCE')
             os.mkdir(ref)
             for f in ['dp_C3P', 'dp_C3PSJ', 'dp_C4P', 'dp_ELE',
-                      'dp_FCP.xml',
-                      'dp_FUE', 'dp_FUE_new', 'dp_transfo',
-                      'meteo']:
+                      'dp_FUE', 'dp_transfo', 'meteo']:
                 abs_f = os.path.join(thch_distpath, f)
                 if os.path.isfile(abs_f):
                     shutil.copy(abs_f, ref)
+                    unset_executable(ref)
             abs_f = os.path.join(datadir, 'cs_user_scripts.py')
             shutil.copy(abs_f, ref)
+            unset_executable(ref)
 
         # Write a wrapper for GUI launching
 
@@ -501,20 +688,45 @@ domains = [
 
         fd.close()
 
-        make_executable(guiscript)
+        set_executable(guiscript)
 
         # User source files directory
 
-        src = 'SRC'
-        os.mkdir(src)
+        if not self.import_only:
+            src = 'SRC'
+            os.mkdir(src)
 
         if self.use_ref:
 
+            user_distpath = os.path.join(datadir, 'user')
+            user_examples_distpath = os.path.join(datadir, 'user_examples')
+
             user = os.path.join(src, 'REFERENCE')
             user_examples = os.path.join(src, 'EXAMPLES')
             shutil.copytree(user_distpath, user)
-            if self.package.name == 'code_saturne' :
-                shutil.copytree(user_examples_distpath, user_examples)
+            shutil.copytree(user_examples_distpath, user_examples)
+
+            add_datadirs = []
+            if self.package.name == 'neptune_cfd' :
+                add_datadirs.append(os.path.join(self.package.get_dir("datadir"),
+                                                 self.package.name))
+
+            for d in add_datadirs:
+                user_distpath = os.path.join(d, 'user')
+                user_examples_distpath = os.path.join(d, 'user_examples')
+
+                if os.path.isdir(user_distpath):
+                    s_files = os.listdir(user_distpath)
+                    for f in s_files:
+                        shutil.copy(os.path.join(user_distpath, f), user)
+
+                if os.path.isdir(user_examples_distpath):
+                    s_files = os.listdir(user_examples_distpath)
+                    for f in s_files:
+                        shutil.copy(os.path.join(user_examples_distpath, f), user_examples)
+
+            unset_executable(user)
+            unset_executable(user_examples)
 
         # Copy data and source files from another case
 
@@ -531,11 +743,15 @@ domains = [
                        f not in [self.package.guiname,
                                  'preprocessor_output']:
                     shutil.copy(os.path.join(ref_data, abs_f), data)
+                    unset_executable(os.path.join(data, f))
 
             # Source files
 
             ref_src = os.path.join(self.copy, src)
-            src_files = os.listdir(ref_src)
+            if os.path.exists(ref_src):
+                src_files = os.listdir(ref_src)
+            else:
+                src_files = []
 
             c_files = fnmatch.filter(src_files, '*.c')
             cxx_files = fnmatch.filter(src_files, '*.cxx')
@@ -548,81 +764,70 @@ domains = [
             for f in c_files + h_files + f_files + \
                     cxx_files + cpp_files + hxx_files + hpp_files:
                 shutil.copy(os.path.join(ref_src, f), src)
+                unset_executable(os.path.join(src, f))
 
         # Results directory (only one for all instances)
 
         resu = 'RESU'
-        os.mkdir(resu)
+        if not os.path.isdir(resu):
+            os.mkdir(resu)
 
         # Script directory (only one for all instances)
 
         scripts = 'SCRIPTS'
-        os.mkdir(scripts)
+        if not os.path.isdir(scripts):
+            os.mkdir(scripts)
 
         self.build_batch_file(distrep = os.path.join(os.getcwd(), scripts),
                               casename = casename)
 
 
-    def build_batch_file(self, distrep, casename, coupling=None):
+    def build_batch_file(self, distrep, casename, coupling=None, cathare_path=None):
         """
         Retrieve batch file for the current system
         Update batch file for the study
         """
 
-
         batch_file = os.path.join(distrep, 'runcase')
         if sys.platform.startswith('win'):
             batch_file = batch_file + '.bat'
 
-        fd = open(batch_file, 'w')
-        cs_exec_environment.write_shell_shebang(fd)
+        if self.copy is not None:
+            ref_runcase_path = os.path.join(self.copy, 'SCRIPTS', 'runcase')
+            if sys.platform.startswith('win'):
+                ref_runcase_path += '.bat'
+            try:
+                shutil.copy(ref_runcase_path, batch_file)
+            except Exception:
+                pass
 
-        # Add batch system info if necessary
+        # Add info from parent in case of copy
 
-        config = configparser.ConfigParser()
-        config.read(self.package.get_configfiles())
+        runcase = cs_runcase.runcase(batch_file,
+                                     package=self.package,
+                                     rebuild=True,
+                                     study_name=self.name,
+                                     case_name=casename)
 
-        if config.has_option('install', 'batch'):
-
-            template = config.get('install', 'batch')
-
-            if not os.path.isabs(template):
-                template = os.path.join(self.package.get_batchdir(),
-                                        'batch.' + template)
-
-            fdt = open(template, 'r')
-
-            kwd1 = re.compile('nameandcase')
-
-            studycasename = string.lower(self.name) + string.lower(casename)
-
-            # For some systems, names are limited to 15 caracters
-            studycasename = studycasename[:15]
-
-            for line in fdt:
-                line = re.sub(kwd1, studycasename, line)
-                fd.write(line)
-
-            fdt.close()
-
-        # Add command to execute.
-
-        cs_exec_environment.write_script_comment(fd,
-                                                 'Ensure the correct command is found:\n')
-        cs_exec_environment.write_prepend_path(fd, 'PATH',
-                                               self.package.get_dir("bindir"))
-        fd.write('\n')
-        cs_exec_environment.write_script_comment(fd, 'Run command:\n')
-        # On Linux systems, add a backslash to prevent aliases
-        if sys.platform.startswith('linux'): fd.write('\\')
-        fd.write(self.package.name + ' run')
         if coupling:
-            fd.write(' --coupling ' + coupling)
-        fd.write('\n')
+            runcase.set_coupling(coupling)
 
-        fd.close()
+        # If a cathare LIBPATH is given, it is added to LD_LIBRARY_PATH.
+        # This modification is needed for the dlopen of the cathare .so file
+        if cathare_path:
+            v25_3_line="export v25_3=%s" % cathare_path
+            new_line="export LD_PATH_LIBRARY=$v25_3/%s/"+":$LD_LIBRARY_PATH"
+            il=0
+            for line in runcase.lines:
+                il+=1
+                if 'export PATH' in line:
+                    runcase.lines.insert(il, new_line % ("lib/ICoCo"))
+                    runcase.lines.insert(il, new_line % ("lib"))
+                    runcase.lines.insert(il, v25_3_line)
+                    runcase.run_cmd_line_id += 3
+                    break
 
-        make_executable(batch_file)
+        runcase.save()
 
 
     def dump(self):
@@ -644,6 +849,10 @@ domains = [
                 print("  " + c)
         if self.ast_case_name != None:
             print("Code_Aster instance:", self.ast_case_name)
+        if self.py_case_name != None:
+            print("Python script instances:")
+            for c in self.py_case_name:
+                print("  " + c)
         print()
 
 
