@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -37,16 +37,20 @@ import sys, logging
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from Base.BrowserForm import Ui_BrowserForm
-from Base.Toolbox import GuiParam, displaySelectedPage
-from Base.QtPage import to_qvariant, from_qvariant, to_text_string
+from code_saturne.Base.BrowserForm import Ui_BrowserForm
+from code_saturne.model.Common import GuiParam
+from code_saturne.Base.Toolbox import displaySelectedPage
+from code_saturne.Base.QtPage import to_qvariant, from_qvariant, to_text_string
+
+import resource_base_rc
 
 #-------------------------------------------------------------------------------
 # log config
@@ -56,6 +60,17 @@ logging.basicConfig()
 log = logging.getLogger("BrowserView")
 log.setLevel(GuiParam.DEBUG)
 
+
+#-------------------------------------------------------------------------------
+#
+#-------------------------------------------------------------------------------
+
+try:
+    _fromUtf8 = QString.fromUtf8
+except Exception:
+    def _fromUtf8(s):
+        return s
+
 #-------------------------------------------------------------------------------
 #
 #-------------------------------------------------------------------------------
@@ -152,17 +167,67 @@ class TreeModel(QAbstractItemModel):
         elif role == Qt.DecorationRole:
             # return icon for first column
             if column == 0:
+                page_name = item.itemData[0]
+
                 style = QWidget().style()
-                if item.itemType == "folder-new":
-                    icon = style.standardIcon(QStyle.SP_FileDialogNewFolder)
+
+                if page_name == 'Mesh':
+                    img_path = ":/icons/22x22/prepro-mode.png"
+                    icon = QIcon()
+                    icon.addPixmap(QPixmap(_fromUtf8(img_path)), QIcon.Normal, QIcon.Off)
+                elif page_name == 'Calculation features':
+                    img_path = ":/icons/22x22/calculation_features.png"
+                    icon = QIcon()
+                    icon.addPixmap(QPixmap(_fromUtf8(img_path)), QIcon.Normal, QIcon.Off)
+                elif page_name == 'Fluid properties':
+                    img_path = ":/icons/22x22/physical_properties.png"
+                    icon = QIcon()
+                    icon.addPixmap(QPixmap(_fromUtf8(img_path)), QIcon.Normal, QIcon.Off)
+                elif page_name == 'Volume zones':
+                    img_path = ":/icons/22x22/volume_zones.png"
+                    icon = QIcon()
+                    icon.addPixmap(QPixmap(_fromUtf8(img_path)), QIcon.Normal, QIcon.Off)
+                elif page_name == 'Boundary zones':
+                    img_path = ":/icons/22x22/boundary_conditions.png"
+                    icon = QIcon()
+                    icon.addPixmap(QPixmap(_fromUtf8(img_path)), QIcon.Normal, QIcon.Off)
+                elif page_name == 'Time settings':
+                    img_path = ":/icons/22x22/time_stepping.png"
+                    icon = QIcon()
+                    icon.addPixmap(QPixmap(_fromUtf8(img_path)), QIcon.Normal, QIcon.Off)
+                elif page_name == 'Numerical parameters':
+                    img_path = ":/icons/22x22/numerical_params.png"
+                    icon = QIcon()
+                    icon.addPixmap(QPixmap(_fromUtf8(img_path)), QIcon.Normal, QIcon.Off)
+                elif page_name == 'Postprocessing':
+                    img_path = ":/icons/22x22/postprocessing.png"
+                    icon = QIcon()
+                    icon.addPixmap(QPixmap(_fromUtf8(img_path)), QIcon.Normal, QIcon.Off)
+                elif page_name == 'Closure modeling':
+                    img_path = ":/icons/22x22/closure_modeling.png"
+                    icon = QIcon()
+                    icon.addPixmap(QPixmap(_fromUtf8(img_path)), QIcon.Normal, QIcon.Off)
+                elif page_name == 'Performance settings':
+                    img_path = ":/icons/22x22/run_parameters.png"
+                    icon = QIcon()
+                    icon.addPixmap(QPixmap(_fromUtf8(img_path)), QIcon.Normal, QIcon.Off)
+
+                elif item.itemType == "folder-new":
+                    icon = style.standardIcon(QStyle.SP_FileLinkIcon)
                 elif item.itemType == "folder-close":
-                    icon = style.standardIcon(QStyle.SP_DirClosedIcon)
+                    icon = style.standardIcon(QStyle.SP_FileIcon)
                 elif item.itemType == "folder-open":
-                    icon = style.standardIcon(QStyle.SP_DirOpenIcon)
+                    icon = style.standardIcon(QStyle.SP_FileIcon)
                 elif item.itemType == "file-open":
                     icon = style.standardIcon(QStyle.SP_FileIcon)
                 elif item.itemType == "file-new":
                     icon = style.standardIcon(QStyle.SP_FileLinkIcon)
+                if sys.platform.startswith("win"):
+                    if item.itemType == "file-open":
+                        icon = style.standardIcon(QStyle.SP_ToolBarHorizontalExtensionButton)
+                    elif item.itemType == "file-new":
+                        icon = style.standardIcon(QStyle.SP_ToolBarHorizontalExtensionButton)
+
                 return to_qvariant(icon)
 
         return to_qvariant()
@@ -283,7 +348,8 @@ class TreeModel(QAbstractItemModel):
                 raise ValueError("This flags is not implemented")
 
             if self.hasChildren(index):
-                result += self.match(self.index(0, index.column(), index), role, value, hits, flags)
+                result += self.match(self.index(0, index.column(), index),
+                                     role, value, hits, flags)
 
         return result
 
@@ -388,68 +454,72 @@ class BrowserView(QWidget, Ui_BrowserForm):
 
         # Popup menu
         self.treeView.setContextMenuPolicy(Qt.CustomContextMenu)
-        self.connect(self.treeView, SIGNAL("customContextMenuRequested(QPoint)"), self.displayPopup)
+        self.treeView.customContextMenuRequested[QPoint].connect(self.displayPopup)
 
         # Receive change in selection
-        self.connect(self.treeView, SIGNAL('pressed(const QModelIndex &)'), self.onItemPressed)
-        self.connect(self.treeView, SIGNAL('expanded(const QModelIndex &)'), self.onFolderOpen)
-        self.connect(self.treeView, SIGNAL('collapsed(const QModelIndex &)'), self.onFolderClose)
+        self.treeView.pressed[QModelIndex].connect(self.onItemPressed)
+        self.treeView.expanded[QModelIndex].connect(self.onFolderOpen)
+        self.treeView.collapsed[QModelIndex].connect(self.onFolderClose)
 
 
     def _browser(self):
         tree ="""
-    Identity and paths
 Calculation environment
-    Meshes selection
-    Mesh quality criteria
-Thermophysical models
-    Calculation features
+    Notebook
+Mesh
+    Preprocessing
+Calculation features
+    Main fields
     Deformable mesh
     Turbulence models
     Thermal model
+    Body forces
     Gas combustion
     Pulverized fuel combustion
     Electrical models
-    Radiative transfers
     Conjugate heat transfer
     Atmospheric flows
     Species transport
-Physical properties
-    Fluid properties
-    Reference values
-    Gravity
-Volume conditions
-    Volume regions definition
+    Turbomachinery
+    Groundwater flows
+    Fans
+    Non condensable gases
+    Thermodynamics
+Closure modeling
+    Interfacial area
+    Interfacial enthalpy transfer
+    Nucleate boiling parameters
+    Droplet condensation-evaporation
+    Particles interactions
+Fluid properties
+Particles and droplets tracking
+    Statistics
+Volume zones
+    Main fields initialization
     Initialization
     Head losses
     Porosity
     Source terms
-    Coriolis Source Terms
-Particles and droplets tracking
-    Global settings
-    Statistics
-Boundary conditions
-    Definition of boundary regions
+    Groundwater laws
+Boundary zones
     Boundary conditions
-    Particles boundary conditions
+    Particle boundary conditions
     Fluid structure interaction
+    Cathare Coupling
+Time settings
+    Start/Restart
 Numerical parameters
-    Global parameters
     Equation parameters
-    Time step
-    Pseudo-Time step
-    Steady flow management
-Calculation control
+Postprocessing
+    Additional user arrays
     Time averages
-    Output control
     Volume solution control
     Surface solution control
     Lagrangian solution control
     Profiles
-Calculation management
-    Start/Restart
-    Performance tuning
-    Prepare batch calculation
+    Balance by zone
+Performance settings
+OpenTurns study
 """
         return tree
 
@@ -474,6 +544,17 @@ Calculation management
             self.treeView.setRowHidden(row, parent, False)
 
 
+    def setRowShow(self, string, status=True):
+        log.debug("setRowVisible(): %s" % string)
+        itemInfoList = self.model.itemLocalization(string)
+        hidden = not status
+        for itemInfo in itemInfoList:
+            row    = itemInfo[0]
+            column = itemInfo[1]
+            parent = itemInfo[2]
+            self.treeView.setRowHidden(row, parent, hidden)
+
+
     def isRowClose(self, string):
         log.debug("isRowClose(): %s" % string)
         itemInfoList = self.model.itemLocalization(string)
@@ -486,14 +567,14 @@ Calculation management
             return self.treeView.isRowHidden(row, index)
 
 
-    @pyqtSignature('const QModelIndex &')
+    @pyqtSlot('QModelIndex')
     def onItemPressed(self, index):
         item = index.internalPointer()
         if item.itemType == "file-new":
             item.itemType = "file-open"
 
 
-    @pyqtSignature('const QModelIndex &')
+    @pyqtSlot('QModelIndex')
     def onFolderOpen(self, index):
         """
         public slot
@@ -508,7 +589,7 @@ Calculation management
             item.itemType = "folder-open"
 
 
-    @pyqtSignature('const QModelIndex &')
+    @pyqtSlot('QModelIndex')
     def onFolderClose(self, index):
         """
         public slot
@@ -523,7 +604,7 @@ Calculation management
             item.itemType = "folder-close"
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def displayPopup(self):
         """
         public slot
@@ -534,11 +615,11 @@ Calculation management
 
         self.actionExpand = QAction(self.tr("Expand"), self.treeView)
         #self.actionExpand.setShortcut(self.tr("F5"))
-        self.connect(self.actionExpand, SIGNAL("triggered()"), self.openTreeFolder)
+        self.actionExpand.triggered.connect(self.openTreeFolder)
 
         self.actionCollapse = QAction(self.tr("Collapse"), self.treeView)
         #self.actionCollapse.setShortcut(self.tr("F6"))
-        self.connect(self.actionCollapse, SIGNAL("triggered()"), self.closeTreeFolder)
+        self.actionCollapse.triggered.connect(self.closeTreeFolder)
 
         # ... TODO
         #self.actionWelcome = QAction(self.tr("Welcome page"), self.treeView)
@@ -554,12 +635,14 @@ Calculation management
     def activeSelectedPage(self, index):
         """
         """
-        self.treeView.selectionModel().select(index, QItemSelectionModel.SelectCurrent)
+        if index != None:
+            self.treeView.selectionModel().select(index,
+                                                  QItemSelectionModel.SelectCurrent)
 
         return
 
 
-    def display(self, root, case, stbar, study, tree):
+    def display(self, root, case, stbar, tree):
         """
         """
         index = self.treeView.currentIndex()
@@ -567,7 +650,7 @@ Calculation management
         name  = item.itemData[0]
         case['current_tab'] = 0
         case['current_index'] = index
-        return displaySelectedPage(name, root, case, stbar, study, tree)
+        return displaySelectedPage(name, root, case, stbar, tree)
 
 
     def isFolder(self):
@@ -592,7 +675,7 @@ Calculation management
             self.treeView.expand(index)
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def openTreeFolder(self):
         """
         public slot
@@ -624,7 +707,7 @@ Calculation management
             self.treeView.collapse(index)
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def closeTreeFolder(self):
         """
         public slot
@@ -640,173 +723,360 @@ Calculation management
             self.onFolderClose(index)
 
 
+    def __configureTreePrepro(self, case):
+        """
+        Public method.
+        Configures the browser with users data.
+        """
+
+        self.setRowClose(self.tr('Notebook'))
+
+        show_prepro = True
+        if case['run_type'] == 'none':
+            show_prepro = False
+
+        self.setRowShow(self.tr('Preprocessing'), show_prepro)
+        self.setRowShow(self.tr('Postprocessing'), show_prepro)
+
+        self.setRowClose(self.tr('Calculation features'))
+        """
+        self.setRowClose(self.tr('Main fields'))
+        self.setRowClose(self.tr('Deformable mesh'))
+        self.setRowClose(self.tr('Turbulence models'))
+        self.setRowClose(self.tr('Thermal model'))
+        self.setRowClose(self.tr('Gas combustion'))
+        self.setRowClose(self.tr('Pulverized fuel combustion'))
+        self.setRowClose(self.tr('Electrical models'))
+        self.setRowClose(self.tr('Conjugate heat transfer'))
+        self.setRowClose(self.tr('Atmospheric flows'))
+        self.setRowClose(self.tr('Species transport'))
+        self.setRowClose(self.tr('Turbomachinery'))
+        self.setRowClose(self.tr('Groundwater flows'))
+        self.setRowClose(self.tr('Fans'))
+        self.setRowClose(self.tr('Non condensable gases'))
+        self.setRowClose(self.tr('Thermodynamics'))
+        """
+
+        self.setRowClose(self.tr('Closure modeling'))
+        """
+        self.setRowClose(self.tr('Interfacial area'))
+        self.setRowClose(self.tr('Interfacial enthalpy transfer'))
+        self.setRowClose(self.tr('Nucleate boiling parameters'))
+        self.setRowClose(self.tr('Droplet condensation-evaporation'))
+        self.setRowClose(self.tr('Particles interactions'))
+        """
+
+        self.setRowClose(self.tr('Fluid properties'))
+        """
+        self.setRowClose(self.tr('Fluid properties'))
+        self.setRowClose(self.tr('Body forces'))
+        """
+
+        self.setRowClose(self.tr('Particles and droplets tracking'))
+        """
+        self.setRowClose(self.tr('Statistics'))
+        """
+
+        self.setRowShow(self.tr('Volume zones'), False)
+        """
+        self.setRowClose(self.tr('Main fields initialization'))
+        self.setRowClose(self.tr('Initialization'))
+        self.setRowClose(self.tr('Head losses'))
+        self.setRowClose(self.tr('Porosity'))
+        self.setRowClose(self.tr('Source terms'))
+        self.setRowClose(self.tr('Groundwater laws'))
+        """
+
+        self.setRowShow(self.tr('Boundary zones'), False)
+
+        """
+        self.setRowClose(self.tr('Boundary_conditions'))
+        self.setRowClose(self.tr('Particle boundary conditions'))
+        self.setRowClose(self.tr('Fluid structure interaction'))
+        self.setRowClose(self.tr('Cathare Coupling'))
+        """
+
+        self.setRowClose(self.tr('Time settings'))
+
+        self.setRowClose(self.tr('Numerical parameters'))
+        """
+        self.setRowClose(self.tr('Equation parameters'))
+        """
+
+        self.setRowClose(self.tr('Time averages'))
+        self.setRowClose(self.tr('Additional user arrays'))
+        self.setRowClose(self.tr('Volume solution control'))
+        self.setRowClose(self.tr('Surface solution control'))
+        self.setRowClose(self.tr('Lagrangian solution control'))
+        self.setRowClose(self.tr('Profiles'))
+        self.setRowClose(self.tr('Balance by zone'))
+
+        self.setRowClose(self.tr('OpenTurns study'))
+
+        self.__hideRow()
+
+
     def configureTree(self, case):
         """
         Public method.
         Configures the browser with users data.
         """
-        self.setRowClose(self.tr('Particles and droplets tracking'))
-        self.setRowClose(self.tr('Gas combustion'))
-        self.setRowClose(self.tr('Pulverized fuel combustion'))
-        self.setRowClose(self.tr('Electrical models'))
-        self.setRowClose(self.tr('Radiative transfers'))
-        self.setRowClose(self.tr('Conjugate heat transfer'))
-        self.setRowClose(self.tr('Atmospheric flows'))
-        self.setRowClose(self.tr('Radiative boundary conditions'))
-        self.setRowClose(self.tr('Particles boundary conditions'))
-        self.setRowClose(self.tr('Steady flow management'))
-        # self.setRowClose(self.tr('Surface solution control'))
-        self.setRowClose(self.tr('Time step'))
-        self.setRowClose(self.tr('Pseudo-Time step'))
-        self.setRowClose(self.tr('Fluid structure interaction'))
-        self.setRowClose(self.tr('Source terms'))
-        self.setRowClose(self.tr('Head losses'))
-        self.setRowClose(self.tr('Porosity'))
-        self.setRowClose(self.tr('Lagrangian solution control'))
 
-        # Steady flow management
+        if case['run_type'] != 'standard':
+            return self.__configureTreePrepro(case)
 
-        nodeanal = case.xmlGetNode('analysis_control')
-        nodeSteady = nodeanal.xmlGetNode('steady_management')
-        node_np = case.xmlGetNode('numerical_parameters')
-        nodeCouplingAlgo = node_np.xmlGetNode('velocity_pressure_algo')
+        p_module = ''
+        if case.xmlRootNode().tagName == "NEPTUNE_CFD_GUI":
+            p_module = 'neptune_cfd'
 
-        if nodeSteady['status'] == 'on':
-            if nodeCouplingAlgo:
-                value = nodeCouplingAlgo['choice']
-                if value == 'simple':
-                    self.setRowClose(self.tr('Time step'))
-                    self.setRowClose(self.tr('Pseudo-Time step'))
-                    self.setRowOpen(self.tr('Steady flow management'))
-                else:
-                    self.setRowClose(self.tr('Time step'))
-                    self.setRowOpen(self.tr('Pseudo-Time step'))
-                    self.setRowClose(self.tr('Steady flow management'))
-            else:
-                self.setRowClose(self.tr('Time step'))
-                self.setRowOpen(self.tr('Pseudo-Time step'))
-                self.setRowClose(self.tr('Steady flow management'))
-        else:
-            nodeSteady['status'] = 'off'
-            self.setRowClose(self.tr('Steady flow management'))
-            self.setRowOpen(self.tr('Time step'))
-            self.setRowClose(self.tr('Pseudo-Time step'))
+        # Precompute some values
+        #-----------------------
 
-        # Multi-phase flow
+        m_tbm = False
+        m_fans = False
+        m_lagr = False
+        m_ale = False
+        m_thermal = 0
+        m_cht = False
+        m_gas_comb = False
+        m_sf_comb = False
+        m_elec = False
+        m_atmo = False
+        m_comp = False
+        m_gwf = False
 
-        nodeLagr = case.xmlGetNode('lagrangian', 'model')
+        node_pm = case.xmlGetNode('thermophysical_models')
 
-        if nodeLagr and nodeLagr['model'] == "on":
-            self.setRowOpen(self.tr('Particles and droplets tracking'))
-            self.setRowOpen(self.tr('Lagrangian solution control'))
-            self.setRowOpen(self.tr('Particles boundary conditions'))
-            self.setRowClose(self.tr('Deformable mesh'))
-        else:
-            self.setRowClose(self.tr('Particles and droplets tracking'))
-            self.setRowClose(self.tr('Lagrangian solution control'))
-            self.setRowClose(self.tr('Particles boundary conditions'))
-            self.setRowOpen(self.tr('Deformable mesh'))
+        if node_pm:
 
-        # OutputSurfacicView
+            node = node_pm.xmlGetNode('ale_method')
+            if node and node['status'] == 'on':
+                m_ale = True
+            node = node_pm.xmlGetNode('turbomachinery', 'model')
+            if node and node['model'] != "off":
+                m_tbm = True
+            node = node_pm.xmlGetNode('fans')
+            if node:
+                m_fans = True
 
-        node_control = case.xmlGetNode('analysis_control')
-        node_out     = node_control.xmlInitNode('output')
-        node_bound   = node_out.xmlGetNode('domain_boundary', 'status')
+            if not m_thermal:
+                node = node_pm.xmlGetNode('gas_combustion', 'model')
+                if node and node['model'] in ('ebu', 'd3p', 'lwp'):
+                    m_gas_comb = True
+                    m_thermal = 1
+            if not m_thermal:
+                node = node_pm.xmlGetNode('solid_fuels', 'model')
+                if node and node['model'] in ('homogeneous_fuel',
+                                              'homogeneous_fuel_moisture',
+                                              'homogeneous_fuel_moisture_lagr'):
+                    m_sf_comb = True
+                    m_thermal = 1
+            if not m_thermal:
+                node = node_pm.xmlGetNode('joule_effect', 'model')
+                if node and node['model'] in ('joule', 'arc'):
+                    m_elec = True
+                    m_thermal = 1
+            if not m_thermal:
+                node = node_pm.xmlGetNode('atmospheric_flows',  'model')
+                if node and node['model'] != 'off':
+                    m_atmo = True
+                    if node['model'] == 'constant':
+                        m_thermal = -1
+                    elif node['model'] in ('dry', 'humid'):
+                        m_thermal = 1
+            if not m_thermal:
+                node = node_pm.xmlGetNode('compressible_model', 'model')
+                if node and node['model'] != 'off':
+                    m_cpr = True
+                    m_thermal = 1
 
-        # FIXME: node_bound = ''
-        if node_bound and node_bound['status'] == 'on':
-            self.setRowOpen(self.tr('Surface solution control'))
+            if not m_thermal:
+                node = node_pm.xmlGetNode('thermal_scalar', 'model')
+                if node and node['model'] != 'off':
+                    m_thermal = 2
 
-        # Reactive flow
+            node = node_pm.xmlGetNode('groundwater_model',  'model')
+            if node and node['model'] != 'off':
+                m_gwf = True
+                m_thermal = -1
+                m_fans = False
 
-        node0 = case.xmlGetNode('thermophysical_models')
-        node1 = node0.xmlGetNode('gas_combustion',     'model')
-        node2 = node0.xmlGetNode('solid_fuels',        'model')
-        node3 = node0.xmlGetNode('joule_effect',       'model')
-        node4 = node0.xmlGetNode('thermal_scalar',     'model')
-        node5 = node0.xmlGetNode('radiative_transfer', 'model')
-        node6 = node0.xmlGetNode('atmospheric_flows',  'model')
-        node7 = node0.xmlGetNode('compressible_model', 'model')
+            if m_thermal > 0:
+                m_cht = True
 
-        if node1['model'] in ('ebu', 'd3p', 'lwp'):
-            self.setRowOpen(self.tr('Thermal model'))
-            self.setRowOpen(self.tr('Gas combustion'))
-            self.setRowOpen(self.tr('Radiative transfers'))
-            self.setRowOpen(self.tr('Conjugate heat transfer'))
-            if node5.xmlGetAttribute('model') != 'off':
-                self.setRowOpen(self.tr('Radiative boundary conditions'))
+        node = case.xmlGetNode('lagrangian', 'model')
+        if node and node['model'] != "off":
+            m_lagr = True
 
-        elif node2['model'] in ('homogeneous_fuel', 'homogeneous_fuel_moisture',
-                                'homogeneous_fuel_moisture_lagr'):
-            self.setRowOpen(self.tr('Thermal model'))
-            self.setRowOpen(self.tr('Pulverized fuel combustion'))
-            self.setRowOpen(self.tr('Radiative transfers'))
-            self.setRowOpen(self.tr('Conjugate heat transfer'))
-            if node5.xmlGetAttribute('model') != 'off':
-                self.setRowOpen(self.tr('Radiative boundary conditions'))
+        # Options for NEPTUNE_CFD if present:
 
-        elif node3['model'] in ('joule', 'arc'):
-            self.setRowOpen(self.tr('Thermal model'))
-            self.setRowOpen(self.tr('Electrical models'))
-            self.setRowOpen(self.tr('Radiative transfers'))
-            self.setRowOpen(self.tr('Conjugate heat transfer'))
-            if node5.xmlGetAttribute('model') != 'off':
-                self.setRowOpen(self.tr('Radiative boundary conditions'))
+        m_ncfd = {}
+        m_ncfd['non_condens'] = False
+        m_ncfd['nucleate_boiling'] = False
+        m_ncfd['droplet_condens'] = False
+        m_ncfd['particles_interactions'] = False
+        m_ncfd['itf_area'] = False
+        m_ncfd['itf_h_transfer'] = False
 
-        elif node6['model'] and node6['model'] != 'off':
-            self.setRowOpen(self.tr('Atmospheric flows'))
-            self.setRowOpen(self.tr('Radiative transfers'))
-            self.setRowOpen(self.tr('Conjugate heat transfer'))
-            if node5.xmlGetAttribute('model') != 'off':
-                self.setRowOpen(self.tr('Radiative boundary conditions'))
+        ncfd_fields = 0
 
-            if node6 and node6['model'] in ('dry', 'humid'):
-                self.setRowOpen(self.tr('Thermal model'))
-            else:
-                self.setRowClose(self.tr('Thermal model'))
+        if p_module == 'neptune_cfd' and node_pm:
 
-        elif node7['model'] != 'off':
-            self.setRowClose(self.tr('Thermal model'))
+            m_fans = False
+            m_thermal = 1
+            m_cht = True
 
-        else:
-            self.setRowOpen(self.tr('Thermal model'))
-            if node4.xmlGetAttribute('model') != 'off':
-                self.setRowOpen(self.tr('Radiative transfers'))
-                self.setRowOpen(self.tr('Conjugate heat transfer'))
-                if node5.xmlGetAttribute('model') != 'off':
-                    self.setRowOpen(self.tr('Radiative boundary conditions'))
+            node_f = node_pm.xmlInitNode('fields')
+            fields = node_f.xmlGetNodeList('field')
+            ncfd_fields = len(fields)
 
-        node7 = node0.xmlGetNode('ale_method', 'status')
-        if node7 and node7['status'] == 'on':
-            self.setRowOpen(self.tr('Fluid structure interaction'))
+        if ncfd_fields > 1:
+            from code_saturne.model.MainFieldsModel import MainFieldsModel
+            from code_saturne.model.NonCondensableModel import NonCondensableModel
+            from code_saturne.model.InterfacialForcesModel import InterfacialForcesModel
+            predefined_flow = MainFieldsModel(case).getPredefinedFlow()
+
+            if (len(MainFieldsModel(case).getSolidFieldIdList()) > 0):
+                m_ncfd['particles_interactions'] = True
+            if (len(MainFieldsModel(case).getDispersedFieldList()) > 0
+                or InterfacialForcesModel(case).getBubblesForLIMStatus() == 'on'):
+                m_ncfd['itf_area'] = True
+
+            if predefined_flow == "free_surface":
+                m_ncfd['non_condens'] = True
+                m_ncfd['nucleate_boiling'] = True
+                m_ncfd['itf_h_transfer'] = True
+            elif predefined_flow == "boiling_flow":
+                m_ncfd['non_condens'] = True
+                m_ncfd['nucleate_boiling'] = True
+                m_ncfd['itf_h_transfer'] = True
+            elif predefined_flow == "droplet_flow":
+                m_ncfd['non_condens'] = True
+                m_ncfd['droplet_condens'] = True
+                m_ncfd['itf_h_transfer'] = True
+            elif predefined_flow == "particles_flow":
+                m_ncfd['particles_interactions'] = True
+                m_ncfd['itf_h_transfer'] = True
+
+        is_ncfd = (p_module == 'neptune_cfd')
+
+        # Manage visibility
+        #------------------
+
+        self.setRowOpen(self.tr('Notebook'))
+
+        # Preprocessing
+
+        self.setRowShow(self.tr('Preprocessing'), True)
+
+        # Thermophysical Models
+
+        self.setRowShow(self.tr('Calculation features'), True)
+        self.setRowShow(self.tr('Main fields'), (p_module == 'neptune_cfd'))
+        self.setRowShow(self.tr('Deformable mesh'), m_ale)
+        self.setRowShow(self.tr('Turbulence models'))
+        self.setRowShow(self.tr('Thermal model'), (m_thermal > -1))
+        self.setRowShow(self.tr('Body forces'), (not m_gwf))
+        self.setRowShow(self.tr('Gas combustion'), m_gas_comb)
+        self.setRowShow(self.tr('Pulverized fuel combustion'), m_sf_comb)
+        self.setRowShow(self.tr('Electrical models'), m_elec)
+        self.setRowShow(self.tr('Conjugate heat transfer'), m_cht)
+        self.setRowShow(self.tr('Atmospheric flows'), m_atmo)
+        self.setRowShow(self.tr('Species transport'))
+        self.setRowShow(self.tr('Turbomachinery'), m_tbm)
+        self.setRowShow(self.tr('Groundwater flows'), m_gwf)
+        self.setRowShow(self.tr('Fans'), m_fans)
+
+        self.setRowShow(self.tr('Non condensable gases'), m_ncfd['non_condens'])
+        self.setRowShow(self.tr('Thermodynamics'), is_ncfd)
+
+        # Closure modeling
+
+        self.setRowShow(self.tr('Closure modeling'), (ncfd_fields > 1))
+        self.setRowShow(self.tr('Interfacial enthalpy transfer'), m_ncfd['itf_h_transfer'])
+        self.setRowShow(self.tr('Interfacial area'), m_ncfd['itf_area'])
+        self.setRowShow(self.tr('Nucleate boiling parameters'), m_ncfd['nucleate_boiling'])
+        self.setRowShow(self.tr('Droplet condensation-evaporation'), m_ncfd['droplet_condens'])
+        self.setRowShow(self.tr('Particles interactions'), m_ncfd['particles_interactions'])
+
+        # Fluid properties
+
+        self.setRowShow(self.tr('Fluid properties'), (not (m_gwf or is_ncfd)))
+
+        # Particles and droplets tracking
+
+        self.setRowShow(self.tr('Particles and droplets tracking'), m_lagr)
+        self.setRowShow(self.tr('Statistics'), m_lagr)
+
+        # Volume zones
+
+        self.setRowShow(self.tr('Volume zones'), True)
 
-        # Source terms view
         node_domain = case.xmlGetNode('solution_domain')
         node_vol = node_domain.xmlGetNode('volumic_conditions')
-        nb_zone = 0
-        nb_zone_losses = 0
-        nb_zone_porosity = 0
+        init = False
+        z_st = False
+        z_head_loss = False
+        z_porosity = False
+        z_groundwater = False
 
         for node in node_vol.xmlGetChildNodeList('zone'):
-            if node['momentum_source_term'] == 'on':
-                nb_zone = nb_zone + 1
-            elif node['mass_source_term'] == 'on':
-                nb_zone = nb_zone + 1
-            elif node['thermal_source_term'] == 'on':
-                nb_zone = nb_zone + 1
-            elif node['scalar_source_term'] == 'on':
-                nb_zone = nb_zone + 1
+            if (node['initialization'] == 'on'):
+                init = True
+            if (node['momentum_source_term'] == 'on'
+                or node['mass_source_term'] == 'on'
+                or node['thermal_source_term'] == 'on'
+                or node['scalar_source_term'] == 'on'):
+                z_st = True
             if node['head_losses'] == 'on':
-                nb_zone_losses = nb_zone_losses + 1
+                z_head_loss = True
             if node['porosity'] == 'on':
-                nb_zone_porosity = nb_zone_porosity + 1
+                z_porosity = True
+            if node['groundwater_law'] == 'on':
+                z_groundwater = True
 
-        if nb_zone > 0:
-            self.setRowOpen(self.tr('Source terms'))
-        if nb_zone_losses > 0:
-            self.setRowOpen(self.tr('Head losses'))
-        if nb_zone_porosity > 0:
-            self.setRowOpen(self.tr('Porosity'))
+        self.setRowShow(self.tr('Main fields initialization'), is_ncfd and init)
+        self.setRowShow(self.tr('Initialization'), (not is_ncfd) and init)
+        self.setRowShow(self.tr('Head losses'), z_head_loss)
+        self.setRowShow(self.tr('Porosity'), z_porosity)
+        self.setRowShow(self.tr('Source terms'), z_st)
+        self.setRowShow(self.tr('Groundwater laws'), z_groundwater)
+
+        # Boundary zones
+
+        self.setRowShow(self.tr('Boundary zones'))
+        self.setRowShow(self.tr('Boundary conditions'))
+        self.setRowShow(self.tr('Particle boundary conditions'), m_lagr)
+        self.setRowShow(self.tr('Fluid structure interaction'), m_ale)
+        self.setRowShow(self.tr('Cathare Coupling'), is_ncfd)
+
+        # Time settings
+
+        self.setRowShow(self.tr('Time settings'))
+
+        # Numerical parameters
+
+        self.setRowShow(self.tr('Numerical parameters'))
+        self.setRowShow(self.tr('Equation parameters'))
+
+        # Postprocessing
+
+        self.setRowShow(self.tr('Postprocessing'), True)
+        self.setRowShow(self.tr('Time averages'), True)
+        self.setRowShow(self.tr('Additional user arrays'))
+        self.setRowShow(self.tr('Volume solution control'), True)
+        self.setRowShow(self.tr('Surface solution control'), True)
+        self.setRowShow(self.tr('Lagrangian solution control'), m_lagr)
+        self.setRowShow(self.tr('Profiles'), True)
+        self.setRowShow(self.tr('Balance by zone'), (not is_ncfd))
+
+        # Calculation management
+
+        self.setRowShow(self.tr('Performance tuning'), True)
+        self.setRowShow(self.tr('Prepare batch calculation'), True)
+        self.setRowShow(self.tr('OpenTurns study'), case['salome'])
+
+        # End of test of physical module
 
         self.__hideRow()
 
