@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -39,20 +39,20 @@ import logging
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
-
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from Base.Toolbox import GuiParam
-from Base.QtPage import ComboModel, IntValidator, DoubleValidator
-from Base.QtPage import from_qvariant, to_text_string
-from Pages.LagrangianStatisticsForm import Ui_LagrangianStatisticsForm
-from Pages.LagrangianStatisticsModel import LagrangianStatisticsModel
-from Pages.LagrangianModel import LagrangianModel
+from code_saturne.model.Common import GuiParam
+from code_saturne.Base.QtPage import ComboModel, IntValidator, DoubleValidator
+from code_saturne.Base.QtPage import from_qvariant, to_text_string
+from code_saturne.Pages.LagrangianStatisticsForm import Ui_LagrangianStatisticsForm
+from code_saturne.model.LagrangianStatisticsModel import LagrangianStatisticsModel
+from code_saturne.model.LagrangianModel import LagrangianModel
 
 
 #-------------------------------------------------------------------------------
@@ -105,18 +105,13 @@ class StandardItemModelVolumicNames(QStandardItemModel):
 
     def data(self, index, role):
 
-        self.kwords = [ "IACTFV", "IACTVX", "IACTVY", "IACTVZ", "IACTTS"]
         if not index.isValid():
             return
 
         # ToolTips
         if role == Qt.ToolTipRole:
-            if index.column() == 0:
-                return self.tr("Code_Saturne key word: NOMLAG")
-            if index.column() == 1:
-                return self.tr("Code_Saturne key word: NOMLAV")
-            elif index.column() in [2,3]:
-                return self.tr("Code_Saturne key word: " + self.kwords[index.row()])
+            if index.column() in [0, 1]:
+                return self.tr("field label base")
 
         # Display
         if role == Qt.DisplayRole:
@@ -173,7 +168,7 @@ class StandardItemModelVolumicNames(QStandardItemModel):
             vname = self.dataVolumicNames[index.row()][0]
             self.model.setPostprocessingVolStatusFromName(vname, status)
 
-        self.emit(SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), index, index)
+        self.dataChanged.emit(index, index)
         return True
 
 
@@ -210,7 +205,7 @@ class StandardItemModelBoundariesNames(QStandardItemModel):
 
     def data(self, index, role):
 
-        self.kwords = [ "INBRBD",   "IFLMBD",   "IANGBD",   "IVITBD",
+        self.kwords = [ "IFLMBD",   "INBRBD",   "IANGBD",   "IVITBD",
                         "IENCNBBD", "IENCMABD", "IENCDIBD", "IENCCKBD"]
         if not index.isValid():
             return
@@ -268,7 +263,7 @@ class StandardItemModelBoundariesNames(QStandardItemModel):
             self.model.setPostprocessingStatusFromName(vname, status)
 
 
-        self.emit(SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), index, index)
+        self.dataChanged.emit(index, index)
         return True
 
 
@@ -294,19 +289,16 @@ class LagrangianStatisticsView(QWidget, Ui_LagrangianStatisticsForm):
         self.case.undoStopGlobal()
         self.model = LagrangianStatisticsModel(self.case)
 
-        self.connect(self.checkBoxISUIST, SIGNAL("clicked()"), self.slotISUIST)
-        self.connect(self.lineEditNBCLST, SIGNAL("textChanged(const QString &)"), self.slotNBCLST)
+        self.checkBoxISUIST.clicked.connect(self.slotISUIST)
+        self.lineEditNBCLST.textChanged[str].connect(self.slotNBCLST)
 
-        self.connect(self.groupBoxISTALA, SIGNAL("clicked()"), self.slotISTALA)
-        self.connect(self.lineEditIDSTNT, SIGNAL("editingFinished()"), self.slotIDSTNT)
-        self.connect(self.lineEditNSTIST, SIGNAL("editingFinished()"), self.slotNSTIST)
+        self.groupBoxISTALA.clicked.connect(self.slotISTALA)
+        self.lineEditIDSTNT.editingFinished.connect(self.slotIDSTNT)
+        self.lineEditNSTIST.editingFinished.connect(self.slotNSTIST)
 
-        self.connect(self.lineEditSEUIL,  SIGNAL("textChanged(const QString &)"), self.slotSEUIL)
-
-        self.connect(self.groupBoxIENSI3, SIGNAL("clicked()"), self.slotIENSI3)
-        self.connect(self.lineEditNSTBOR, SIGNAL("textChanged(const QString &)"), self.slotNSTBOR)
-        self.connect(self.lineEditSEUILF, SIGNAL("textChanged(const QString &)"), self.slotSEUILF)
+        self.lineEditSEUIL.textChanged[str].connect(self.slotSEUIL)
 
+        self.groupBoxIENSI3.clicked.connect(self.slotIENSI3)
 
         validatorNBCLST = IntValidator(self.lineEditNBCLST, min=0) # max=100
 
@@ -314,16 +306,11 @@ class LagrangianStatisticsView(QWidget, Ui_LagrangianStatisticsForm):
         validatorIDSTNT.setExclusiveMin(True)
         validatorNSTIST = IntValidator(self.lineEditNSTIST, min=0)
         validatorSEUIL = DoubleValidator(self.lineEditSEUIL, min=0.)
-        validatorNSTBOR = IntValidator(self.lineEditNSTBOR, min=0)
-        validatorNSTBOR.setExclusiveMin(True)
-        validatorSEUILF = DoubleValidator(self.lineEditSEUILF, min=0.)
 
         self.lineEditNBCLST.setValidator(validatorNBCLST)
         self.lineEditIDSTNT.setValidator(validatorIDSTNT)
         self.lineEditNSTIST.setValidator(validatorNSTIST)
         self.lineEditSEUIL.setValidator(validatorSEUIL)
-        self.lineEditNSTBOR.setValidator(validatorNSTBOR)
-        self.lineEditSEUILF.setValidator(validatorSEUILF)
 
         # initialize Widgets
         # FIXME
@@ -342,6 +329,15 @@ class LagrangianStatisticsView(QWidget, Ui_LagrangianStatisticsForm):
         nclust = self.model.getGroupOfParticlesValue()
         self.lineEditNBCLST.setText(str(nclust))
 
+        it = self.model.getIterationStart()
+        self.lineEditIDSTNT.setText(str(it))
+
+        it = self.model.getIterSteadyStart()
+        self.lineEditNSTIST.setText(str(it))
+
+        seuil = self.model.getThresholdValue()
+        self.lineEditSEUIL.setText(str(seuil))
+
         # volume
         status = self.model.getVolumeStatisticsStatus()
         if status == "on":
@@ -372,7 +368,10 @@ class LagrangianStatisticsView(QWidget, Ui_LagrangianStatisticsForm):
         self.tableViewVolumicNames.setSelectionBehavior(QAbstractItemView.SelectItems)
         self.tableViewVolumicNames.setSelectionMode(QAbstractItemView.ExtendedSelection)
         self.tableViewVolumicNames.setEditTriggers(QAbstractItemView.DoubleClicked)
-        self.tableViewVolumicNames.horizontalHeader().setResizeMode(QHeaderView.Stretch)
+        if QT_API == "PYQT4":
+            self.tableViewVolumicNames.horizontalHeader().setResizeMode(QHeaderView.Stretch)
+        elif QT_API == "PYQT5":
+            self.tableViewVolumicNames.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
 
 
     def _initBoundariesNames(self):
@@ -386,10 +385,13 @@ class LagrangianStatisticsView(QWidget, Ui_LagrangianStatisticsForm):
         self.tableViewBoundariesNames.setSelectionBehavior(QAbstractItemView.SelectItems)
         self.tableViewBoundariesNames.setSelectionMode(QAbstractItemView.ExtendedSelection)
         self.tableViewBoundariesNames.setEditTriggers(QAbstractItemView.DoubleClicked)
-        self.tableViewBoundariesNames.horizontalHeader().setResizeMode(QHeaderView.Stretch)
+        if QT_API == "PYQT4":
+            self.tableViewBoundariesNames.horizontalHeader().setResizeMode(QHeaderView.Stretch)
+        elif QT_API == "PYQT5":
+            self.tableViewBoundariesNames.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotISUIST(self):
         """
         Input ISUIST.
@@ -401,17 +403,16 @@ class LagrangianStatisticsView(QWidget, Ui_LagrangianStatisticsForm):
         self.model.setRestartStatisticsStatus(status)
 
 
-    @pyqtSignature("const QString")
+    @pyqtSlot(str)
     def slotNBCLST(self, text):
         """
         Input NBCLST.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditNBCLST.validator().state == QValidator.Acceptable:
             value = from_qvariant(text, int)
             self.model.setGroupOfParticlesValue(value)
 
-
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotISTALA(self):
         """
         Input ISTALA.
@@ -421,15 +422,6 @@ class LagrangianStatisticsView(QWidget, Ui_LagrangianStatisticsForm):
             self.model.setVolumeStatisticsStatus("on")
             self._initVolumicNames()
 
-            it = self.model.getIterationStartVolume()
-            self.lineEditIDSTNT.setText(str(it))
-
-            it = self.model.getIterSteadyStartVolume()
-            self.lineEditNSTIST.setText(str(it))
-
-            seuil = self.model.getThresholdValueVolume()
-            self.lineEditSEUIL.setText(str(seuil))
-
         else:
 
             self.model.setVolumeStatisticsStatus("off")
@@ -437,57 +429,57 @@ class LagrangianStatisticsView(QWidget, Ui_LagrangianStatisticsForm):
                 del self.modelVolumicNames
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotIDSTNT(self):
         """
         Input IDSTNT.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditIDSTNT.validator().state == QValidator.Acceptable:
             text = self.lineEditIDSTNT.text()
             value = from_qvariant(text, int)
-            valnds =  self.model.getIterSteadyStartVolume()
+            valnds =  self.model.getIterSteadyStart()
 
             if value > valnds:
                 self.lineEditNSTIST.setText(str(value))
-                self.model.setIterSteadyStartVolume(value)
+                self.model.setIterSteadyStart(value)
             else:
                 valndsl = from_qvariant(self.lineEditNSTIST.text(), int)
-                self.model.setIterSteadyStartVolume(valndsl)
+                self.model.setIterSteadyStart(valndsl)
 
-            self.model.setIterationStartVolume(value)
+            self.model.setIterationStart(value)
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotNSTIST(self):
         """
         Input NSTIST.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditNSTIST.validator().state == QValidator.Acceptable:
             text = self.lineEditNSTIST.text()
             value = from_qvariant(text, int)
-            valids =  self.model.getIterationStartVolume()
+            valids =  self.model.getIterationStart()
 
             if value < valids:
                 self.lineEditIDSTNT.setText(str(value))
-                self.model.setIterationStartVolume(value)
+                self.model.setIterationStart(value)
             else:
                 validsl = from_qvariant(self.lineEditIDSTNT.text(), int)
-                self.model.setIterationStartVolume(validsl)
+                self.model.setIterationStart(validsl)
 
-            self.model.setIterSteadyStartVolume(value)
+            self.model.setIterSteadyStart(value)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotSEUIL(self, text):
         """
         Input SEUIL.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditSEUIL.validator().state == QValidator.Acceptable:
             value = from_qvariant(text, float)
-            self.model.setThresholdValueVolume(value)
+            self.model.setThresholdValue(value)
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotIENSI3(self):
         """
         Input IENSI3.
@@ -497,12 +489,6 @@ class LagrangianStatisticsView(QWidget, Ui_LagrangianStatisticsForm):
             self.model.setBoundaryStatisticsStatus("on")
             self._initBoundariesNames()
 
-            it = self.model.getIterationStartBoundary()
-            self.lineEditNSTBOR.setText(str(it))
-
-            seuil = self.model.getThresholdValueBoundary()
-            self.lineEditSEUILF.setText(str(seuil))
-
         else:
 
             self.model.setBoundaryStatisticsStatus("off")
@@ -510,26 +496,16 @@ class LagrangianStatisticsView(QWidget, Ui_LagrangianStatisticsForm):
                 del self.modelBoundariesNames
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotNSTBOR(self, text):
         """
         Input NSTBOR.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditNSTBOR.validator().state == QValidator.Acceptable:
             value = from_qvariant(text, int)
             self.model.setIterationStartBoundary(value)
 
 
-    @pyqtSignature("const QString&")
-    def slotSEUILF(self, text):
-        """
-        Input SEUILF.
-        """
-        if self.sender().validator().state == QValidator.Acceptable:
-            value = from_qvariant(text, float)
-            self.model.setThresholdValueBoundary(value)
-
-
     def tr(self, text):
         """
         Translation
