@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -37,18 +37,18 @@ import logging
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from Base.Toolbox import GuiParam
-from Base.QtPage import ComboModel, DoubleValidator, from_qvariant
-from Pages.NumericalParamGlobalForm import Ui_NumericalParamGlobalForm
-from Pages.NumericalParamGlobalModel import NumericalParamGlobalModel
-from Pages.SteadyManagementModel import SteadyManagementModel
+from code_saturne.model.Common import GuiParam
+from code_saturne.Base.QtPage import ComboModel, DoubleValidator, from_qvariant
+from code_saturne.Pages.NumericalParamGlobalForm import Ui_NumericalParamGlobalForm
+from code_saturne.model.NumericalParamGlobalModel import NumericalParamGlobalModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -82,12 +82,10 @@ class NumericalParamGlobalView(QWidget, Ui_NumericalParamGlobalForm):
 
         self.labelSRROM.hide()
         self.lineEditSRROM.hide()
-        self.line_5.hide()
 
         # Combo models
         self.modelEXTRAG = ComboModel(self.comboBoxEXTRAG,2,1)
-        self.modelIMRGRA = ComboModel(self.comboBoxIMRGRA,5,1)
-        self.modelNTERUP = ComboModel(self.comboBoxNTERUP,3,1)
+        self.modelIMRGRA = ComboModel(self.comboBoxIMRGRA,7,1)
 
         self.modelEXTRAG.addItem(self.tr("Neumann 1st order"), 'neumann')
         self.modelEXTRAG.addItem(self.tr("Extrapolation"), 'extrapolation')
@@ -97,31 +95,26 @@ class NumericalParamGlobalView(QWidget, Ui_NumericalParamGlobalForm):
         self.modelIMRGRA.addItem(self.tr("Least squares method over extended cell neighborhood"),'2')
         self.modelIMRGRA.addItem(self.tr("Least squares method over partial extended cell neighborhood"),'3')
         self.modelIMRGRA.addItem(self.tr("Iterative method with least squares initialization"),'4')
-
-        self.modelNTERUP.addItem(self.tr("SIMPLE"), 'simple')
-        self.modelNTERUP.addItem(self.tr("SIMPLEC"),'simplec')
-        self.modelNTERUP.addItem(self.tr("PISO"),'piso')
+        self.modelIMRGRA.addItem(self.tr("Iterative method with least squares initialization and extended neighbordood"),'5')
+        self.modelIMRGRA.addItem(self.tr("Iterative method with least squares initialization and partial extended neighbordood"),'6')
 
         self.comboBoxEXTRAG.setSizeAdjustPolicy(QComboBox.AdjustToContents)
-        self.comboBoxNTERUP.setSizeAdjustPolicy(QComboBox.AdjustToContents)
 
         # Connections
-        self.connect(self.checkBoxIVISSE, SIGNAL("clicked()"), self.slotIVISSE)
-        self.connect(self.checkBoxIPUCOU, SIGNAL("clicked()"), self.slotIPUCOU)
-        self.connect(self.checkBoxICFGRP, SIGNAL("clicked()"), self.slotICFGRP)
-        self.connect(self.checkBoxImprovedPressure, SIGNAL("clicked()"), self.slotImprovedPressure)
-        self.connect(self.comboBoxEXTRAG, SIGNAL("activated(const QString&)"), self.slotEXTRAG)
-        self.connect(self.lineEditRELAXP, SIGNAL("textChanged(const QString &)"), self.slotRELAXP)
-        self.connect(self.comboBoxIMRGRA, SIGNAL("activated(const QString&)"), self.slotIMRGRA)
-        self.connect(self.lineEditSRROM,  SIGNAL("textChanged(const QString &)"), self.slotSRROM)
-        self.connect(self.comboBoxNTERUP, SIGNAL("activated(const QString&)"), self.slotNTERUP)
-        self.connect(self.spinBoxNTERUP, SIGNAL("valueChanged(int)"), self.slotNTERUP2)
+        self.checkBoxIVISSE.clicked.connect(self.slotIVISSE)
+        self.checkBoxIPUCOU.clicked.connect(self.slotIPUCOU)
+        self.checkBoxICFGRP.clicked.connect(self.slotICFGRP)
+        self.checkBoxImprovedPressure.clicked.connect(self.slotImprovedPressure)
+        self.comboBoxEXTRAG.activated[str].connect(self.slotEXTRAG)
+        self.lineEditRELAXP.textChanged[str].connect(self.slotRELAXP)
+        self.comboBoxIMRGRA.activated[str].connect(self.slotIMRGRA)
+        self.lineEditSRROM.textChanged[str].connect(self.slotSRROM)
 
         # Validators
         validatorRELAXP = DoubleValidator(self.lineEditRELAXP, min=0., max=1.)
         validatorRELAXP.setExclusiveMin(True)
         validatorSRROM = DoubleValidator(self.lineEditSRROM, min=0., max=1.)
-        validatorSRROM.setExclusiveMin(True)
+        validatorSRROM.setExclusiveMax(True)
         self.lineEditRELAXP.setValidator(validatorRELAXP)
         self.lineEditSRROM.setValidator(validatorSRROM)
 
@@ -135,9 +128,10 @@ class NumericalParamGlobalView(QWidget, Ui_NumericalParamGlobalForm):
         else:
             self.checkBoxIPUCOU.setChecked(False)
 
-        import Pages.FluidCharacteristicsModel as FluidCharacteristics
+        import code_saturne.model.FluidCharacteristicsModel as FluidCharacteristics
         fluid = FluidCharacteristics.FluidCharacteristicsModel(self.case)
-        modl_atmo, modl_joul, modl_thermo, modl_gas, modl_coal, modl_comp = fluid.getThermoPhysicalModel()
+        modl_atmo, modl_joul, modl_thermo, modl_gas, modl_coal, modl_comp, modl_hgn = \
+            fluid.getThermoPhysicalModel()
 
         if self.model.getHydrostaticPressure() == 'on':
             self.checkBoxImprovedPressure.setChecked(True)
@@ -146,34 +140,16 @@ class NumericalParamGlobalView(QWidget, Ui_NumericalParamGlobalForm):
 
         self.lineEditRELAXP.setText(str(self.model.getPressureRelaxation()))
         self.modelEXTRAG.setItem(str_model=self.model.getWallPressureExtrapolation())
-        self.modelIMRGRA.setItem(str_model=self.model.getGradientReconstruction())
+        self.modelIMRGRA.setItem(str_model=str(self.model.getGradientReconstruction()))
 
         if modl_joul != 'off' or modl_gas != 'off' or modl_coal != 'off':
             self.labelSRROM.show()
             self.lineEditSRROM.show()
             self.lineEditSRROM.setText(str(self.model.getDensityRelaxation()))
-            self.line_5.show()
-
-        algo = self.model.getVelocityPressureAlgorithm()
-        status = SteadyManagementModel(self.case).getSteadyFlowManagement()
-        if status == 'on':
-            self.modelNTERUP.enableItem(str_model = 'simple')
-            self.modelNTERUP.disableItem(str_model = 'piso')
-        else:
-            self.modelNTERUP.disableItem(str_model = 'simple')
-            self.modelNTERUP.enableItem(str_model = 'piso')
-
-        self.modelNTERUP.setItem(str_model=algo)
-
-        if algo == 'piso':
-            self.spinBoxNTERUP.show()
-        else:
-            self.spinBoxNTERUP.hide()
 
         if modl_comp != 'off':
             self.labelICFGRP.show()
             self.checkBoxICFGRP.show()
-            self.line_4.show()
             if self.model.getHydrostaticEquilibrium() == 'on':
                 self.checkBoxICFGRP.setChecked(True)
             else:
@@ -184,30 +160,15 @@ class NumericalParamGlobalView(QWidget, Ui_NumericalParamGlobalForm):
             self.labelRELAXP.hide()
             self.checkBoxImprovedPressure.hide()
             self.labelImprovedPressure.hide()
-            self.line_2.hide()
-            self.line_5.hide()
-            self.line_7.hide()
-            self.line_8.hide()
-            self.labelNTERUP.setText("Velocity-Pressure algorithm\nsub-iterations on Navier-Stokes")
-            self.comboBoxNTERUP.hide()
-            self.spinBoxNTERUP.show()
         else:
             self.labelICFGRP.hide()
             self.checkBoxICFGRP.hide()
-            self.line_4.hide()
             self.checkBoxIPUCOU.show()
             self.labelIPUCOU.show()
             self.lineEditRELAXP.show()
             self.labelRELAXP.show()
             self.checkBoxImprovedPressure.show()
             self.labelImprovedPressure.show()
-            self.line_2.show()
-            self.line_5.show()
-            self.line_7.show()
-            self.line_8.show()
-
-        value = self.model.getPisoSweepNumber()
-        self.spinBoxNTERUP.setValue(value)
 
         # Update the Tree files and folders
         self.browser.configureTree(self.case)
@@ -215,7 +176,7 @@ class NumericalParamGlobalView(QWidget, Ui_NumericalParamGlobalForm):
         self.case.undoStartGlobal()
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotIVISSE(self):
         """
         Set value for parameter IVISSE
@@ -226,7 +187,7 @@ class NumericalParamGlobalView(QWidget, Ui_NumericalParamGlobalForm):
             self.model.setTransposedGradient("off")
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotIPUCOU(self):
         """
         Set value for parameter IPUCOU
@@ -237,7 +198,7 @@ class NumericalParamGlobalView(QWidget, Ui_NumericalParamGlobalForm):
             self.model.setVelocityPressureCoupling("off")
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotICFGRP(self):
         """
         Set value for parameter IPUCOU
@@ -248,7 +209,7 @@ class NumericalParamGlobalView(QWidget, Ui_NumericalParamGlobalForm):
             self.model.setHydrostaticEquilibrium("off")
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotImprovedPressure(self):
         """
         Input IHYDPR.
@@ -259,7 +220,7 @@ class NumericalParamGlobalView(QWidget, Ui_NumericalParamGlobalForm):
             self.model.setHydrostaticPressure("off")
 
 
-    @pyqtSignature("const QString &")
+    @pyqtSlot(str)
     def slotEXTRAG(self, text):
         """
         Set value for parameter EXTRAG
@@ -269,64 +230,38 @@ class NumericalParamGlobalView(QWidget, Ui_NumericalParamGlobalForm):
         log.debug("slotEXTRAG-> %s" % extrag)
 
 
-    @pyqtSignature("const QString &")
+    @pyqtSlot(str)
     def slotRELAXP(self, text):
         """
         Set value for parameter RELAXP
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditRELAXP.validator().state == QValidator.Acceptable:
             relaxp = from_qvariant(text, float)
             self.model.setPressureRelaxation(relaxp)
             log.debug("slotRELAXP-> %s" % relaxp)
 
 
-    @pyqtSignature("const QString &")
+    @pyqtSlot(str)
     def slotSRROM(self, text):
         """
         Set value for parameter SRROM
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditSRROM.validator().state == QValidator.Acceptable:
             srrom = from_qvariant(text, float)
             self.model.setDensityRelaxation(srrom)
             log.debug("slotSRROM-> %s" % srrom)
 
 
-    @pyqtSignature("const QString &")
+    @pyqtSlot(str)
     def slotIMRGRA(self, text):
         """
         Set value for parameter IMRGRA
         """
-        imrgra = self.modelIMRGRA.getIndex(str_view=str(text))
-        self.model.setGradientReconstruction(imrgra)
+        imrgra = self.modelIMRGRA.dicoV2M[str(text)]
+        self.model.setGradientReconstruction(int(imrgra))
         log.debug("slotIMRGRA-> %s" % imrgra)
 
 
-    @pyqtSignature("const QString &")
-    def slotNTERUP(self,text):
-        """
-        Set value for parameterNTERUP
-        """
-        NTERUP = self.modelNTERUP.dicoV2M[str(text)]
-        self.model.setVelocityPressureAlgorithm(NTERUP)
-        if NTERUP == 'piso':
-            self.spinBoxNTERUP.show()
-            value = self.model.getPisoSweepNumber()
-            self.spinBoxNTERUP.setValue(value)
-        else:
-            self.spinBoxNTERUP.hide()
-        self.browser.configureTree(self.case)
-        log.debug("slotNTERUP-> %s" % NTERUP)
-
-
-    @pyqtSignature("const QString &")
-    def slotNTERUP2(self, var):
-        """
-        Set value for parameter piso sweep number
-        """
-        self.model.setPisoSweepNumber(var)
-        log.debug("slotNTERUP2-> %s" % var)
-
-
     def tr(self, text):
         """
         Translation
