@@ -6,8 +6,8 @@
 
 import sys
 
-if sys.version_info[:2] < (2,4):
-    sys.stderr.write("This script needs Python 2.4 at least\n")
+if sys.version_info[:2] < (2,6):
+    sys.stderr.write("This script needs Python 2.6 at least\n")
 
 import platform
 
@@ -39,6 +39,7 @@ def run_command(cmd, stage, app, log):
 
     p = subprocess.Popen(cmd,
                          shell=True,
+                         universal_newlines=True,
                          stdout=subprocess.PIPE,
                          stderr=subprocess.STDOUT)
 
@@ -46,7 +47,7 @@ def run_command(cmd, stage, app, log):
     log.write(output[0])
 
     if p.returncode != 0:
-        sys.stderr.write("Error during " + string.lower(stage) +
+        sys.stderr.write("Error during " + stage.lower() +
                          " stage of " + app + ".\n")
         sys.stderr.write("See " + log.name + " for more information.\n")
         sys.exit(1)
@@ -65,6 +66,7 @@ def run_test(cmd):
 
     p = subprocess.Popen(cmd,
                          shell=True,
+                         universal_newlines=True,
                          stdout=subprocess.PIPE,
                          stderr=subprocess.STDOUT)
 
@@ -87,8 +89,7 @@ def check_directory():
     top_srcdir, script_name = os.path.split(script_path)
     abscwd = os.path.abspath(os.getcwd())
 
-    if abscwd.find(top_srcdir) == 0:
-            
+    if not os.path.relpath(abscwd, top_srcdir)[0:2] == '..':
         message = \
 """
 The '%(script_name)s' installer script should not be run from inside the
@@ -100,7 +101,7 @@ We recommend running for example:
   cd %(top_srcdir)s/..
   mkdir saturne_build
   cd saturne_build
-  ../%(rel_script_path)s
+  ../%(rel_dir_name)s/%(rel_script_path)s
 
 or (using absolute paths):
 
@@ -110,10 +111,12 @@ or (using absolute paths):
 
 """
         rel_script_path = os.path.basename(script_path)
+        rel_dir_name = os.path.basename(top_srcdir)
         build_path = top_srcdir + '_build'
 
         sys.stdout.write(message % {'script_name': script_name,
                                     'top_srcdir': top_srcdir,
+                                    'rel_dir_name': rel_dir_name,
                                     'rel_script_path': rel_script_path,
                                     'build_path': build_path,
                                     'script_path': script_path})
@@ -132,8 +135,9 @@ def find_executable(names, env_var=None):
     # test it first.
 
     if env_var:
-        if os.environ.has_key(env_var):
-            return os.environ(env_var)
+        k = os.environ.get(env_var)
+        if k:
+            return k
         else:
             for a in sys.argv[1:]:
                 if a.find(env_var) == 0:
@@ -142,7 +146,7 @@ def find_executable(names, env_var=None):
     # Otherwise, use standard path.
 
     p = os.getenv('PATH').split(':')
-        
+
     for name in names:
         if os.path.isabs(name):
             if os.path.isfile(name):
@@ -172,7 +176,10 @@ class Package:
         self.version = version
         self.archive = archive
         if self.archive:
-            self.url = url % self.archive
+            try:
+                self.url = url % self.archive
+            except Exception:
+                self.url = url
         else:
             self.url = None
 
@@ -261,6 +268,7 @@ class Package:
 
             p = subprocess.Popen('unzip ' + self.archive,
                                  shell=True,
+                                 universal_newlines=True,
                                  stdout=sys.stdout,
                                  stderr=sys.stderr)
             output = p.communicate()
@@ -382,19 +390,6 @@ class Package:
 
         os.chdir(os.path.join(build_dir, 'src'))
 
-        # Work around Ubuntu Metis build bug
-        ldflags_add = ''
-        try:
-            p = subprocess.Popen(self.cc + ' -Xlinker --help',
-                                 shell=True,
-                                 stdout=subprocess.PIPE,
-                                 stderr=subprocess.PIPE)
-            output = p.communicate()[0]
-            if output.find("--no-as-needed") > -1:
-                ldflags_add = ' -Wl,--no-as-needed\n'
-        except Exception:
-            pass
-
         if self.shared:
             fdr = open('Make.inc/Makefile.inc.x86-64_pc_linux2.shlib')
         else:
@@ -415,8 +410,6 @@ class Package:
             line = re.sub(re_intsize32, '', line)
             line = re.sub(re_intsize64, '', line)
             line = re.sub(re_idxsize64, '-DIDXSIZE64 -DINTSIZE64', line)
-            if ldflags_add and line[0:7] == 'LDFLAGS':
-                line = line[:-1] + ldflags_add
 
             fd.write(line)
 
@@ -532,7 +525,10 @@ class Setup:
         self.top_srcdir = os.path.abspath(os.path.dirname(sys.argv[0]))
 
         # Optional libraries
-        self.optlibs = ['hdf5', 'cgns', 'med', 'scotch', 'parmetis', 'libxml2']
+        self.optlibs = ['hdf5', 'cgns', 'med', 'scotch', 'parmetis']
+
+        # Optional libraries configure could find in salome
+        self.salome_optlibs = ['hdf5', 'cgns', 'med']
 
         # Logging file
         self.log_file = sys.stdout
@@ -586,7 +582,7 @@ class Setup:
                     package="code_saturne",
                     version=None,
                     archive=None,
-                    url="http://code-saturne.org")
+                    url="https://code-saturne.org")
 
         p = self.packages['code_saturne']
 
@@ -601,22 +597,22 @@ class Setup:
             Package(name="HDF5",
                     description="Hierarchical Data Format",
                     package="hdf5",
-                    version="1.8.10",
-                    archive="hdf5-1.8.10.tar.gz",
-                    url="http://www.hdfgroup.org/ftp/HDF5/releases/hdf5-1.8.10/src/%s")
+                    version="1.10.5",
+                    archive="hdf5-1.10.5.tar.gz",
+                    url="https://support.hdfgroup.org/ftp/HDF5/releases/hdf5-1.10/hdf5-1.10.5/src/%s")
 
         p = self.packages['hdf5']
-        p.config_opts = "--enable-production"
+        p.config_opts = "--enable-build-mode=production"
 
         # CGNS library
 
         self.packages['cgns'] = \
             Package(name="CGNS",
                     description="CFD General Notation System",
-                    package="cgnslib",
-                    version="3.2.1",
-                    archive="cgnslib_3.2.1.tar.gz",
-                    url="http://sourceforge.net/projects/cgns/files/cgnslib_3.2/%s/download")
+                    package="cgns",
+                    version="3.4.0",
+                    archive="CGNS-3.4.0.tar.gz",
+                    url="https://github.com/CGNS/CGNS/archive/v3.4.0.tar.gz")
 
         p = self.packages['cgns']
         p.config_opts = "-DCGNS_ENABLE_64BIT=ON -DCGNS_ENABLE_SCOPING=ON"
@@ -627,25 +623,12 @@ class Setup:
             Package(name="MED",
                     description="Model for Exchange of Data",
                     package="med",
-                    version="3.0.7",
-                    archive="med-3.0.7.tar.gz",
+                    version="4.0.0",
+                    archive="med-4.0.0.tar.gz",
                     url="http://files.salome-platform.org/Salome/other/%s")
 
         p = self.packages['med']
-        p.config_opts = "--with-med_int=int --disable-fortran"
-
-        # Libxml2 library (possible mirror at "ftp://fr.rpmfind.net/pub/libxml/%s")
-
-        self.packages['libxml2'] = \
-            Package(name="libxml2",
-                    description="XML library",
-                    package="libxml2",
-                    version="2.7.8",
-                    archive="libxml2-sources-2.7.8.tar.gz",
-                    url="ftp://xmlsoft.org/libxml2/%s")
-
-        p = self.packages['libxml2']
-        p.config_opts = "--with-ftp=no --with-http=no"
+        p.config_opts = "--with-med_int=long --disable-fortran --disable-python"
 
         # ParMETIS
 
@@ -663,17 +646,17 @@ class Setup:
             Package(name="scotch",
                     description="PT-Scotch",
                     package="scotch",
-                    version="6.0.0",
-                    archive="scotch_6.0.0.tar.gz",
-                    url="https://gforge.inria.fr/frs/download.php/31831/%s")
+                    version="6.0.6",
+                    archive="scotch_6.0.6.tar.gz",
+                    url="https://gforge.inria.fr/frs/download.php/file/37622/%s")
 
     #---------------------------------------------------------------------------
 
     def setup_defaults(self):
 
-        self.cc = find_executable(['cc', 'gcc', 'icc', 'xlc'], 'CC')
+        self.cc = find_executable(['cc', 'gcc', 'icc', 'xlc', 'clang'], 'CC')
         self.fc = find_executable(['f95', 'gfortran', 'ifort'], 'FC')
-        self.cxx = find_executable(['c++', 'g++', 'icpc', 'xlc++'], 'CXX')
+        self.cxx = find_executable(['c++', 'g++', 'icpc', 'xlc++', 'clang++'], 'CXX')
         self.mpicc = find_executable(['mpicc', 'mpicc.openmpi', 'mpicc.mpich'])
         self.mpicxx = find_executable(['mpicxx', 'mpicxx.openmpi', 'mpicxx.mpich'])
         self.python = find_executable(['python'], 'PYTHON')
@@ -708,12 +691,6 @@ class Setup:
                        header='med.h',
                        libname='medC')
 
-        # Libxml2 library (possible mirror at "ftp://fr.rpmfind.net/pub/libxml/%s")
-        p = self.packages['libxml2']
-
-        p.test_library(header='libxml2/libxml/parser.h',
-                       libname='xml2')
-
         # Expand user variables
 
         p = self.packages['code_saturne']
@@ -756,7 +733,7 @@ to start the installation.
         # setup file reading
         #
         try:
-            setupFile = file('setup', mode='r')
+            setupFile = open('setup', mode='r')
         except IOError:
             sys.stderr.write('Error: opening setup file\n')
             sys.exit(1)
@@ -921,7 +898,7 @@ Check the setup file and some utilities presence.
                                      "Please check your setup file.\n\n"
                                      % compiler)
                     sys.exit(1)
-        
+
         # Looking for Python executable provided by the user
         python = 'python'
         if self.python: python = self.python
@@ -939,11 +916,12 @@ Check the setup file and some utilities presence.
                                  "Please check your setup file.\n\n")
             sys.exit(1)
         else:
-            cmd = python + " -c \'import sys; print sys.version[:3]\'"
+            cmd = python + " -c \'import sys; print(sys.version[:3])\'"
             if verbose == 'yes':
                 sys.stdout.write("     Python version is ")
             p = subprocess.Popen(cmd,
                                  shell=True,
+                                 universal_newlines=True,
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.STDOUT)
 
@@ -952,6 +930,15 @@ Check the setup file and some utilities presence.
                 if p.returncode == 0:
                     sys.stdout.write(output[0])
 
+        # Looking for SALOME path provided by the user
+        if self.salome and not os.path.isdir(self.salome):
+            sys.stderr.write("\n*** Aborting installation:\n"
+                             "\'%s\' SALOME directory is provided in the setup "
+                             "file but is not present.\n"
+                             "Please check your setup file.\n\n"
+                             % self.salome)
+            sys.exit(1)
+
         # Checking libraries options
         for lib in self.optlibs:
             p = self.packages[lib]
@@ -970,7 +957,9 @@ Check the setup file and some utilities presence.
                                  % lib)
                 sys.exit(1)
             if p.installation == 'no' and p.use == 'yes':
-                if not os.path.isdir(p.install_dir):
+                use_salome = self.salome and lib in self.salome_optlibs and \
+                             p.install_dir == 'salome'
+                if not os.path.isdir(p.install_dir) and not use_salome:
                     sys.stderr.write("\n*** Aborting installation:\n"
                                      "\'%(path)s\' path is provided for "
                                      "\'%(lib)s\' in the setup "
@@ -979,16 +968,6 @@ Check the setup file and some utilities presence.
                                      % {'path':p.install_dir, 'lib':lib})
                     sys.exit(1)
 
-        # Looking for SALOME path probided by the user
-        if self.salome and not os.path.isdir(self.salome):
-            sys.stderr.write("\n*** Aborting installation:\n"
-                             "\'%s\' SALOME directory is provided in the setup "
-                             "file but is not present.\n"
-                             "Please check your setup file.\n\n"
-                             % self.salome)
-            sys.exit(1)
-
-
         # Looking for make utility
         ret = run_test("make")
         if ret != 0:
@@ -1019,9 +998,9 @@ Check the setup file and some utilities presence.
             # Compilers
             p.cc = self.cc
             p.cxx = self.cxx
-            if lib in ['code_saturne'] and self.mpicc:
+            if lib in ['scotch'] and self.mpicc:
                 p.cc = self.mpicc
-            elif lib in ['scotch', 'parmetis']:
+            elif lib in ['code_saturne', 'parmetis']:
                 if self.mpicc:
                     p.cc = self.mpicc
                 if self.mpicxx:
@@ -1041,7 +1020,6 @@ Check the setup file and some utilities presence.
         med= self.packages['med']
         scotch = self.packages['scotch']
         parmetis = self.packages['parmetis']
-        libxml2 = self.packages['libxml2']
 
         # Disable GUI
 
@@ -1062,8 +1040,8 @@ Check the setup file and some utilities presence.
             if hdf5.install_dir:
                 config_opts = config_opts + " --with-hdf5=" + hdf5.install_dir
                 med.config_opts += " --with-hdf5=" + hdf5.install_dir
-                cgns.config_opts += " -DHDF5_INCLUDE_PATH=" + hdf5.install_dir + "/include" \
-                    + " -DHDF5_LIBRARY=" + hdf5.install_dir + "/lib/libhdf5.so"
+                cgns.config_opts += "-DCMAKE_PREFIX_PATH=" + hdf5.install_dir \
+                    + " -DHDF5_INCLUDE_PATH=" + hdf5.install_dir + "/include"
 
         # CGNS
 
@@ -1095,15 +1073,6 @@ Check the setup file and some utilities presence.
         else:
             config_opts = config_opts + " --with-scotch=" + scotch.install_dir
 
-        # Libxml2
-
-        if libxml2.use == 'no':
-            config_opts = config_opts + " --without-libxml2"
-        else:
-            if libxml2.install_dir:
-                config_opts = config_opts + \
-                    " --with-libxml2=" + libxml2.install_dir
-
         # Python
 
         if self.python:
@@ -1136,7 +1105,7 @@ Check the setup file and some utilities presence.
             for lib in self.optlibs:
                 p = self.packages[lib]
                 if p.installation == 'yes':
-                    sys.stdout.write("Download of %s\n" % p.name)
+                    sys.stdout.write("Download of %s\n  (%s)\n" % (p.name, p.url))
                     p.download()
             self.download = 'no'
             self.write_setup()
@@ -1181,7 +1150,7 @@ Check the setup file and some utilities presence.
         #
         # setup file update
         #
-        sf = file(os.path.join(os.getcwd(), "setup"), mode='w')
+        sf = open(os.path.join(os.getcwd(), "setup"), mode='w')
 
         setupMain = \
 """#========================================================
@@ -1281,27 +1250,26 @@ salome    %(salome)s
 #
 #   For Linux workstations, HDF5, CGNS, and even MED
 # packages may be available through the package manager.
-# HDF5 is also often available on large systems such as
-# IBM Blue Gene or Cray XT/XE/XK.
+# HDF5 is also often available on large systems.
+# When building with SALOME, the platform distribution's
+# packages may be used, by setting in the matching entry
+# 'yes' under the "Use" column and simply 'salome' under
+# the path column.
 #
 # Scotch and Pt-Scotch are available in some Linux
-# distributtions, but may be built with options
+# distributions, but may be built with options
 # incompatible with non-threaded Code_Saturne runs.
 #
 #   To install CGNS or ParMetis, the CMake
 # configuration/installation tool is required
-# (it is available in most Linux distributions)
-#
-#   Libxml2 is needed to read XML files output by the
-# Graphical User Interface. It is generally available
-# through the package manager.
+# (it is available in most Linux distributions).
 #--------------------------------------------------------
 #
 #  Name    Use   Install  Path
 #
 """
         setupLib= \
-"""%(lib)-9s  %(use)-4s  %(install)-3s      %(dir)s    
+"""%(lib)-9s  %(use)-4s  %(install)-3s      %(dir)s
 """
         setupEnd= \
 """#
@@ -1372,7 +1340,7 @@ You can have a look at the log file meanwhile.
 Before using Code_Saturne, please update your environment with:
 
   cspath=%(cspath)s
-  alias code_saturne="$cspath/bin"
+  alias code_saturne="$cspath/code_saturne"
 
 The documentation should then be available through the commands:
   code_saturne info -g refcard
