@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -19,28 +19,12 @@
 ! Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 !-------------------------------------------------------------------------------
-
+!> \file atprop.f90
+!> \brief Add if needed the variables fields for temperature and liquid water
+!>
+!> \brief Add if needed the variable field for temperature and liquid water \n
+!>    Nota : ippmod(iatmos) = 1 --> Dry atmosphere, = 2 --> Humid atmosphere
 subroutine atprop
-!================
-
-!===============================================================================
-!  FONCTION  :
-!  ---------
-
-!     INIT DES POSITIONS DES VARIABLES D'ETAT
-!            POUR LE MODULE ATMOSPHERIQUE
-
-!-------------------------------------------------------------------------------
-! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-!__________________!____!_____!________________________________________________!
-
-!     Type: i (integer), r (real), s (string), a (array), l (logical),
-!           and composite types (ex: ra real array)
-!     mode: <-- input, --> output, <-> modifies data, --- work array
-!===============================================================================
 
 !===============================================================================
 ! Module files
@@ -57,7 +41,7 @@ use ppppar
 use ppthch
 use ppincl
 use ihmpre
-
+use field
 use atincl
 
 !===============================================================================
@@ -66,37 +50,48 @@ implicit none
 
 ! Local variables
 
-integer       nprini
+integer          f_id, idim1, ityloc, itycat
 
 !===============================================================================
 
-nprini = nproce
-
 !===============================================================================
 ! 1. POSITIONNEMENT DES PROPRIETES
 !    Atmospheric modules:  dry and humid atmosphere
 !===============================================================================
 
-! Temperature (IPPMOD(IATMOS) = 1 or 2)
-!--------------------------------------
+! Temperature (ippmod(iatmos) = 1 or 2)
 
 if (ippmod(iatmos).ge.1) then
-  call add_property_field('t_celcius', 'RealTemp', itempc)
+  call add_property_field_1d('real_temperature', 'RealTemp', itempc)
+
+  call add_boundary_property_field_owner('non_neutral_scalar_correction',  &
+                                         'Non Neutral Scalar Correction', f_id)
+  call field_set_key_int(f_id, keylog, 0)
 endif
 
-! Liquid water content (IPPMOD(IATMOS) = 2)
-!------------------------------------------
+! Liquid water content (ippmod(iatmos) = 2)
 
 if (ippmod(iatmos).eq.2) then
-  call add_property_field('liquid_water_content', 'LiqWater', iliqwt)
+  call add_property_field_1d('liquid_water', 'LiqWater', iliqwt)
+
+  ! sedimentation and deposition model enabled
+  if (modsedi.eq.1.and.moddep.gt.0) then
+    idim1  = 1
+    itycat = FIELD_INTENSIVE + FIELD_PROPERTY
+    ityloc = 3 ! boundary faces
+
+    ! wall friction velocity if not already created
+    call field_find_or_create('ustar', itycat, ityloc, idim1, f_id)
+
+    ! boundary roughness
+    call add_boundary_property_field_owner('boundary_roughness',  &
+                                           'Boundary Roughness', f_id)
+
+    ! boundary thermal roughness
+    call add_boundary_property_field_owner('boundary_thermal_roughness',  &
+                                           'Boundary Thermal Roughness', f_id)
+  endif
 endif
 
-! Nb algebraic (or state) variables
-!   specific to specific physic: nsalpp
-!   total: nsalto
-
-nsalpp = nproce - nprini
-nsalto = nproce
-
 return
 end subroutine atprop
