@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -19,43 +19,27 @@
 ! Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 !-------------------------------------------------------------------------------
-
-subroutine solvar &
-!================
-   ( temp , qv , rom , dt ,                                                  &
-     rcodcl , rtp )
-
-
-!==============================================================================
-! FUNCTION
-!==============================================================================
-! *--------                                                            *
-! *  module d'interface sol/atmosphere                                 *
-! *  calcule les variables de surface (temperature et humidite):       *
-! *    - pour la temperature : avec une equation prognostique du type  *
-! *      "force-restore" (deardorff 1978)                              *
-! *    - pour l'humidite : par une methode a 2 reservoirs              *
-! *                                                                    *
-!==============================================================================
+!> \file solvar.f90
+!> \brief Atmospheric soil module - Compute ground level variables
+!
+!> \brief   Soil / atmosphere interface routine
+!>   Compute the following surface variables :
+!>-     temperature with a prognostic equation of the "force-restore" type
+!>                       (deardorff 1978)
+!>-     humidity with a two reservoirs method
+!
 !-------------------------------------------------------------------------------
 ! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-!  temp            ! ra ! <-- ! temperature                                    !
-!  qv              ! ra ! <-- ! humidite specifique                            !
-!  pa              ! ra ! <-- ! pression projetee sur maillage vitesse         !
-!  rom             ! ra ! <-- ! masse volumique                                !
-!  dt              ! ra ! <-- ! rapport du pas de temps local au pas           !
-!                  !    !     ! de temps de reference                          !
+!______________________________________________________________________________.
+!  mode           name          role
 !______________________________________________________________________________!
-
-!     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
-!            L (LOGIQUE)   .. ET TYPES COMPOSES (EX : TR TABLEAU REEL)
-!     MODE : <-- donnee, --> resultat, <-> Donnee modifiee
-!            --- tableau de travail
-!===============================================================================
-
+!> \param[in]   temp    temperature
+!> \param[in]   qv      specific humidity
+!> \param[in]   rom     Density
+!> \param[in]   dt      ratio of the local time step to the reference one
+!> \param[in]   rcodcl  Boundary conditions type
+!-------------------------------------------------------------------------------
+subroutine solvar ( temp , qv ,rom , dt, rcodcl )
 
 !===============================================================================
 ! Module files
@@ -65,7 +49,6 @@ use paramx
 use dimens
 use numvar
 use optcal
-use dimens, only: nvar
 use cstphy
 use cstnum
 use entsor
@@ -89,18 +72,15 @@ parameter ( idim2 = 26 )
 !==============================================================================
 
 
-!... declaration des variables externes
+! Arguments
 
-
-!double precision rcodcl(nfabor,nvar,3),velipb(nfabor,3)
-double precision rcodcl(nfabor,nvarcl,3),rtp(ncelet,nflown:nvar)
+double precision rcodcl(nfabor,nvar,3)
 
 double precision temp(ncelet)
 double precision qv(ncelet)
 double precision rom(ncelet),dt(ncelet)
 
-
-!... declaration des variables internes
+! Local variables
 
 integer ifac,iel,isol
 double precision rcodcx, rcodcy, rcodcz, rcodsn
@@ -134,7 +114,6 @@ call field_get_val_v(ivarfl(iu), vel)
 !     1) initialisations locales
 !     ==========================
 
-
 b = 5.d0
 c = 5.d0
 d = 5.d0
@@ -375,6 +354,8 @@ do isol = 1, nfmodsol
 
 enddo
 
+call cs_user_atmo_soil(temp, qv, rom, dt, rcodcl)
+
 return
 
 end subroutine solvar
