@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -27,6 +27,8 @@ module albase
 
   !=============================================================================
 
+  use, intrinsic :: iso_c_binding
+
   implicit none
 
   !=============================================================================
@@ -36,14 +38,15 @@ module albase
   !> \addtogroup albase
   !> \{
 
-  !> Activates (=1) or not (=0), activate the ALE module
-  integer, save :: iale
+  !> Activates (=1 o) or not (=0), activate the ALE module:
+  !>  - 0: not activated
+  !>  - 1: legacy solver
+  !>  - 2: CDO solver
+  integer(c_int), pointer, save :: iale
   !> the number of sub-iterations of initialization of the fluid
   integer, save :: nalinf
   !> maximum number of imlicitation iterations of of the structure displacement
   integer, save :: nalimx
-  !> viscosity type of mesh : isotropic=0, orthotropic=1.
-  integer, save :: iortvm
   !> relative precision of implicitation of the structure displacement
   double precision, save :: epalim
   !> iteration (yes=1, no=0) to initialize ALE
@@ -51,17 +54,15 @@ module albase
 
   !> indicator of imposed displacement
   integer, allocatable, dimension(:) :: impale
-  !> defines the mesh velocity from the colour of the boundary faces,
-  !> or more generally from their properties (colours, groups, ...),
+  !> defines the mesh velocity from the color of the boundary faces,
+  !> or more generally from their properties (colors, groups, ...),
   !> from the boundary conditions defined in cs user boundary conditions,
   !> or even from their coordinates.
   integer, allocatable, dimension(:) :: ialtyb
   !> initial mesh coordinates
   double precision, allocatable, dimension(:,:) :: xyzno0
-  !> mesh displacement
-  double precision, allocatable, dimension(:,:) :: depale
-  !> mesh displacement of the previous time-step
-  double precision, allocatable, dimension(:,:) :: disala
+  !> Pointer to field over vertices: mesh displacement
+  integer, save :: fdiale
   !> \}
 
 contains
@@ -77,15 +78,10 @@ contains
 
     integer, intent(in) :: nfabor, nnod
 
-    if (iale.eq.1.or.imobil.eq.1) then
+    if (iale.ge.1) then
       allocate(xyzno0(3,nnod))
-    endif
-
-    if (iale.eq.1) then
       allocate(impale(nnod))
       allocate(ialtyb(nfabor))
-      allocate(depale(3,nnod))
-      allocate(disala(3,nnod))
     endif
 
   end subroutine init_ale
@@ -96,14 +92,9 @@ contains
 
     use cplsat
 
-    if (iale.eq.1.or.imobil.eq.1) then
+    if (iale.ge.1) then
       deallocate(xyzno0)
-    endif
-
-    if (iale.eq.1) then
       deallocate(impale)
-      deallocate(depale)
-      if (allocated(disala)) deallocate(disala)
       deallocate(ialtyb)
     endif
 
@@ -111,4 +102,21 @@ contains
 
   !=============================================================================
 
+  subroutine map_ale
+
+    use, intrinsic :: iso_c_binding
+    use cs_c_bindings
+
+    ! Local variables
+
+    type(c_ptr) :: c_iale
+
+    call cs_f_ale_get_pointers(c_iale)
+
+    call c_f_pointer(c_iale, iale)
+
+  end subroutine map_ale
+
+  !=============================================================================
+
 end module albase
