@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -21,7 +21,7 @@
 !-------------------------------------------------------------------------------
 
 !> \file altycl.f90
-!> \brief Boundary condition code (in \ref ialtyb) for the ALE module
+!> \brief Boundary condition code for the ALE module
 !>
 !------------------------------------------------------------------------------
 
@@ -40,7 +40,7 @@
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
 !>                               - 5 smooth wall and
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
-!>                               - 6 rought wall and
+!>                               - 6 rough wall and
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
 !>                               - 9 free inlet/outlet
 !>                                 (input mass flux blocked to 0)
@@ -53,7 +53,7 @@
 !>                               - rcodcl(2) value of the exterior exchange
 !>                                 coefficient (infinite if no exchange)
 !>                               - rcodcl(3) value flux density
-!>                                 (negative if gain) in w/m2 or roughtness
+!>                                 (negative if gain) in w/m2 or roughness
 !>                                 in m if icodcl=6
 !>                                 -# for the velocity \f$ (\mu+\mu_T)
 !>                                    \gradv \vect{u} \cdot \vect{n}  \f$
@@ -62,14 +62,13 @@
 !>                                 -# for a scalar \f$ cp \left( K +
 !>                                     \dfrac{K_T}{\sigma_T} \right)
 !>                                     \grad T \cdot \vect{n} \f$
-!> \param[in,out] depale        nodes displacement
 !> \param[in]     xyzno0        initial mesh nodes coordinates
 !______________________________________________________________________________
 
 subroutine altycl &
  ( itypfb , ialtyb , icodcl , impale ,                            &
    dt     ,                                                      &
-   rcodcl , xyzno0 , depale )
+   rcodcl , xyzno0)
 
 !===============================================================================
 ! Module files
@@ -79,10 +78,14 @@ use paramx
 use numvar
 use optcal
 use cstnum
+use dimens, only: nvar
 use cstphy
 use entsor
 use parall
 use mesh
+use field
+use albase, only: fdiale, iale
+use cs_c_bindings
 
 !===============================================================================
 
@@ -91,12 +94,12 @@ implicit none
 ! Arguments
 
 integer          itypfb(nfabor)
-integer          ialtyb(nfabor), icodcl(nfabor,nvarcl)
+integer          ialtyb(nfabor), icodcl(nfabor,nvar)
 integer          impale(nnod)
 
 double precision dt(ncelet)
-double precision rcodcl(nfabor,nvarcl,3)
-double precision depale(3,nnod), xyzno0(3,nnod)
+double precision rcodcl(nfabor,nvar,3)
+double precision xyzno0(3,nnod)
 
 ! Local variables
 
@@ -106,11 +109,13 @@ double precision ddepx, ddepy, ddepz
 double precision srfbnf, rnx, rny, rnz
 double precision rcodcx, rcodcy, rcodcz, rcodsn
 
+double precision, dimension(:,:), pointer :: disale
+double precision, allocatable, dimension(:,:) :: b_fluid_vel
 
 !===============================================================================
 
 !===============================================================================
-! 1.  INITIALISATIONS
+! 1. Initializations
 !===============================================================================
 
 irkerr = -1
@@ -125,21 +130,47 @@ icoder(2) = -1
 icoder(3) = -1
 icoder(4) = -1
 
-! Mise a zero des RCODCL non specifies
+call field_get_val_v(fdiale, disale)
+
+! Set to 0 non specified RCODCL arrays
 do ifac = 1, nfabor
   if (rcodcl(ifac,iuma,1).gt.rinfin*0.5d0)                        &
-       rcodcl(ifac,iuma,1) = 0.d0
+    rcodcl(ifac,iuma,1) = 0.d0
   if (rcodcl(ifac,ivma,1).gt.rinfin*0.5d0)                        &
-       rcodcl(ifac,ivma,1) = 0.d0
+    rcodcl(ifac,ivma,1) = 0.d0
   if (rcodcl(ifac,iwma,1).gt.rinfin*0.5d0)                        &
-       rcodcl(ifac,iwma,1) = 0.d0
+    rcodcl(ifac,iwma,1) = 0.d0
 enddo
 
 !===============================================================================
-! 2.  VERIFICATION DE LA CONSISTANCE DES IALTYB DONNES DANS USALCL
+! 2. Check the consistency of BC types (IALTYB) given in USALCL
 !===============================================================================
-!  (valeur 0 autorisee)
 
+! When using CDO solver, no need of checks.
+if (iale.eq.2) then
+
+  allocate(b_fluid_vel(3, nfabor))
+
+  do ifac = 1, nfabor
+    b_fluid_vel(1, ifac) = 0.d0
+    b_fluid_vel(2, ifac) = 0.d0
+    b_fluid_vel(3, ifac) = 0.d0
+  enddo
+
+  call cs_ale_update_bcs(ialtyb, b_fluid_vel)
+
+  ! Copy back to deprecated rcodcl
+  do ifac = 1, nfabor
+    rcodcl(ifac, iuma, 1) = b_fluid_vel(1, ifac)
+    rcodcl(ifac, ivma, 1) = b_fluid_vel(2, ifac)
+    rcodcl(ifac, iwma, 1) = b_fluid_vel(3, ifac)
+  enddo
+
+  goto 100
+
+endif
+
+!  (valeur 0 autorisee)
 do ifac = 1, nfabor
   if (ialtyb(ifac).ne.0      .and.                                &
       ialtyb(ifac).ne.ibfixe .and.                                &
@@ -154,17 +185,17 @@ do ifac = 1, nfabor
 enddo
 
 if (irangp.ge.0) call parcmx(ierror(1))
-if(ierror(1).ne.0) then
+if (ierror(1).ne.0) then
   write(nfecra,1000)
-  call bcderr(ialtyb)
+  call boundary_conditions_error(ialtyb)
 endif
 
 !===============================================================================
-! 3.  CONVERSION EN RCODCL ICODCL
+! 3. Conversion  into BC codes and values (ICODCL and RCODCL)
 !===============================================================================
 
-! Calcul ou ecrasement de RCODCL a partir du deplacement impose,
-!   si tous les noeuds d'une face sont a deplacement impose
+! If all the nodes of a face have an imposed displacement, RCODCL is computed or
+! overwritten, ALE BC type is therfore ivimpo
 
 do ifac = 1, nfabor
   iecrw = 0
@@ -176,11 +207,12 @@ do ifac = 1, nfabor
     inod = nodfbr(ii)
     if (impale(inod).eq.0) iecrw = iecrw + 1
     icpt = icpt + 1
-    ddepx = ddepx + depale(1,inod)+xyzno0(1,inod)-xyznod(1,inod)
-    ddepy = ddepy + depale(2,inod)+xyzno0(2,inod)-xyznod(2,inod)
-    ddepz = ddepz + depale(3,inod)+xyzno0(3,inod)-xyznod(3,inod)
+    ddepx = ddepx + disale(1,inod)+xyzno0(1,inod)-xyznod(1,inod)
+    ddepy = ddepy + disale(2,inod)+xyzno0(2,inod)-xyznod(2,inod)
+    ddepz = ddepz + disale(3,inod)+xyzno0(3,inod)-xyznod(3,inod)
   enddo
-  if (iecrw.eq.0) then
+  ! For Sliding walls, no imposed velocity
+  if (iecrw.eq.0.and.ialtyb(ifac).ne.igliss) then
     iel = ifabor(ifac)
     ialtyb(ifac) = ivimpo
     rcodcl(ifac,iuma,1) = ddepx/dt(iel)/icpt
@@ -195,9 +227,9 @@ do ifac = 1, nfabor
 
   iel = ifabor(ifac)
 
-! --> Faces fixes
-!     On force alors les noeuds en question a etre fixes, pour eviter
-!       des problemes eventuels aux coins
+  ! --> Fixed faces
+  !     On force alors les noeuds en question a etre fixes, pour eviter
+  !       des problemes eventuels aux coins
 
   if (ialtyb(ifac).eq.ibfixe) then
     icpt = 0
@@ -216,38 +248,36 @@ do ifac = 1, nfabor
       icodcl(ifac,iwma) = 1
       rcodcl(ifac,iwma,1) = 0.d0
     endif
-!      Si on a fixe les trois composantes, alors on fixe les noeuds
-!        correspondants. Sinon c'est que l'utilisateur a modifie quelque
-!        chose     ... on le laisse seul maitre
+    ! Si on a fixe les trois composantes, alors on fixe les noeuds
+    !   correspondants. Sinon c'est que l'utilisateur a modifie quelque
+    !   chose     ... on le laisse seul maitre
     if (icpt.eq.3) then
       do ii = ipnfbr(ifac), ipnfbr(ifac+1)-1
         inod = nodfbr(ii)
         if (impale(inod).eq.0) then
-          depale(1,inod) = 0.d0
-          depale(2,inod) = 0.d0
-          depale(3,inod) = 0.d0
+          disale(1,inod) = 0.d0
+          disale(2,inod) = 0.d0
+          disale(3,inod) = 0.d0
           impale(inod) = 1
         endif
       enddo
     endif
 
-
-! --> Faces de glissement
+  ! --> Sliding face
   elseif (ialtyb(ifac).eq.igliss) then
 
     if (icodcl(ifac,iuma).eq.0) icodcl(ifac,iuma) = 4
     if (icodcl(ifac,ivma).eq.0) icodcl(ifac,ivma) = 4
     if (icodcl(ifac,iwma).eq.0) icodcl(ifac,iwma) = 4
 
-! --> Faces a vitesse imposee
+  ! --> Imposed mesh velocity face
   elseif (ialtyb(ifac).eq.ivimpo) then
 
     if (icodcl(ifac,iuma).eq.0) icodcl(ifac,iuma) = 1
     if (icodcl(ifac,ivma).eq.0) icodcl(ifac,ivma) = 1
     if (icodcl(ifac,iwma).eq.0) icodcl(ifac,iwma) = 1
 
-! --> Free surface face: the mesh velocity is imposed by the mass flux
-!TODO set default condition on the fluid velocity to homogenous Neumann
+  ! --> Free surface face: the mesh velocity is imposed by the mass flux
   elseif (ialtyb(ifac).eq.ifresf) then
 
     if (icodcl(ifac,iuma).eq.0) then
@@ -265,13 +295,13 @@ do ifac = 1, nfabor
 enddo
 
 !===============================================================================
-! 4.  VERIFICATION DE LA CONSISTANCE DES ICODCL
+! 4. Check ICODCL consistency
 !===============================================================================
 
-!     IERROR a ete initialise plus haut
 do ifac = 1, nfabor
 
   if (icodcl(ifac,iuma).ne.1 .and.                                &
+      icodcl(ifac,iuma).ne.2 .and.                                &
       icodcl(ifac,iuma).ne.3 .and.                                &
       icodcl(ifac,iuma).ne.4 ) then
     if (ialtyb(ifac).gt.0) then
@@ -280,6 +310,7 @@ do ifac = 1, nfabor
     ierror(1) = ierror(1) + 1
   endif
   if (icodcl(ifac,ivma).ne.1 .and.                                &
+      icodcl(ifac,ivma).ne.2 .and.                                &
       icodcl(ifac,ivma).ne.3 .and.                                &
       icodcl(ifac,ivma).ne.4 ) then
     if (ialtyb(ifac).gt.0) then
@@ -288,6 +319,7 @@ do ifac = 1, nfabor
     ierror(2) = ierror(2) + 1
   endif
   if (icodcl(ifac,iwma).ne.1 .and.                                &
+      icodcl(ifac,iwma).ne.2 .and.                                &
       icodcl(ifac,iwma).ne.3 .and.                                &
       icodcl(ifac,iwma).ne.4 ) then
     if (ialtyb(ifac).gt.0) then
@@ -334,7 +366,7 @@ endif
 if (ierror(4).gt.0) then
   ierror(1) = 1
   if (irangp.ge.0) then
-    call parimx(1, irkerr)
+    call parcmx(irkerr)
     call parbci(irkerr, 4, icoder)
     write(nfecra,3000) icoder(1), icoder(2), icoder(3), icoder(4)
   endif
@@ -342,13 +374,15 @@ endif
 
 if (ierror(1).gt.0) then
   write(nfecra,4000)
-  call bcderr(ialtyb)
+  call boundary_conditions_error(ialtyb)
 endif
 
 !===============================================================================
-! 5.  VITESSE DE DEFILEMENT POUR LES PAROIS FLUIDES ET SYMETRIES
+! 5. Fluid velocity BCs for walls and symmetries (due to mesh movment)
 !===============================================================================
 
+100  continue
+
 ! Pour les symetries on rajoute toujours la vitesse de maillage, car on
 !   ne conserve que la vitesse normale
 ! Pour les parois, on prend la vitesse de maillage si l'utilisateur n'a
@@ -361,17 +395,18 @@ do ifac = 1, nfabor
 
   if (ialtyb(ifac).eq.ivimpo) then
 
-    if ( itypfb(ifac).eq.isymet ) then
+    ! Warning: only the normal component is kept in clsyvt
+    if (itypfb(ifac).eq.isymet) then
       rcodcl(ifac,iu,1) = rcodcl(ifac,iuma,1)
       rcodcl(ifac,iv,1) = rcodcl(ifac,ivma,1)
       rcodcl(ifac,iw,1) = rcodcl(ifac,iwma,1)
     endif
 
-    if ( itypfb(ifac).eq.iparoi .or.                        &
-         itypfb(ifac).eq.iparug ) then
-! Si une des composantes de vitesse de glissement a ete
-!    modifiee par l'utilisateur, on ne fixe que la vitesse
-!    normale
+    if (itypfb(ifac).eq.iparoi .or.                        &
+        itypfb(ifac).eq.iparug) then
+      ! WARNING
+      ! If nothing is set by the user, then the wall is supposed
+      ! to move with the sliding wall!
       if (rcodcl(ifac,iu,1).gt.rinfin*0.5d0 .and.              &
           rcodcl(ifac,iv,1).gt.rinfin*0.5d0 .and.              &
           rcodcl(ifac,iw,1).gt.rinfin*0.5d0) then
@@ -379,13 +414,16 @@ do ifac = 1, nfabor
         rcodcl(ifac,iv,1) = rcodcl(ifac,ivma,1)
         rcodcl(ifac,iw,1) = rcodcl(ifac,iwma,1)
       else
-! On met a 0 les composantes de RCODCL non specifiees
+        ! Otherwise, if the user has set the fluid velocity
+        ! Then the normal part is the one of the mesh velocity
+        ! and the tangential part is the one of the user (completed with 0
+        ! for non set values)
         if (rcodcl(ifac,iu,1).gt.rinfin*0.5d0)                 &
-             rcodcl(ifac,iu,1) = 0.d0
+          rcodcl(ifac,iu,1) = 0.d0
         if (rcodcl(ifac,iv,1).gt.rinfin*0.5d0)                 &
-             rcodcl(ifac,iv,1) = 0.d0
+          rcodcl(ifac,iv,1) = 0.d0
         if (rcodcl(ifac,iw,1).gt.rinfin*0.5d0)                 &
-             rcodcl(ifac,iw,1) = 0.d0
+          rcodcl(ifac,iw,1) = 0.d0
 
         srfbnf = surfbn(ifac)
         rnx = surfbo(1,ifac)/srfbnf
@@ -396,7 +434,7 @@ do ifac = 1, nfabor
         rcodcz = rcodcl(ifac,iw,1)
         rcodsn = (rcodcl(ifac,iuma,1)-rcodcx)*rnx                 &
              +   (rcodcl(ifac,ivma,1)-rcodcy)*rny                 &
-             +   (rcodcl(ifac,ivma,1)-rcodcz)*rnz
+             +   (rcodcl(ifac,iwma,1)-rcodcz)*rnz
         rcodcl(ifac,iu,1) = rcodcx + rcodsn*rnx
         rcodcl(ifac,iv,1) = rcodcy + rcodsn*rny
         rcodcl(ifac,iw,1) = rcodcz + rcodsn*rnz
@@ -407,9 +445,9 @@ do ifac = 1, nfabor
 
 enddo
 
-!===============================================================================
-! FORMATS
-!===============================================================================
+!========
+! Formats
+!========
 
 #if defined(_CS_LANG_FR)
 
@@ -417,7 +455,7 @@ enddo
 '@'                                                            ,/,&
 '@ METHODE ALE'                                                ,/,&
 '@'                                                            ,/,&
-'@ Au moins une face de bord a un type de bord non reconnu.'  ,/,&
+'@ Au moins une face de bord a un type de bord non reconnu.'   ,/,&
 '@    Le calcul ne sera pas execute.'                          ,/,&
 '@'                                                            ,/,&
 '@ Verifier les conditions aux limites dans l''Interface'      ,/,&
