@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -50,11 +50,9 @@
 #include "bft_printf.h"
 #include "bft_error.h"
 
+#include "fvm_io_num.h"
 #include "fvm_nodal.h"
-
 #include "fvm_nodal_extract.h"
-#include "fvm_nodal_project.h"
-#include "fvm_nodal_triangulate.h"
 
 #include "cs_calcium.h"
 #include "cs_interface.h"
@@ -62,6 +60,8 @@
 #include "cs_mesh_quantities.h"
 #include "cs_mesh_connect.h"
 #include "cs_parall.h"
+#include "cs_part_to_block.h"
+#include "cs_block_to_part.h"
 #include "cs_post.h"
 
 /*----------------------------------------------------------------------------
@@ -74,6 +74,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*=============================================================================
  * Local Structure Definitions
  *============================================================================*/
@@ -81,29 +83,28 @@ BEGIN_C_DECLS
 struct _cs_ast_coupling_t {
 
   cs_gnum_t    n_g_faces;
-  cs_gnum_t    n_g_nodes;
+  cs_gnum_t    n_g_vertices;
 
-  /* #if defined(HAVE_MPI) */
+  cs_lnum_t    n_faces;
+  cs_lnum_t    n_vertices;
 
-  cs_int_t    *n_faces_by_domain;
-  cs_int_t    *n_nodes_by_domain;
+  cs_lnum_t    *s_vtx_num;
 
-  cs_int_t    *face_index;
-  cs_int_t    *node_index;
+#if defined(HAVE_MPI)
 
-  cs_gnum_t   *NUM_GLOB_PTS;
+  cs_part_to_block_t *face_p2b;
+  cs_block_to_part_t *vtx_b2p;
 
-  /* #endif */
+#endif
 
 };
 
 /*============================================================================
- *  Global variables
+ * Global variables
  *============================================================================*/
 
 cs_ast_coupling_t  *cs_glob_ast_coupling = NULL;
 
-
 static int  comp_id = 0;
 
 static double  cur_time = 0.;
@@ -112,137 +113,279 @@ static double  max_time = 0.;
 
 static cs_calcium_timedep_t  time_dep = CALCIUM_iteration;
 
-
 /*============================================================================
  * Private function definitions
  *============================================================================*/
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
+/*============================================================================
+ * Public function definitions for Fortran API
+ *============================================================================*/
+
 /*----------------------------------------------------------------------------
- * Send nodes coordinates and structure numbering of coupled mesh.
+ * Send vertices coordinates and structure numbering of coupled mesh.
  *----------------------------------------------------------------------------*/
 
 void CS_PROCF(astgeo, ASTGEO)
 (
   cs_int_t   *nbfast,
-  cs_int_t   *nbnast,
   cs_int_t   *lstfac,
   cs_int_t   *idfast,
   cs_int_t   *idnast,
   cs_real_t  *almax
 )
 {
-  int j, n_faces;
-  int n_nodes;
+  cs_lnum_t j, n_faces;
+  cs_lnum_t n_vertices;
 
-  fvm_nodal_t  *ifs_mesh;
+  cs_lnum_t *faces_color = NULL;
+  cs_lnum_t *vertices_color = NULL;
 
-  cs_int_t *faces_color = NULL;
-  cs_int_t *nodes_color = NULL;
-
-  cs_real_t *faces_coords = NULL;
-  cs_real_t *nodes_coords = NULL;
+  cs_real_t *face_centers = NULL;
+  cs_real_t *vtx_coords = NULL;
 
   cs_real_t *b_face_cog = cs_glob_mesh_quantities->b_face_cog;
 
-  cs_ast_coupling_t  *ast_coupling = cs_glob_ast_coupling;
+  cs_ast_coupling_t  *ast_coupling = NULL;
 
   n_faces = *(nbfast);
 
-  ifs_mesh = cs_mesh_connect_faces_to_nodal(cs_glob_mesh,
-                                            "MaillageExtraitAster_1",
-                                            false,
-                                            0,
-                                            n_faces,
-                                            NULL,
-                                            lstfac);
-
-  n_nodes = (cs_int_t) fvm_nodal_get_n_entities(ifs_mesh, 0);
-
-
-  assert(cs_glob_n_ranks == 1);
+  fvm_nodal_t
+    *fsi_mesh = cs_mesh_connect_faces_to_nodal(cs_glob_mesh,
+                                               "MaillageExtraitAster_1",
+                                               false,
+                                               0,
+                                               n_faces,
+                                               NULL,
+                                               lstfac);
 
   /* Creation of the information structure for Code_Saturne/Code_Aster
      coupling */
 
   BFT_MALLOC(ast_coupling, 1, cs_ast_coupling_t);
 
-  ast_coupling->n_g_nodes = fvm_nodal_get_n_g_vertices(ifs_mesh);
+  n_vertices = fvm_nodal_get_n_entities(fsi_mesh, 0);
+  ast_coupling->n_vertices = n_vertices;
+  ast_coupling->n_g_vertices = fvm_nodal_get_n_g_vertices(fsi_mesh);
+
+  ast_coupling->n_faces = n_faces;
   ast_coupling->n_g_faces = n_faces;
 
-  BFT_MALLOC(ast_coupling->n_faces_by_domain, cs_glob_n_ranks, cs_int_t);
-  BFT_MALLOC(ast_coupling->n_nodes_by_domain, cs_glob_n_ranks, cs_int_t);
+  BFT_MALLOC(ast_coupling->s_vtx_num, ast_coupling->n_vertices, cs_lnum_t);
 
-  ast_coupling->n_nodes_by_domain[0] = n_nodes;
-  ast_coupling->n_faces_by_domain[0] = n_faces;
+  fvm_nodal_get_parent_num(fsi_mesh, 0, ast_coupling->s_vtx_num);
 
-
-  BFT_MALLOC(faces_color, n_faces, cs_int_t);
-  BFT_MALLOC(nodes_color, n_nodes, cs_int_t);
-  BFT_MALLOC(faces_coords, 3*n_faces, cs_real_t);
-  BFT_MALLOC(nodes_coords, 3*n_nodes, cs_real_t);
+  BFT_MALLOC(faces_color, n_faces, cs_lnum_t);
+  BFT_MALLOC(vertices_color, n_vertices, cs_lnum_t);
+  BFT_MALLOC(face_centers, 3*n_faces, cs_real_t);
+  BFT_MALLOC(vtx_coords, 3*n_vertices, cs_real_t);
 
   assert(sizeof(cs_coord_t)==sizeof(cs_real_t));
 
-  fvm_nodal_get_vertex_coords(ifs_mesh, CS_INTERLACE,
-                              (cs_coord_t *) nodes_coords);
+  fvm_nodal_get_vertex_coords(fsi_mesh, CS_INTERLACE,
+                              (cs_coord_t *)vtx_coords);
 
   for (j = 0; j < n_faces; j++) {
 
-    faces_coords[3*j]   = b_face_cog[3*(lstfac[j]-1)];
-    faces_coords[3*j+1] = b_face_cog[3*(lstfac[j]-1)+1];
-    faces_coords[3*j+2] = b_face_cog[3*(lstfac[j]-1)+2];
+    cs_lnum_t f_id = lstfac[j] - 1;
+    face_centers[3*j]   = b_face_cog[3*f_id];
+    face_centers[3*j+1] = b_face_cog[3*f_id+1];
+    face_centers[3*j+2] = b_face_cog[3*f_id+2];
 
     faces_color[j]      = idfast[j];
 
   }
 
-  for (j = 0; j < n_nodes; j++) {
+  for (j = 0; j < n_vertices; j++) {
+    vertices_color[j]   = idnast[j];
+  }
 
-    nodes_color[j]      = idnast[j];
+  /* In parallel, all YACS/Calcium I/O goes through rank 0;
+     This is about 1990's level technology/scalability, so a rewrite
+     (for example switch to PLE or MedCoupling) would be useful */
+
+#if defined(HAVE_MPI)
+
+  ast_coupling->face_p2b = NULL;
+  ast_coupling->vtx_b2p = NULL;
+
+  cs_gnum_t *s_vtx_gnum = NULL;
+  cs_part_to_block_t *vtx_p2b = NULL;
+
+  if (cs_glob_n_ranks > 1) {
+
+    /* For faces, which are not shared, use local data, as temporary
+       mesh access functions are either based on element type sections,
+       or ordered by such. */
+
+    cs_parall_counter(&(ast_coupling->n_g_faces), 1);
+
+    fvm_io_num_t *face_gnum
+      = fvm_io_num_create(lstfac,
+                          cs_glob_mesh->global_b_face_num,
+                          n_faces,
+                          0);
+    cs_gnum_t *s_face_gnum = fvm_io_num_transfer_global_num(face_gnum);
+    assert(ast_coupling->n_g_faces == fvm_io_num_get_global_count(face_gnum));
+
+    face_gnum = fvm_io_num_destroy(face_gnum);
+
+    cs_block_dist_info_t  face_bi;
+
+    face_bi = cs_block_dist_compute_sizes(cs_glob_rank_id,
+                                          cs_glob_n_ranks,
+                                          cs_glob_n_ranks, /* all on rank 0 */
+                                          0,
+                                          ast_coupling->n_g_faces);
+
+    ast_coupling->face_p2b
+      = cs_part_to_block_create_by_gnum(cs_glob_mpi_comm,
+                                        face_bi,
+                                        ast_coupling->n_faces,
+                                        s_face_gnum);
+    cs_part_to_block_transfer_gnum(ast_coupling->face_p2b, s_face_gnum);
+
+    /* For vertices, which may be shared, copy global numbering
+       associated with temporary mesh. */
+
+    BFT_MALLOC(s_vtx_gnum, ast_coupling->n_vertices, cs_gnum_t);
+    fvm_nodal_get_global_vertex_num(fsi_mesh, s_vtx_gnum);
+
+    cs_block_dist_info_t  vtx_bi;
+
+    vtx_bi = cs_block_dist_compute_sizes(cs_glob_rank_id,
+                                         cs_glob_n_ranks,
+                                         cs_glob_n_ranks, /* all on rank 0 */
+                                         0,
+                                         ast_coupling->n_g_vertices);
+
+    ast_coupling->vtx_b2p
+      = cs_block_to_part_create_by_gnum(cs_glob_mpi_comm,
+                                        vtx_bi,
+                                        ast_coupling->n_vertices,
+                                        s_vtx_gnum);
+
+    vtx_p2b = cs_part_to_block_create_by_gnum(cs_glob_mpi_comm,
+                                              vtx_bi,
+                                              ast_coupling->n_vertices,
+                                              s_vtx_gnum);
 
   }
 
-  ifs_mesh = fvm_nodal_destroy(ifs_mesh);
+#endif
 
-  /* TODO: Adapter l'algo parallel en sequentiel */
+  fsi_mesh = fvm_nodal_destroy(fsi_mesh);
 
   if (cs_glob_rank_id <= 0) {
 
-    cs_int_t *geom = NULL;
+    int sizes[2] = {ast_coupling->n_g_faces, ast_coupling->n_g_vertices};
 
-    BFT_MALLOC(geom,2,cs_int_t);
-
-    geom[0] = n_faces;
-    geom[1] = n_nodes;
-
-    cs_calcium_write_int(comp_id, time_dep, cur_time, 0,
-                         "DONGEO", 2, &(geom[0]));
-
-    BFT_FREE(geom);
+    /* For "milieu" */
 
     cs_calcium_write_double(comp_id, time_dep, cur_time, 0,
                             "ALMAXI", 1, almax);
 
-    cs_calcium_write_double(comp_id, time_dep, cur_time, 0,
-                            "COOFAC", 3*n_faces,&(faces_coords[0]));
-
-    cs_calcium_write_double(comp_id, time_dep, cur_time, 0,
-                            "COONOD", 3*n_nodes,&(nodes_coords[0]));
+    /* Directly for code_aster */
 
     cs_calcium_write_int(comp_id, time_dep, cur_time, 0,
-                         "COLFAC", n_faces, &(faces_color[0]));
+                         "NB_DYN", 1, &(sizes[1]));
 
     cs_calcium_write_int(comp_id, time_dep, cur_time, 0,
-                         "COLNOD", n_nodes, &(nodes_color[0]));
+                         "NB_FOR", 1, &(sizes[0]));
+
+
+  }
+
+#if defined(HAVE_MPI)
+
+  if (cs_glob_n_ranks > 1) {
+
+    cs_real_t *g_face_centers = NULL;
+    if (cs_glob_rank_id == 0)
+      BFT_MALLOC(g_face_centers, 3*ast_coupling->n_g_faces, cs_real_t);
+    cs_part_to_block_copy_array(ast_coupling->face_p2b,
+                                CS_REAL_TYPE,
+                                3,
+                                face_centers,
+                                g_face_centers);
+    if (cs_glob_rank_id == 0) {
+      cs_calcium_write_double(comp_id, time_dep, cur_time, 0, "COOFAC",
+                              3*ast_coupling->n_g_faces, g_face_centers);
+      BFT_FREE(g_face_centers);
+    }
+
+    cs_real_t *g_vtx_coords = NULL;
+    if (cs_glob_rank_id == 0)
+      BFT_MALLOC(g_vtx_coords, 3*ast_coupling->n_g_vertices, cs_real_t);
+    cs_part_to_block_copy_array(vtx_p2b,
+                                CS_REAL_TYPE,
+                                3,
+                                vtx_coords,
+                                g_vtx_coords);
+    if (cs_glob_rank_id == 0) {
+      cs_calcium_write_double(comp_id, time_dep, cur_time, 0, "COONOD",
+                              3*ast_coupling->n_g_vertices, g_vtx_coords);
+      BFT_FREE(g_vtx_coords);
+    }
+
+    int *g_face_color = NULL;
+    if (cs_glob_rank_id == 0)
+      BFT_MALLOC(g_face_color, ast_coupling->n_g_faces, int);
+    cs_part_to_block_copy_array(ast_coupling->face_p2b,
+                                CS_INT_TYPE,
+                                1,
+                                faces_color,
+                                g_face_color);
+    if (cs_glob_rank_id == 0) {
+      cs_calcium_write_int(comp_id, time_dep, cur_time, 0, "COLFAC",
+                           ast_coupling->n_g_faces, g_face_color);
+      BFT_FREE(g_face_color);
+    }
+
+    int *g_vtx_color = NULL;
+    if (cs_glob_rank_id == 0)
+      BFT_MALLOC(g_vtx_color, ast_coupling->n_g_vertices, int);
+    cs_part_to_block_copy_array(vtx_p2b,
+                                CS_INT_TYPE,
+                                1,
+                                vertices_color,
+                                g_vtx_color);
+    if (cs_glob_rank_id == 0) {
+      cs_calcium_write_int(comp_id, time_dep, cur_time, 0, "COLNOD",
+                           ast_coupling->n_g_vertices, g_vtx_color);
+      BFT_FREE(g_vtx_color);
+    }
+
+    cs_part_to_block_destroy(&vtx_p2b);
+    BFT_FREE(s_vtx_gnum);
+
+  }
+
+#endif
+
+  if (cs_glob_n_ranks == 1) {
+
+    cs_calcium_write_double(comp_id, time_dep, cur_time, 0,
+                            "COOFAC", 3*n_faces, face_centers);
+
+    cs_calcium_write_double(comp_id, time_dep, cur_time, 0,
+                            "COONOD", 3*n_vertices, vtx_coords);
+
+    cs_calcium_write_int(comp_id, time_dep, cur_time, 0,
+                         "COLFAC", n_faces, faces_color);
+
+    cs_calcium_write_int(comp_id, time_dep, cur_time, 0,
+                         "COLNOD", n_vertices, vertices_color);
 
   }
 
   cs_glob_ast_coupling = ast_coupling;
 
   BFT_FREE(faces_color);
-  BFT_FREE(nodes_color);
-  BFT_FREE(faces_coords);
-  BFT_FREE(nodes_coords);
+  BFT_FREE(vertices_color);
+  BFT_FREE(face_centers);
+  BFT_FREE(vtx_coords);
 }
 
 /*----------------------------------------------------------------------------
@@ -251,56 +394,43 @@ void CS_PROCF(astgeo, ASTGEO)
 
 void CS_PROCF(astfor, ASTFOR)
 (
- cs_int_t    *ntcast,
- cs_int_t    *nbfast,
- cs_real_t   *forast
+ cs_lnum_t    *ntcast,
+ cs_lnum_t    *nbfast,
+ cs_real_t    *forast
 )
 {
   cs_ast_coupling_t  *ast_coupling = cs_glob_ast_coupling;
 
-  const int  n_faces = *nbfast;
-  const int  n_g_faces = ast_coupling->n_g_faces;
+  const cs_lnum_t n_faces = *nbfast;
+  const cs_gnum_t  n_g_faces = ast_coupling->n_g_faces;
 
-  cs_real_t  *_forast = NULL;
+  cs_real_t  *g_forast = NULL;
 
-
-  if (cs_glob_rank_id <= 0)
-    BFT_MALLOC(_forast, 3*n_g_faces, cs_real_t);
-
-
-  if (cs_glob_n_ranks == 1) {
-
-    int  i;
-
-    assert(n_faces == n_g_faces);
-
-    for (i = 0; i < 3*n_g_faces; i++)
-      _forast[i] = forast[i];
-
-  }
+  if (cs_glob_rank_id < 1)
+    BFT_MALLOC(g_forast, 3*n_g_faces, cs_real_t);
 
 #if defined(HAVE_MPI)
 
-  if (cs_glob_n_ranks > 1) {
-
-    MPI_Gatherv(forast, 3*n_faces, CS_MPI_REAL,
-                _forast, ast_coupling->n_faces_by_domain,
-                ast_coupling->face_index, CS_MPI_REAL,
-                0, cs_glob_mpi_comm);
-
-  }
+  if (cs_glob_n_ranks > 1)
+    cs_part_to_block_copy_array(ast_coupling->face_p2b,
+                                CS_REAL_TYPE,
+                                3,
+                                forast,
+                                g_forast);
 
 #endif
 
-  if (cs_glob_rank_id <= 0) {
-
-    cs_calcium_write_double(comp_id, time_dep, cur_time, *ntcast,
-                            "FORSAT", 3*n_g_faces, _forast);
-
-    BFT_FREE(_forast);
-
+  if (cs_glob_n_ranks == 1) {
+    assert((cs_gnum_t)n_faces == n_g_faces);
+    for (cs_lnum_t i = 0; i < 3*n_faces; i++)
+      g_forast[i] = forast[i];
   }
 
+  if (cs_glob_rank_id < 1) {
+    cs_calcium_write_double(comp_id, time_dep, cur_time, *ntcast,
+                            "FORSAT", 3*n_g_faces, g_forast);
+    BFT_FREE(g_forast);
+  }
 }
 
 /*----------------------------------------------------------------------------
@@ -310,102 +440,70 @@ void CS_PROCF(astfor, ASTFOR)
 void CS_PROCF(astcin, ASTCIN)
 (
  cs_int_t    *ntcast,
- cs_int_t    *nbfast,
- cs_int_t    *lstfac,
- cs_real_3_t *depale
+ cs_real_3_t *disale
 )
 {
-  cs_int_t  i;
-  cs_int_t *parent_num      = NULL;
-
-  const cs_int_t local_rank = (cs_glob_rank_id == -1) ? 0 : cs_glob_rank_id;
+  cs_lnum_t  i;
 
   cs_ast_coupling_t  *ast_coupling = cs_glob_ast_coupling;
 
-  int  n_nodes;
-  const int  n_g_nodes = ast_coupling->n_g_nodes;
-  const int  n_faces = *nbfast;
+  const int  n_vertices = ast_coupling->n_vertices;
+  const int  n_g_vertices = ast_coupling->n_g_vertices;
 
-  fvm_nodal_t *mesh_ifs = NULL;
+  cs_real_t  *g_xast = NULL, *xast = NULL;
 
-  cs_real_t  *_xast = NULL, *xast = NULL;
-
-  n_nodes = ast_coupling->n_nodes_by_domain[local_rank];
-
-  BFT_MALLOC(xast, 3*n_nodes, cs_real_t);
+  BFT_MALLOC(xast, 3*n_vertices, cs_real_t);
 
   if (cs_glob_rank_id <= 0) {
 
     int  n_val_read = 0;
 
-    BFT_MALLOC(_xast, 3*n_g_nodes, cs_real_t);
+    BFT_MALLOC(g_xast, 3*n_g_vertices, cs_real_t);
 
     cs_calcium_read_double(comp_id, time_dep, &min_time, &max_time,
                            ntcast,
-                           "DEPSAT", 3*n_g_nodes,
-                           &n_val_read, _xast);
+                           "DEPSAT", 3*n_g_vertices,
+                           &n_val_read, g_xast);
 
-    assert(n_val_read == 3*n_g_nodes);
-
-  }
-
-  if (cs_glob_n_ranks == 1) {
-
-    assert(n_nodes == n_g_nodes);
-
-    for (i = 0; i < 3*n_nodes; i++)
-      xast[i] = _xast[i];
+    assert(n_val_read == 3*n_g_vertices);
 
   }
 
 #if defined(HAVE_MPI)
 
-  if (cs_glob_n_ranks > 1) {
-
-    MPI_Scatterv(_xast, ast_coupling->n_nodes_by_domain,
-                 ast_coupling->node_index, CS_MPI_REAL,
-                 xast, n_nodes,
-                 CS_MPI_REAL, 0, cs_glob_mpi_comm);
-
-  }
+  if (cs_glob_n_ranks > 1)
+    cs_block_to_part_copy_array(ast_coupling->vtx_b2p,
+                                CS_REAL_TYPE,
+                                3,
+                                g_xast,
+                                xast);
 
 #endif
 
+  if (cs_glob_n_ranks == 1) {
+    assert(n_vertices == n_g_vertices);
+    for (i = 0; i < 3*n_vertices; i++)
+      xast[i] = g_xast[i];
+  }
+
   if (cs_glob_rank_id <= 0)
-    BFT_FREE(_xast);
-
-  /* Creation maillage */
-
-  mesh_ifs = cs_mesh_connect_faces_to_nodal(cs_glob_mesh,
-                                            "MaillageExtraitAster_1",
-                                            false,
-                                            0,
-                                            n_faces,
-                                            NULL,
-                                            lstfac);
-
-  /* Tableau de numerotation */
-
-  BFT_MALLOC(parent_num, n_nodes, cs_int_t);
-  fvm_nodal_get_parent_num(mesh_ifs, 0, parent_num);
-
-  mesh_ifs = fvm_nodal_destroy(mesh_ifs);
+    BFT_FREE(g_xast);
 
   /* On remplit dans DEPALE les valeurs de deplacement aux noeuds
      couples a deplacement impose */
 
-  for (i = 0; i < n_nodes; i++) {
+  for (i = 0; i < n_vertices; i++) {
 
-    cs_lnum_t id_node_parent = parent_num[i] - 1;
+    cs_lnum_t parent_vtx_id = ast_coupling->s_vtx_num[i] - 1;
 
-    depale[id_node_parent][0] = xast[3*i];
-    depale[id_node_parent][1] = xast[3*i + 1];
-    depale[id_node_parent][2] = xast[3*i + 2];
+    disale[parent_vtx_id][0] = xast[3*i];
+    disale[parent_vtx_id][1] = xast[3*i + 1];
+    disale[parent_vtx_id][2] = xast[3*i + 2];
 
   }
 
   /* Liberation du(des) tableau(x) */
-  BFT_FREE(parent_num);
+  BFT_FREE(xast);
 }
 
 
@@ -431,41 +529,31 @@ void CS_PROCF(astpar, ASTPAR)
 
     /* Initialization of communication with calcium */
 
-    char *instance = NULL;
-
     double ttanc = 0.;
 
-    BFT_MALLOC(instance,200,char);
+    char instance[200];
 
     cs_calcium_connect(comp_id, instance);
 
-    BFT_FREE(instance);
-
-
     /* Reception des donnees */
 
     iteration = 0;
 
     /* commande reception nb iterations */
-    cs_calcium_read_int(comp_id, time_dep, &min_time, &max_time,
-                        &iteration,
+    cs_calcium_read_int(comp_id, time_dep, &min_time, &max_time, &iteration,
                         "NBPDTM", 1, &n_val_read, nbpdt);
     /* commande reception nb sous-it.  */
-    cs_calcium_read_int(comp_id, time_dep, &min_time, &max_time,
-                        &iteration,
-                        "NBSSIT",1,&n_val_read, nbsspdt);
+    cs_calcium_read_int(comp_id, time_dep, &min_time, &max_time, &iteration,
+                        "NBSSIT", 1, &n_val_read, nbsspdt);
     /* commande reception de la variable epsilo */
-    cs_calcium_read_double(comp_id, time_dep, &min_time, &max_time,
-                           &iteration,
-                           "EPSILO",1,&n_val_read, delta);
+    cs_calcium_read_double(comp_id, time_dep, &min_time, &max_time, &iteration,
+                           "EPSILO", 1, &n_val_read, delta);
     /* commande reception de la variable ttinit */
-    cs_calcium_read_double(comp_id, time_dep, &min_time, &max_time,
-                           &iteration,
-                           "TTINIT",1,&n_val_read, &ttanc);
+    cs_calcium_read_double(comp_id, time_dep, &min_time, &max_time, &iteration,
+                           "TTINIT", 1, &n_val_read, &ttanc);
     /* commande reception de la variable dtref */
-    cs_calcium_read_double(comp_id, time_dep, &min_time, &max_time,
-                           &iteration,
-                           "PDTREF",1,&n_val_read, dt);
+    cs_calcium_read_double(comp_id, time_dep, &min_time, &max_time, &iteration,
+                           "PDTREF", 1, &n_val_read, dt);
 
     if (fabs(*tt - ttanc) > 1.e-16)
       bft_error(__FILE__, __LINE__, 0,
@@ -491,14 +579,14 @@ void CS_PROCF(astpar, ASTPAR)
   bft_printf("@                                                          \n"
                "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n"
                "@                                                          \n"
-               "@ @@ ATTENTION : MODIFICATION DES PARAMETRES UTILISATEURS  \n"
+               "@ @@ ATTENTION: Change in user parameters                  \n"
                "@    *********                                             \n"
                "@                                                          \n"
-               "@    Presence du couplage Code_Saturne/Code_Aster :        \n"
-               "@    Les donnees rentrees dans l'outil 'Milieu'            \n"
-               "@    ecrasent les donnees rentrees par l'utilisateur       \n"
+               "@    In presence of coupling with code_aster, values       \n"
+               "@    provided in the 'Milieu' tool overwrite those         \n"
+               "@    in the base setup                                     \n"
                "@                                                          \n"
-               "@   Nouvelles valeurs:                                     \n"
+               "@   New values:                                            \n"
                "@      NTMABS = %i                                         \n"
                "@      NALIMX = %i                                         \n"
                "@      EPALIM = %f                                         \n"
@@ -520,7 +608,6 @@ void CS_PROCF(astpdt, ASTPDT)
   cs_int_t  *nbpdt
 )
 {
-  int        i;
   cs_real_t  dttmp = 0.;
 
   if (cs_glob_rank_id <= 0) {
@@ -530,13 +617,12 @@ void CS_PROCF(astpdt, ASTPDT)
 
     dttmp = dttab[0];
 
-    /* Send time step (calculated in "ddtvar" Subroutine) to'Milieu'*/
+    /* Send time step (calculated in "ddtvar" Subroutine) to 'Milieu'*/
     cs_calcium_write_double(comp_id, time_dep, 0.0, *nbpdt,
                             "DTSAT", 1, &dttmp);
 
     /* Receive time step sent by 'Milieu' */
-    cs_calcium_read_double(comp_id, time_dep, &min_time, &max_time,
-                           nbpdt,
+    cs_calcium_read_double(comp_id, time_dep, &min_time, &max_time, nbpdt,
                            "DTCALC", 1, &n_val_read, &(dtloc));
 
     assert(n_val_read == 1);
@@ -553,10 +639,9 @@ void CS_PROCF(astpdt, ASTPDT)
 #endif
 
 
-  for (i = 0; i < *ncelet; i++)
+  for (cs_lnum_t i = 0; i < *ncelet; i++)
     dttab[i] = dttmp;
 
-
   bft_printf("@                                                          \n"
                "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n"
                "@                                                          \n"
@@ -576,11 +661,9 @@ void CS_PROCF(astpdt, ASTPDT)
                "@  Valeur du pas de temps retenue pour le calcul couple:   \n"
                "@  dt = %f                                                 \n"
                "@                                                          \n"
-               "@                                                          \n"
                "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n"
                "@                                                          \n",
                dttmp);
-
 }
 
 /*----------------------------------------------------------------------------
@@ -613,7 +696,7 @@ void CS_PROCF(astcv1, ASTCV1)
 }
 
 /*----------------------------------------------------------------------------
- * Send global convergence value of IFS calculations
+ * Send global convergence value of FSI calculations
  * (internal and external structures)
  *----------------------------------------------------------------------------*/
 
@@ -631,6 +714,35 @@ void CS_PROCF(astcv2, ASTCV2)
   }
 }
 
+/*============================================================================
+ * Public function definitions
+ *============================================================================*/
+
+/*----------------------------------------------------------------------------
+ * Free coupling structure.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_ast_coupling_finalize(void)
+{
+  cs_ast_coupling_t  *ast_coupling = cs_glob_ast_coupling;
+
+#if defined(HAVE_MPI)
+
+  if (ast_coupling->vtx_b2p != NULL)
+    cs_block_to_part_destroy(&(ast_coupling->vtx_b2p));
+  if (ast_coupling->face_p2b != NULL)
+    cs_part_to_block_destroy(&(ast_coupling->face_p2b));
+
+#endif
+
+  BFT_FREE(ast_coupling->s_vtx_num);
+
+  BFT_FREE(ast_coupling);
+
+  cs_glob_ast_coupling = ast_coupling;
+}
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
