@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -66,6 +66,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*============================================================================
  * Local Macro Definitions
  *============================================================================*/
@@ -81,9 +83,13 @@ BEGIN_C_DECLS
 static FILE  *_bft_printf_file = NULL;
 
 /*============================================================================
- * Priototypes for Fortran subroutines
+ * Prototypes for Fortran subroutines
  *============================================================================*/
 
+/* Flush standard output */
+
+extern void cs_f_flush_logs(void);
+
 /*----------------------------------------------------------------------------
  * Print a message to standard output.
  *----------------------------------------------------------------------------*/
@@ -95,7 +101,7 @@ extern void CS_PROCF (csprnt, CSPRNT)
 );
 
 /*----------------------------------------------------------------------------
- * Initialize Fortran log (listing) files
+ * Initialize Fortran log files
  *----------------------------------------------------------------------------*/
 
 extern void CS_PROCF (csopli, CSOPLI)
@@ -106,7 +112,7 @@ extern void CS_PROCF (csopli, CSOPLI)
 );
 
 /*----------------------------------------------------------------------------
- * Close log (listing) handled by Fortran: (CLose LIsting)
+ * Close log handled by Fortran: (CLose LIsting)
  *----------------------------------------------------------------------------*/
 
 extern void CS_PROCF (csclli, CSCLLI)
@@ -194,8 +200,9 @@ _bft_printf_flush(void)
 {
   if (_bft_printf_file != NULL)
     return fflush(_bft_printf_file);
-  else
-    return 0;
+  else if (bft_printf_proxy_get() == _bft_printf_f)
+    cs_f_flush_logs();
+  return 0;
 }
 
 /*----------------------------------------------------------------------------
@@ -205,12 +212,14 @@ _bft_printf_flush(void)
 static void
 _close_c_log_file(void)
 {
-  if (_bft_printf_file != NULL) {
+  if (_bft_printf_file != NULL && _bft_printf_file != stdout) {
     fclose(_bft_printf_file);
     _bft_printf_file = NULL;
   }
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions for Fortran API
  *============================================================================*/
@@ -338,35 +347,6 @@ void CS_PROCF (cserf, CSERF)
 #endif
 }
 
-/*----------------------------------------------------------------------------
- * Copy a Fortan string buffer to a C string buffer
- *
- * The aim of this function is to aviod issues with Fortran array bounds
- * checking when compilers such as icc 11 consider a character array from C
- * as an array of 1-character length strings.
- *
- * Fortran interface
- *
- * subroutine cssf2c (len, cstr, fstr)
- * *****************
- *
- * integer          len         : <-- : String length
- * character*       fstr        : <-- : Fortran string
- * character*       cstr        : --> : C string
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (cssf2c, CSSF2C)
-(
- const cs_int_t   *len,
- const char       *fstr,
- char             *cstr
- CS_ARGF_SUPP_CHAINE              /*     (possible 'length' arguments added
-                                         by many Fortran compilers) */
-)
-{
-  memcpy(cstr, fstr, *len);
-}
-
 /*----------------------------------------------------------------------------
  * Get log name file information.
  *
@@ -468,17 +448,14 @@ void CS_PROCF (csdatadir, CSDATADIR)
  *
  * parameters:
  *   log_name    <-- base file name for log, or NULL for stdout
- *   r0_log_flag <-- redirection for rank 0 log;
- *                   0: not redirected; 1: redirected to <log_name> file
  *   rn_log_flag <-- redirection for ranks > 0 log:
- *                   0: not redirected; 1: redirected to <log_name>_n*" file;
- *                   2: redirected to "/dev/null" (suppressed)
+ *                   false:  to "/dev/null" (suppressed)
+ *                   true: redirected to <log_name>_n*.log" file;
  *----------------------------------------------------------------------------*/
 
 void
 cs_base_fortran_bft_printf_set(const char  *log_name,
-                               int          r0_log_flag,
-                               int          rn_log_flag)
+                               bool         rn_log_flag)
 {
   const char *name = NULL;
   bool suppress = false;
@@ -486,13 +463,24 @@ cs_base_fortran_bft_printf_set(const char  *log_name,
 
   /* C output */
 
-  cs_base_bft_printf_init(log_name, r0_log_flag, rn_log_flag);
+  cs_base_bft_printf_init(log_name, rn_log_flag);
 
   name = cs_base_bft_printf_name();
   suppress = cs_base_bft_printf_suppressed();
 
   if (suppress == false) {
 
+    /* Allow bypassing this with environment variable to accomodate
+       some debug habits */
+
+    const char *p = getenv("CS_LOG_TO_STDOUT");
+    if (p != NULL) {
+      if (atoi(p) > 0)
+        name = NULL;
+    }
+
+    /* In standard case, redirect if possible */
+
     if (name != NULL) {
 
       _bft_printf_file = fopen(name, "w");
@@ -620,6 +608,46 @@ cs_base_fortran_bft_printf_to_f(void)
   ple_printf_function_set(_bft_printf_f);
 }
 
+/*----------------------------------------------------------------------------
+ * Wrappers to cs_user_*
+ *----------------------------------------------------------------------------*/
+
+void
+cs_user_extra_operations_initialize_wrapper(void)
+{
+  cs_user_extra_operations_initialize(cs_glob_domain);
+}
+
+void
+cs_user_physical_properties_wrapper(void)
+{
+  cs_user_physical_properties(cs_glob_domain);
+}
+
+void
+cs_user_extra_operations_wrapper(void)
+{
+  cs_user_extra_operations(cs_glob_domain);
+}
+
+void
+cs_user_initialization_wrapper(void)
+{
+  cs_user_initialization(cs_glob_domain);
+}
+
+void
+cs_user_parameters_wrapper(void)
+{
+  cs_user_parameters(cs_glob_domain);
+}
+
+void
+cs_user_finalize_setup_wrapper(void)
+{
+  cs_user_finalize_setup(cs_glob_domain);
+}
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
