@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -139,32 +139,6 @@ void CS_PROCF (cserf, CSERF)
  cs_real_t        *ferf
  );
 
-/*----------------------------------------------------------------------------
- * Copy a Fortan string buffer to a C string buffer
- *
- * The aim of this function is to aviod issues with Fortran array bounds
- * checking when compilers such as icc 11 consider a character array from C
- * as an array of 1-character length strings.
- *
- * Fortran interface
- *
- * subroutine cssf2c (len, cstr, fstr)
- * *****************
- *
- * integer          len         : <-- : String length
- * character*       fstr        : <-- : Fortran string
- * character*       cstr        : --> : C string
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (cssf2c, CSSF2C)
-(
- const cs_int_t   *len,
- const char       *fstr,
- char             *cstr
- CS_ARGF_SUPP_CHAINE              /*     (possible 'length' arguments added
-                                         by many Fortran compilers) */
-);
-
 /*----------------------------------------------------------------------------
  * Get log name file information.
  *
@@ -226,17 +200,14 @@ void CS_PROCF (csdatadir, CSDATADIR)
  *
  * parameters:
  *   log_name    <-- base file name for log, or NULL for stdout
- *   r0_log_flag <-- redirection for rank 0 log;
- *                   0: not redirected; 1: redirected to <log_name> file
  *   rn_log_flag <-- redirection for ranks > 0 log:
- *                   0: not redirected; 1: redirected to <log_name>_n*" file;
- *                   2: redirected to "/dev/null" (suppressed)
+ *                   false:  to "/dev/null" (suppressed)
+ *                   true: redirected to <log_name>_n*.log" file;
  *----------------------------------------------------------------------------*/
 
 void
 cs_base_fortran_bft_printf_set(const char  *log_name,
-                               int          r0_log_flag,
-                               int          rn_log_flag);
+                               bool         rn_log_flag);
 
 /*----------------------------------------------------------------------------
  * Switch bft_printf() mechanism to C output.
@@ -256,6 +227,25 @@ cs_base_fortran_bft_printf_to_c(void);
 void
 cs_base_fortran_bft_printf_to_f(void);
 
+/*----------------------------------------------------------------------------
+ * Wrappers for Fortran C
+ *----------------------------------------------------------------------------*/
+
+void
+cs_user_extra_operations_initialize_wrapper(void);
+
+void
+cs_user_extra_operations_wrapper(void);
+
+void
+cs_user_initialization_wrapper(void);
+
+void
+cs_user_parameters_wrapper(void);
+
+void
+cs_user_finalize_setup_wrapper(void);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
