@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -32,9 +32,12 @@
  *----------------------------------------------------------------------------*/
 
 #include "cs_base.h"
+#include "cs_domain.h"
 #include "cs_mesh.h"
 #include "cs_mesh_quantities.h"
 #include "cs_mesh_bad_cells.h"
+#include "cs_probe.h"
+#include "cs_volume_zone.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -57,6 +60,20 @@ extern void CS_PROCF (caltri, CALTRI)
  void
 );
 
+/*----------------------------------------------------------------------------
+ * Convert gas temperature to and from enthalpy based on concentrations
+ *----------------------------------------------------------------------------*/
+
+extern void CS_PROCF (cpthp1, CPTHP1)
+(
+ const cs_int_t   *mode,    /* <-- 1: h to t, 2: t to h */
+ cs_real_t        *eh,      /* <-> enthalpy of gas mix */
+ cs_real_t        *xesp,    /* <-- mas fraction of species */
+ cs_real_t        *f1mc,    /* <-- mean f1 */
+ cs_real_t        *f2mc,    /* <-- mean f2 */
+ cs_real_t        *tp       /* <-- gas temperature (K) */
+);
+
 /*----------------------------------------------------------------------------
  * Initialize Fortran base common block values
  *----------------------------------------------------------------------------*/
@@ -67,6 +84,16 @@ extern void CS_PROCF (csinit, CSINIT)
  const cs_int_t  *nrgpar   /* <-- Number of MPI processes, or 1 */
 );
 
+/*----------------------------------------------------------------------------
+ * Compute distance to wall by solving a 3d diffusion equation.
+ *----------------------------------------------------------------------------*/
+
+extern void CS_PROCF (distpr, DISTPR)
+(
+ const cs_int_t  *itypfb,    /* <-- boudnary face types */
+ cs_real_t       *distpa     /* <-- wall distance */
+);
+
 /*----------------------------------------------------------------------------
  * Developer function for output of variables on a post-processing mesh
  *----------------------------------------------------------------------------*/
@@ -77,15 +104,10 @@ extern void CS_PROCF (dvvpst, DVVPST)
  const cs_int_t  *numtyp,    /* <-- number or post-processing type
                               *     (-1 as volume, -2 as boundary, or nummai) */
  const cs_int_t  *nvar,      /* <-- number of variables */
- const cs_int_t  *nscal,     /* <-- number of scalars */
- const cs_int_t  *nvlsta,    /* <-- number of statistical variables (lagr) */
- const cs_int_t  *nvisbr,    /* <-- number of boundary stat. variables (lagr) */
  const cs_int_t  *ncelps,    /* <-- number of post-processed cells */
  const cs_int_t  *nfbrps,    /* <-- number of post processed boundary faces */
  const cs_int_t   lstcel[],  /* <-- list of post-processed cells */
  const cs_int_t   lstfbr[],  /* <-- list of post-processed boundary faces */
- const cs_real_t  rtp[],     /* <-- cell variables */
- const cs_real_t  propce[],  /* <-- cell physical properties */
  cs_real_t        tracel[],  /* --- work array for output cells */
  cs_real_t        trafbr[]   /* --- work array for output boundary faces */
 );
@@ -106,16 +128,6 @@ extern void CS_PROCF (findpt, FINDPT)
        cs_int_t   *ndrang    /* --> rank of associated process */
 );
 
-/*----------------------------------------------------------------------------
- * Generator for distribution function of p's
- *----------------------------------------------------------------------------*/
-
-extern void CS_PROCF (fische, FISCHE)
-(
- const cs_int_t   *n,
- const cs_real_t  *mu,
-       cs_int_t    p[]);
-
 /*----------------------------------------------------------------------------
  * Check necessity of extended mesh from FORTRAN options.
  *
@@ -140,120 +152,105 @@ extern void CS_PROCF (initi1, INITI1)
 );
 
 /*----------------------------------------------------------------------------
- * Free Fortran allocated memory
+ * Set the CDO mode in the FORTRAN part
  *----------------------------------------------------------------------------*/
 
-extern void CS_PROCF (memfin, MEMFIN) (void);
-
-/*----------------------------------------------------------------------------
- * Mesh renumbering for vector processors
- *----------------------------------------------------------------------------*/
-
-extern void CS_PROCF (numvec, NUMVEC)
+extern void CS_PROCF (set_cdo_mode, SET_CDO_MODE)
 (
- const cs_int_t  *ncelet,   /* <-- number of extended (real + ghost) cells */
- const cs_int_t  *ncel,     /* <-- number of local cells */
- const cs_int_t  *nfac,     /* <-- number of interior faces */
- const cs_int_t  *nfabor,   /* <-- number of boundary faces */
- const cs_int_t  *lregis,   /* <-- vector registor length */
- cs_int_t        *irveci,   /* <-> interior face vectorization indic. */
- cs_int_t        *irvecb,   /* <-> boundary face vectorization indic. */
- cs_int_t         ifacel[], /* <-- interior face->cell connectivity */
- cs_int_t         ifabor[], /* <-- boundary face->cell connectivity */
- cs_int_t         inumfi[], /* <-> interior faces renumbering (size: nfac) */
- cs_int_t         inumfb[], /* <-> boundary faces renumbering (size: nfabor) */
- cs_int_t         iworkf[], /* --- work array, size: max(nfac, nfabor) */
- cs_int_t         ismbs[]   /* --- work array, size: ncelet */
+ const cs_int_t   *mode     /* <-- -1: no CDO, 1: with CDO, 2: CDO only */
 );
 
 /*----------------------------------------------------------------------------
- * Test renumbering for vector processors
+ * User function for enthalpy <-> temperature conversion
  *----------------------------------------------------------------------------*/
 
-extern void CS_PROCF (tstvec, TSTVEC)
+void CS_PROCF (usthht, USTHHT)
 (
- const cs_int_t  *ncelet,   /* <-- number of extended (real + ghost) cells */
- const cs_int_t  *ncel,     /* <-- number of local cells */
- const cs_int_t  *nfac,     /* <-- number of interior faces */
- const cs_int_t  *nfabor,   /* <-- number of boundary faces */
- const cs_int_t   ifacel[], /* <-- interior face->cell connectivity */
- const cs_int_t   ifabor[], /* <-- boundary face->cell connectivity */
- cs_int_t         iworkf[], /* --- work array, size: max(nfac, nfabor) */
- cs_int_t         ismbs[],  /* --- work array, size: ncelet */
- cs_int_t         ismbv[],  /* --- work array, size: ncelet */
- cs_real_t        rworkf[], /* --- work array, size: max(nfac, nfabor) */
- cs_real_t        rsmbs[],  /* --- work array, size: ncelet */
- cs_real_t        rsmbv[]   /* --- work array, size: ncelet */
+ const cs_int_t  *mode,      /* <-- -1 : t -> h ; 1 : h -> t */
+ cs_real_t       *enthal,    /* <-- enthalpy */
+ cs_real_t       *temper     /* <-- temperature */
 );
 
-/*----------------------------------------------------------------------------
- * User function for output of variables on a post-processing mesh
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (usvpst, USVPST)
-(
- const cs_int_t  *nummai,    /* <-- number or post-processing mesh */
- const cs_int_t  *nvar,      /* <-- number of variables */
- const cs_int_t  *nscal,     /* <-- number of scalars */
- const cs_int_t  *nvlsta,    /* <-- number of statistical variables (lagr) */
- const cs_int_t  *ncelps,    /* <-- number of post-processed cells */
- const cs_int_t  *nfacps,    /* <-- number of post processed interior faces */
- const cs_int_t  *nfbrps,    /* <-- number of post processed boundary faces */
- const cs_int_t   itypps[3], /* <-- flag (0 or 1) for presence of cells, */
-                             /*     interior faces, and boundary faces */
- const cs_int_t   lstcel[],  /* <-- list of post-processed cells */
- const cs_int_t   lstfac[],  /* <-- list of post-processed interior faces */
- const cs_int_t   lstfbr[],  /* <-- list of post-processed boundary faces */
- const cs_real_t  dt[]       /* <-- local time step */
-);
-
-/*----------------------------------------------------------------------------
- * Uniform random number generator
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (zufall, zufall)
-(
- const cs_int_t   *n,             /* --> size of the vector */
- const cs_real_t  *a              /* <-- generated random number vector */
-);
-
-/*----------------------------------------------------------------------------
- * Gaussian random number generator
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (normalen, normalen)
-(
- const cs_int_t   *n,             /* --> size of the vector */
- const cs_real_t  *x              /* <-- generated random number vector */
-);
-
-/*----------------------------------------------------------------------------
- * Initialize Lagrangian module parameters for a given zone and class
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Compute GUI-defined head losses for a given volume zone.
  *
- * parameters:
- *   i_cz_params <-- integer parameters for this class and zone
- *   r_cz_params <-- real parameters for this class and zone
+ * Head loss tensor coefficients for each cell are organized as follows:
+ * cku11, cku22, cku33, cku12, cku13, cku23.
+ *
+ * \param[in]       zone  pointer to zone structure
+ * \param[in, out]  cku   head loss coefficients
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_user_head_losses(const cs_zone_t  *zone,
+                    cs_real_t         cku[][6]);
+
+/*----------------------------------------------------------------------------
+ * Absorption coefficient for radiative module
  *----------------------------------------------------------------------------*/
 
 void
-cs_lagr_init_zone_class_param(const cs_int_t   i_cs_params[],
-                              const cs_real_t  r_cs_params[]);
+cs_user_rad_transfer_absorption(const int         bc_type[],
+                                const cs_real_t   dt[],
+                                cs_real_t         ck[]);
 
 /*----------------------------------------------------------------------------
- * Define Lagrangian module parameters for a given zone and class
- *
- * parameters:
- *   class_id    <-- id of given particle class
- *   zone_id     <-- id of given boundary zone
- *   i_cz_params <-- integer parameters for this class and zone
- *   r_cz_params <-- real parameters for this class and zone
+ * Compute the net radiation flux
  *----------------------------------------------------------------------------*/
 
 void
-cs_lagr_define_zone_class_param(cs_int_t         class_id,
-                                cs_int_t         zone_id,
-                                const cs_int_t   i_cs_params[],
-                                const cs_real_t  r_cs_params[]);
+cs_user_rad_transfer_net_flux(const int        itypfb[],
+                              const cs_real_t  dt[],
+                              const cs_real_t  coefap[],
+                              const cs_real_t  coefbp[],
+                              const cs_real_t  cofafp[],
+                              const cs_real_t  cofbfp[],
+                              const cs_real_t  twall[],
+                              const cs_real_t  qincid[],
+                              const cs_real_t  xlam[],
+                              const cs_real_t  epa[],
+                              const cs_real_t  eps[],
+                              const cs_real_t  ck[],
+                              cs_real_t        net_flux[]);
+
+/*----------------------------------------------------------------------------
+ * Convert temperature to enthalpy at boundary
+ *----------------------------------------------------------------------------*/
+
+void CS_PROCF (b_t_to_h, b_t_to_h)
+(
+ const cs_lnum_t *nlst,          /* --> number of faces in list */
+ const cs_lnum_t *lstfac,        /* --> list of boundary faces at which
+                                    conversion is requested */
+ const cs_real_t *t_b,           /* --> temperature at boundary */
+ cs_real_t       *h_b            /* --> enthalpy at boundary */
+);
+
+/*----------------------------------------------------------------------------
+ * Convert enthalpy to temperature at cells
+ *----------------------------------------------------------------------------*/
+
+void CS_PROCF (c_h_to_t, c_h_to_t)
+(
+ const cs_real_t *h,           /* --> enthalpy */
+ cs_real_t       *t            /* --> temperature */
+);
+
+/*----------------------------------------------------------------------------
+ * Add field indexes associated with a new non-user solved variable,
+ * with default options
+ *
+ * parameters:
+ *   f_id <--   field id
+ *
+ * returns:
+ *   scalar number for defined field
+ *----------------------------------------------------------------------------*/
+
+int
+cs_add_model_field_indexes(int f_id);
 
 /*----------------------------------------------------------------------------
  * Return Lagrangian model status.
@@ -273,6 +270,14 @@ cs_lagr_status(int  *model_flag,
  *  User function prototypes
  *============================================================================*/
 
+/*----------------------------------------------------------------------------
+ * Data Entry of the 1D wall thermal module.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_user_1d_wall_thermal(int iappel,
+                        int isuit1);
+
 /*----------------------------------------------------------------------------
  * Define global options for couplings.
  *
@@ -284,14 +289,133 @@ void
 cs_user_coupling(void);
 
 /*----------------------------------------------------------------------------
- * This function is called at the end of each time step.
- *
- * It and has a very general purpose, although it is recommended to handle
- * mainly postprocessing or data-extraction type operations.
+ * This function is called at each time step for boundary conditions.
  *----------------------------------------------------------------------------*/
 
 void
-cs_user_extra_operations(void);
+cs_user_boundary_conditions(int         nvar,
+                            int         icodcl[],
+                            int         bc_type[],
+                            cs_real_t   rcodcl[]);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Initialize variables.
+ *
+ * This function is called at beginning of the computation
+ * (restart or not) before the time step loop.
+ *
+ * This is intended to initialize or modify (when restarted)
+ * variable and time step values.
+
+ * \param[in, out]  domain   pointer to a cs_domain_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_user_extra_operations_initialize(cs_domain_t     *domain);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief This function is called at the end of each time step.
+ *
+ * It has a very general purpose, although it is recommended to handle
+ * mainly postprocessing or data-extraction type operations.
+ *
+ * \param[in, out]  domain   pointer to a cs_domain_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_user_extra_operations(cs_domain_t     *domain);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief This function is called at the end of the calculation.
+ *
+ * It has a very general purpose, although it is recommended to handle
+ * mainly postprocessing or data-extraction type operations.
+
+ * \param[in, out]  domain   pointer to a cs_domain_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_user_extra_operations_finalize(cs_domain_t     *domain);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief This function is called one time step to initialize problem.
+ *
+ * \param[in, out]  domain   pointer to a cs_domain_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_user_initialization(cs_domain_t     *domain);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define internal coupling options.
+ *
+ * Options are usually defined using cs_internal_coupling_add_entity.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_user_internal_coupling(void);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define volumes as internal coupling zones.
+ *
+ * These zones will be separated from the rest of the domain using automatically
+ * defined thin walls.
+ *
+ * \param[in, out] mesh  pointer to a cs_mesh_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_user_internal_coupling_add_volumes(cs_mesh_t  *mesh);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define volumesi from separated meshes as internal coupling zones.
+ *
+ * These zones must be disjoint and the face selection criteria must be
+ * specified.
+ *
+ * \param[in, out]  mesh  pointer to a cs_mesh_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_user_internal_coupling_from_disjoint_meshes(cs_mesh_t  *mesh);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief This function is called each time step to define physical properties.
+ *
+ * \param[in, out]   domain    pointer to a cs_domain_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_user_physical_properties(cs_domain_t  *domain);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Compute the porosity (volume factor \f$ \epsilon \f$
+ *        when the porosity model is activated
+ *        (iporos greater than 1 in cs_user_parameters.f90).
+ *
+ * This function is called at the begin of the simulation only.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_user_porosity(void);
 
 /*----------------------------------------------------------------------------
  * Define mesh joinings.
@@ -300,6 +424,27 @@ cs_user_extra_operations(void);
 void
 cs_user_join(void);
 
+/*----------------------------------------------------------------------------
+ * Define linear solver options.
+ *
+ * This function is called at the setup stage, once user and most model-based
+ * fields are defined.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_user_linear_solvers(void);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define or modify output user parameters.
+ * For CDO schemes, specify the elements such as properties, advection fields,
+ * user-defined equations and modules which have been previously added.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_user_finalize_setup(cs_domain_t *domain);
+
 /*----------------------------------------------------------------------------
  * Tag bad cells within the mesh based on geometric criteria.
  *----------------------------------------------------------------------------*/
@@ -323,11 +468,11 @@ void
 cs_user_mesh_modify(cs_mesh_t  *mesh);
 
 /*----------------------------------------------------------------------------
- * Insert thin wall into a mesh.
+ * Insert boundary wall into a mesh.
  *----------------------------------------------------------------------------*/
 
 void
-cs_user_mesh_thinwall(cs_mesh_t  *mesh);
+cs_user_mesh_boundary(cs_mesh_t  *mesh);
 
 /*----------------------------------------------------------------------------
  * Mesh smoothing.
@@ -361,6 +506,15 @@ cs_user_mesh_save(cs_mesh_t  *mesh);
 void
 cs_user_mesh_warping(void);
 
+/*----------------------------------------------------------------------------
+ * Select physical model options, including user fields.
+ *
+ * This function is called at the earliest stages of the data setup.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_user_model(void);
+
 /*----------------------------------------------------------------------------
  * Define advanced mesh numbering options.
  *----------------------------------------------------------------------------*/
@@ -389,6 +543,48 @@ cs_user_partition(void);
 void
 cs_user_matrix_tuning(void);
 
+/*----------------------------------------------------------------------------
+ * Define or modify general numerical and physical user parameters.
+ *
+ * At the calling point of this function, most model-related most variables
+ * and other fields have been defined, so specific settings related to those
+ * fields may be set here.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_user_parameters(cs_domain_t *domain);
+
+/*----------------------------------------------------------------------------
+ * User function for input of radiative transfer module options.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_user_radiative_transfer_parameters(void);
+
+/*-----------------------------------------------------------------------------
+ * User subroutine for input of radiative transfer boundary conditions
+ *----------------------------------------------------------------------------*/
+
+void
+cs_user_radiative_transfer_bcs(int               nvar,
+                               const int         bc_type[],
+                               int               icodcl[],
+                               int               isothp[],
+                               cs_real_t        *tmin,
+                               cs_real_t        *tmax,
+                               cs_real_t        *tx,
+                               const cs_real_t   dt[],
+                               cs_real_t         rcodcl[],
+                               const cs_real_t   thwall[],
+                               const cs_real_t   qincid[],
+                               cs_real_t         hfcnvp[],
+                               cs_real_t         flcnvp[],
+                               cs_real_t         xlamp[],
+                               cs_real_t         epap[],
+                               cs_real_t         epsp[],
+                               cs_real_t         textp[],
+                               cs_real_t         tintp[]);
+
 /*----------------------------------------------------------------------------
  * Define periodic faces.
  *----------------------------------------------------------------------------*/
@@ -408,6 +604,13 @@ cs_user_periodicity(void);
 void
 cs_user_postprocess_writers(void);
 
+/*-----------------------------------------------------------------------------
+ * Define monitoring probes and profiles. A profile is seen as a set of probes.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_user_postprocess_probes(void);
+
 /*----------------------------------------------------------------------------
  * Define post-processing meshes.
  *
@@ -419,6 +622,25 @@ cs_user_postprocess_writers(void);
 void
 cs_user_postprocess_meshes(void);
 
+/*----------------------------------------------------------------------------
+ * User function for output of values on a post-processing mesh.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_user_postprocess_values(const char            *mesh_name,
+                           int                    mesh_id,
+                           int                    cat_id,
+                           cs_probe_set_t        *probes,
+                           cs_lnum_t              n_cells,
+                           cs_lnum_t              n_i_faces,
+                           cs_lnum_t              n_b_faces,
+                           cs_lnum_t              n_vertices,
+                           const cs_lnum_t        cell_list[],
+                           const cs_lnum_t        i_face_list[],
+                           const cs_lnum_t        b_face_list[],
+                           const cs_lnum_t        vertex_list[],
+                           const cs_time_step_t  *ts);
+
 /*----------------------------------------------------------------------------
  * Override default frequency or calculation end based output.
  *
@@ -464,6 +686,13 @@ cs_user_solver(const cs_mesh_t             *mesh,
 void
 cs_user_syrthes_coupling(void);
 
+/*----------------------------------------------------------------------------
+ * Define time moments.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_user_time_moments(void);
+
 /*----------------------------------------------------------------------------
  * Define rotor/stator model.
  *----------------------------------------------------------------------------*/
@@ -478,6 +707,139 @@ cs_user_turbomachinery(void);
 void
 cs_user_turbomachinery_rotor(void);
 
+/*----------------------------------------------------------------------------
+ * Define rotation velocity of rotor.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_user_turbomachinery_set_rotation_velocity(void);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define volume and surface zones.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_user_zones(void);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Define scaling parameter for electric model
+*/
+/*----------------------------------------------------------------------------*/
+
+void
+cs_user_scaling_elec(const cs_mesh_t             *mesh,
+                     const cs_mesh_quantities_t  *mesh_quantities,
+                     cs_real_t                   *dt);
+
+/*----------------------------------------------------------------------------
+ * Computation of the relaxation time-scale to equilibrium in the frame of
+ * the homogeneous two-phase model.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_user_hgn_thermo_relax_time(const cs_mesh_t *mesh,
+                              const cs_real_t *alpha_eq,
+                              const cs_real_t *y_eq,
+                              const cs_real_t *z_eq,
+                              const cs_real_t *ei,
+                              const cs_real_t *v,
+                              cs_real_t       *relax_tau);
+
+/*============================================================================
+ *  CDO User function prototypes
+ *============================================================================*/
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Specify for each soil and tracer how is defined each term of the
+ *         the tracer equation. Soils and tracer equations have to be added
+ *         previously
+ *
+ * \param[in, out]   domain    pointer to a cs_domain_t structure
+*/
+/*----------------------------------------------------------------------------*/
+
+void
+cs_user_gwf_setup(cs_domain_t   *domain);
+
+/*----------------------------------------------------------------------------*/
+
+/*============================================================================
+ *  MEG function prototypes
+ *============================================================================*/
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \file cs_meg_boundary_function.c
+ *
+ * \brief This function is used to compute user defined values for fields over a
+ * given boundary zone
+ *
+ * \param[in]  zone         pointer to cs_zone_t structure related to boundary
+ * \param[in]  field_name   name of the field (const char *)
+ * \param[in]  condition    condition type (const char *)
+ *
+ * \return a pointer to an array of cs_real_t values
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_real_t *
+cs_meg_boundary_function(const cs_zone_t  *zone,
+                         const char       *field_name,
+                         const char       *condition);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief This function is used to compute user defined values for fields over a
+ *        given volume zone
+ *
+ * \param[in]       zone        pointer to cs_zone_t structure related to a volume
+ * \param[in, out]  f[]         array of pointers to cs_field_t
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_meg_volume_function(const cs_zone_t  *zone,
+                       cs_field_t       *f[]);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Evaluate GUI defined mathematical expressions over volume zones for
+ *         initialization.
+ *
+ * \param[in]   zone  pointer to a cs_volume_zone_t structure
+ * \param[in]   f     char pointer: variable name
+ *
+*/
+/*----------------------------------------------------------------------------*/
+
+cs_real_t *
+cs_meg_initialization(const cs_zone_t *zone,
+                      const char      *field_name);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \file cs_meg_source_terms.c
+ *
+ * \brief This function is used to compute source terms over a volume zone
+ *
+ * \param[in]       zone         pointer to cs_volume_zone_t
+ * \param[in]       name         char pointer: variable name
+ * \param[in]       source_type  char pointer: source term type
+ *
+ * \returns new_vals: a cs_real_t pointer containing the values
+ *
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_real_t *
+cs_meg_source_terms(const cs_zone_t  *zone,
+                    const char       *name,
+                    const char       *source_type);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
