@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -63,6 +63,8 @@ use ihmpre
 use parall
 use period
 use mesh
+use post
+use field
 
 !===============================================================================
 
@@ -74,6 +76,7 @@ double precision dt(ncelet)
 
 ! Local variables
 
+integer          n_fields, f_id, flag
 integer          ifac  , istr, icompt, ii
 integer          mbstru, mbaste
 
@@ -120,18 +123,15 @@ do ifac = 1, nfabor
 enddo
 
 !===============================================================================
-! 2.  REMPLISSAGE DE IDFSTR PAR L'UTILISATEUR
+! 2.  User definition of idfstr
 !===============================================================================
 
-! 2.1 STRUCTURES INTERNES :
-! -----------------------
+! Internal structures
 
 if (iihmpr.eq.1) then
 
   call uistr1 &
-  !==========
-( nfabor,                  &
-  idfstr, mbstru,          &
+( idfstr, mbstru,          &
   aexxst, bexxst, cfopre,  &
   ihistr,                  &
   xstp, xstreq, xpstr )
@@ -139,18 +139,14 @@ if (iihmpr.eq.1) then
 endif
 
 call usstr1                                                       &
-!==========
  ( idfstr ,                                                       &
    aexxst , bexxst , cfopre ,                                     &
    xstp   , xpstr  , xstreq )
 
-! 2.2 STRUCTURES EXTERNES : COUPLAGE CODE_SATURNE / CODE_ASTER
-! -----------------------
-
-call usaste                                                       &
-!==========
- ( idfstr )
+! External structures: Code_Saturne / Code_Aster coupling
 
+call uiaste(idfstr, asddlf)
+call usaste(idfstr)
 
 !===============================================================================
 ! 3.  CALCUL DE NBSTRU ET NBASTE
@@ -266,14 +262,10 @@ if (nbaste.gt.0) then
   ! Free memory
 
 !       Recuperation des parametres commun du couplage
-  call astpar                                                     &
-  !==========
- ( ntmabs, nalimx, epalim, ttpabs, dtref )
+  call astpar(ntmabs, nalimx, epalim, ttpabs, dtref)
 
 !       Envoi des donnees geometriques a Code_Aster
-  call astgeo                                                     &
-  !==========
- ( nbfast, nbnast, lstfac, idfloc, idnloc, almax )
+  call astgeo(nbfast, lstfac, idfloc, idnloc, almax)
 
   ! Free memory
   deallocate(lstfac)
@@ -289,11 +281,14 @@ endif
 !     Valeur par defaut et verifiction de IHISTR
 if (nbstru.eq.0) ihistr = 0
 
+call field_get_n_fields(n_fields)
+
 icompt = 0
-do ii = 2, nvppmx
-  if(ihisvr(ii,1).ne.0) icompt = icompt+1
+do f_id = 0, n_fields - 1
+  call field_get_key_int(f_id, keyvis, flag)
+  if (iand(flag, POST_MONITOR).ne.0) icompt = icompt+1
 enddo
-if( (icompt.eq.0.or.ncapt.eq.0) .and. ihistr.eq.0 ) then
+if(icompt.eq.0 .and. ihistr.eq.0) then
   nthist = -1
   frhist = -1.d0
 endif
@@ -330,9 +325,9 @@ else if (nbstru.gt.0) then
 endif
 write(nfecra,3000)
 
-!----
-! FORMATS
-!----
+!--------
+! Formats
+!--------
 
 #if defined(_CS_LANG_FR)
 
@@ -352,16 +347,16 @@ write(nfecra,3000)
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
- 2000 format(                                                           &
-    /,'TTES PHASES  : MODE COUPLAGE DE STRUCTURES NON ACTIVE  ',/,&
+ 2000 format( &
+    /,'MODE COUPLAGE DE STRUCTURES NON ACTIVE  ',/,&
       '                 NBSTRU = ',I10                         ,/)
- 2001 format(                                                           &
+ 2001 format( &
       '               NALIMX INUTILE ET POSITIONNE A 1        ',/)
- 2002 format(                                                           &
-    /,'TTES PHASES  : MODE COUPLAGE CODE_ASTER NON ACTIVE     ',/,&
+ 2002 format( &
+    /,'MODE COUPLAGE CODE_ASTER NON ACTIVE     ',/,&
       '                 NBASTE = ',I10                         ,/)
- 2010 format(                                                           &
-    /,'TTES PHASES  : MODE COUPLAGE DE STRUCTURES ACTIVE      ',/,&
+ 2010 format( &
+    /,'MODE COUPLAGE DE STRUCTURES ACTIVE      ',/,&
       '                 AVEC NBSTRU = ',I10   ,' STRUCTURE(S) ',/,&
       '                                                       ',/,&
       '               COEFFICIENTS DE NEWMARK :               ',/,&
@@ -371,25 +366,25 @@ write(nfecra,3000)
       '                                                       ',/,&
       '               FICHIERS HISTORIQUES DES STRUCTURES :   ',/,&
       '                 IHISTR = ',I4,' ( 1 : active)         ',/)
- 2012 format(                                                           &
-    /,'TTES PHASES  : MODE COUPLAGE CODE_ASTER ACTIVE         ',/,&
+ 2012 format( &
+    /,'MODE COUPLAGE CODE_ASTER ACTIVE         ',/,&
       '                 AVEC NBASTE = ',I10   ,' STRUCTURE(S) ',/)
- 2020 format(                                                           &
-    /,'TTES PHASES  : SCHEMA DE COUPLAGE EXPLICITE ACTIVE     ',/,&
+ 2020 format( &
+    /,'SCHEMA DE COUPLAGE EXPLICITE ACTIVE     ',/,&
       '                                                       ',/,&
       '               COEFFICIENTS DU SCHEMA :                ',/,&
       '                 AEXXST = ',E12.4                       ,/,&
       '                 BEXXST = ',E12.4                       ,/,&
       '                 CFOPRE = ',E12.4                       ,/)
- 2030 format(                                                           &
-    /,'TTES PHASES  : SCHEMA DE COUPLAGE IMPLICITE ACTIVE     ',/,&
+ 2030 format( &
+    /,'SCHEMA DE COUPLAGE IMPLICITE ACTIVE     ',/,&
       '                                                       ',/,&
       '               NB DE SOUS-ITERATIONS MAX. : ',I10       ,/,&
       '               SEUIL DE CONVERGENCE       : ',E12.4     ,/)
 
- 3000 format(                                                           &
+ 3000 format( &
 '-------------------------------------------------------------',/)
- 4000 format(                                                           &
+ 4000 format( &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/,&
@@ -407,7 +402,7 @@ write(nfecra,3000)
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
- 4001 format(                                                           &
+ 4001 format( &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/,&
@@ -426,7 +421,7 @@ write(nfecra,3000)
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
- 4002 format(                                                           &
+ 4002 format( &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/,&
@@ -444,7 +439,7 @@ write(nfecra,3000)
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
- 4003 format(                                                           &
+ 4003 format( &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/,&
@@ -466,7 +461,7 @@ write(nfecra,3000)
 
 #else
 
- 1000 format(                                                           &
+ 1000 format( &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/,&
@@ -482,16 +477,16 @@ write(nfecra,3000)
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
- 2000 format(                                                           &
-    /,'ALL PHASES: COUPLING MODE FOR STRUCTURES NOT ACTIVATED ',/,&
+ 2000 format( &
+    /,'COUPLING MODE FOR STRUCTURES NOT ACTIVATED ',/,&
       '              NBSTRU = ',I10                            ,/)
- 2001 format(                                                           &
+ 2001 format( &
       '            NALIMX USELESS AND SET TO 1                ',/)
- 2002 format(                                                           &
-    /,'ALL PHASES: CODE_ASTER COUPLING MODE NOT ACTIVATED     ',/,&
+ 2002 format( &
+    /,'CODE_ASTER COUPLING MODE NOT ACTIVATED     ',/,&
       '              NBASTE = ',I10                            ,/)
- 2010 format(                                                           &
-    /,'ALL PHASES: COUPLING MODE FOR STRUCTURES ACTIVATED     ',/,&
+ 2010 format( &
+    /,'COUPLING MODE FOR STRUCTURES ACTIVATED     ',/,&
       '              WITH NBSTRU = ',I10   ,' STRUCTURE(S)    ',/,&
       '                                                       ',/,&
       '            NEWMARK COEFFICIENTS:                      ',/,&
@@ -501,25 +496,25 @@ write(nfecra,3000)
       '                                                       ',/,&
       '            MONITORING FILES FOR STRUCTURES:           ',/,&
       '                 IHISTR = ',I4,' ( 1 : activated)      ',/)
- 2012 format(                                                           &
-    /,'ALL PHASES: CPDE_ASTER COUPLING MODE ACTIVATED         ',/,&
+ 2012 format( &
+    /,'CODE_ASTER COUPLING MODE ACTIVATED         ',/,&
       '              WITH NBASTE = ',I10   ,' STRUCTURE(S)    ',/)
- 2020 format(                                                           &
-    /,'ALL PHASES: EXPLICIT SCHEME FOR COUPLING ACTIVATED     ',/,&
+ 2020 format( &
+    /,'EXPLICIT SCHEME FOR COUPLING ACTIVATED     ',/,&
       '                                                       ',/,&
       '            SCHEME COEFFICIENTS:                       ',/,&
       '              AEXXST = ',E12.4                          ,/,&
       '              BEXXST = ',E12.4                          ,/,&
       '              CFOPRE = ',E12.4                          ,/)
  2030 format(                                                           &
-    /,'ALL PHASES: IMPLICIT SCHEME FOR COUPING ACTIVATED      ',/,&
+    /,'IMPLICIT SCHEME FOR COUPING ACTIVATED      ',/,&
       '                                                       ',/,&
       '            NB OF MAX INNER ITERATIONS : ',I10          ,/,&
       '            CONVERGENCE THRESHOLD      : ',E12.4        ,/)
 
- 3000 format(                                                           &
+ 3000 format( &
 '-------------------------------------------------------------',/)
- 4000 format(                                                           &
+ 4000 format( &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/,&
@@ -536,7 +531,7 @@ write(nfecra,3000)
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
- 4001 format(                                                           &
+ 4001 format( &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/,&
@@ -555,7 +550,7 @@ write(nfecra,3000)
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
- 4002 format(                                                           &
+ 4002 format( &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/,&
@@ -573,7 +568,7 @@ write(nfecra,3000)
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
- 4003 format(                                                           &
+ 4003 format( &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/,&
@@ -596,7 +591,7 @@ write(nfecra,3000)
 #endif
 
 !----
-! FIN
+! End
 !----
 
 end subroutine
