@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -23,10 +23,10 @@
 subroutine cfmsfp &
 !================
 
- ( nvar   , nscal  , iterns , ncepdp , ncesmp ,                   &
-   icepdc , icetsm , itypsm ,                                     &
-   dt     , rtpa   , propce , vela   ,                            &
-   ckupdc , smacel ,                                              &
+ ( nvar   , nscal  , iterns , ncepdp , ncesmp ,          &
+   icepdc , icetsm , itypsm ,                            &
+   dt     , vela   ,                                     &
+   ckupdc , smacel ,                                     &
    flumas , flumab )
 
 !===============================================================================
@@ -50,17 +50,14 @@ subroutine cfmsfp &
 ! icetsm(ncesmp)   ! te ! <-- ! numero des cellules a source de masse          !
 ! itypsm           ! te ! <-- ! type de source de masse pour les               !
 ! dt(ncelet)       ! ra ! <-- ! time step (per cell)                           !
-! rtpa             ! ra ! <-- ! calculated variables at cell centers           !
-!  (ncelet, *)     !    !     !  (at previous time steps)                      !
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
 ! vela             ! ra ! <-- ! variable value at time step beginning          !
 ! ckupdc           ! tr ! <-- ! work array for the head loss                   !
 !  (ncepdp,6)      !    !     !                                                !
 ! smacel           ! tr ! <-- ! variable value associated to the mass source   !
-! (ncesmp,*   )    !    !     ! term (for ivar=ipr, smacel is the mass flux    !
+! (ncesmp,*)       !    !     ! term (for ivar=ipr, smacel is the mass flux    !
 !                  !    !     ! \f$ \Gamma^n \f$)                              !
 ! flumas(nfac)     ! tr ! --> ! flux de masse aux faces internes               !
-! flumab(nfabor    ! tr ! --> ! flux de masse aux faces de bord                !
+! flumab(nfabor)   ! tr ! --> ! flux de masse aux faces de bord                !
 !__________________!____!_____!________________________________________________!
 
 !     Type: i (integer), r (real), s (string), a (array), l (logical),
@@ -87,6 +84,7 @@ use ppincl
 use mesh
 use field
 use cs_f_interfaces
+use cs_c_bindings
 
 !===============================================================================
 
@@ -100,30 +98,29 @@ integer          ncepdp , ncesmp
 integer          icepdc(ncepdp)
 integer          icetsm(ncesmp), itypsm(ncesmp,nvar)
 
-double precision dt(ncelet), rtpa(ncelet,nflown:nvar)
-double precision propce(ncelet,*)
-double precision ckupdc(ncepdp,6), smacel(ncesmp,nvar)
+double precision dt(ncelet)
+double precision ckupdc(6,ncepdp), smacel(ncesmp,nvar)
 double precision flumas(nfac), flumab(nfabor)
 double precision vela  (3  ,ncelet)
 
 ! Local variables
 
-integer          ivar, ipcvst, ipcvis
 integer          ifac  , iel, ischcp, idftnp, ircflp
-integer          init  , inc   , iccocg, ippu, isstpp
+integer          init  , inc   , iccocg, isstpp
 integer          nswrgp, imligp, iwarnp, iconvp, idiffp
 integer          icvflb, f_id0
 integer          isou  , jsou
 integer          iflmb0, itypfl
-integer          itsqdm
+integer          itsqdm, imasac
 
 double precision epsrgp, climgp, extrap, thetap, blencp, relaxp
 double precision rom
 
-double precision, allocatable, dimension(:) :: w1, w8, w9
-double precision, allocatable, dimension(:) :: w10, w11, w12
+double precision, allocatable, dimension(:) :: w1
 double precision, dimension(:), pointer :: crom, brom
+double precision, dimension(:), pointer :: viscl, visct
 double precision, dimension(:,:), allocatable :: tsexp, gavinj
+double precision, allocatable, dimension(:,:) :: vel0
 double precision, dimension(:,:,:), allocatable :: tsimp
 double precision, allocatable, dimension(:,:,:), target :: viscf
 double precision, allocatable, dimension(:,:) :: viscce
@@ -134,6 +131,9 @@ double precision, allocatable, dimension(:,:,:) :: coefbv
 double precision, dimension(:,:), pointer :: coefau, cofafu
 double precision, dimension(:,:,:), pointer :: coefbu, cofbfu
 
+type(var_cal_opt) :: vcopt_u, vcopt_p
+
+double precision rvoid(1)
 !===============================================================================
 
 !===============================================================================
@@ -146,16 +146,19 @@ call field_get_coefaf_v(ivarfl(iu), cofafu)
 call field_get_coefbf_v(ivarfl(iu), cofbfu)
 
 ! Allocate work arrays
-allocate(w1(ncelet), w8(ncelet), w9(ncelet))
-allocate(w10(ncelet), w11(ncelet), w12(ncelet))
+allocate(w1(ncelet))
 allocate(tsexp(3,ncelet))
 allocate(gavinj(3,ncelet))
 allocate(tsimp(3,3,ncelet))
 allocate(coefbv(3,3,nfabor))
+allocate(vel0(3,ncelet))
 
-if (idften(iu).eq.1) then
+call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt_u)
+call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt_p)
+
+if (iand(vcopt_u%idften, ISOTROPIC_DIFFUSION).ne.0) then
   allocate(viscf(1, 1, nfac), viscb(nfabor))
-else if (idften(iu).eq.6) then
+else if (iand(vcopt_u%idften, ANISOTROPIC_LEFT_DIFFUSION).ne.0) then
   allocate(viscf(3, 3, nfac), viscb(nfabor))
   allocate(viscce(6,ncelet))
 endif
@@ -166,10 +169,6 @@ endif
 
 f_id0 = -1
 
-! --- Number of the computational variable
-!     Pressure
-ivar     = ipr
-
 ! Density
 
 call field_get_val_s(icrom, crom)
@@ -201,16 +200,6 @@ call field_get_val_s(ibrom, brom)
 !     (Without these terms we are in the configuration Euler + gravity)
 
 ! --- Initialization
-do iel = 1, ncel
-  w10(iel) = 0.d0
-enddo
-do iel = 1, ncel
-  w11(iel) = 0.d0
-enddo
-do iel = 1, ncel
-  w12(iel) = 0.d0
-enddo
-
 do iel = 1, ncel
   do isou = 1, 3
     tsexp(isou,iel) = 0.d0
@@ -224,33 +213,39 @@ enddo
 itsqdm = 0
 if (itsqdm.ne.0) then
 
-  ! --- User soure term
+  if (ivisse.eq.1) then
+
+    call visecv &
+ ( secvif , secvib )
+
+  endif
+
+
+  ! --- User source term
   call ustsnv &
-  !==========
  ( nvar   , nscal  , ncepdp , ncesmp ,                            &
    iu  ,                                                          &
    icepdc , icetsm , itypsm ,                                     &
    dt     ,                                                       &
-   ckupdc , smacel , tsimp  , tsexp     )
+   ckupdc , smacel , tsexp  , tsimp )
 
 
   ! Convective of the momentum equation
   ! in upwind and without reconstruction
-  iconvp = iconv(iu)
+  iconvp = vcopt_u%iconv
 
   init   = 1
   inc    = 1
   iccocg = 1
   iflmb0 = 1
-  nswrgp = nswrgr(iu)
-  imligp = imligr(iu)
-  iwarnp = iwarni(iu)
-  epsrgp = epsrgr(iu)
-  climgp = climgr(iu)
-  extrap = extrag(iu)
-  relaxp = relaxv(iu)
-  thetap = thetav(iu)
-  ippu   = ipprtp(iu)
+  nswrgp = vcopt_u%nswrgr
+  imligp = vcopt_u%imligr
+  iwarnp = vcopt_u%iwarni
+  epsrgp = vcopt_u%epsrgr
+  climgp = vcopt_u%climgr
+  extrap = vcopt_u%extrag
+  relaxp = vcopt_u%relaxv
+  thetap = vcopt_u%thetav
 
   itypfl = 1
 
@@ -267,24 +262,24 @@ if (itsqdm.ne.0) then
   flumas , flumab )
 
   ! ---> Face diffusivity for the velocity
-  idiffp = idiff(iu)
+  idiffp = vcopt_u%idiff
   if (idiffp.ge. 1) then
 
-     ipcvis = ipproc(iviscl)
-     ipcvst = ipproc(ivisct)
+     call field_get_val_s(iviscl, viscl)
+     call field_get_val_s(ivisct, visct)
 
      if (itytur.eq.3) then
         do iel = 1, ncel
-           w1(iel) = propce(iel,ipcvis)
+           w1(iel) = viscl(iel)
         enddo
      else
         do iel = 1, ncel
-           w1(iel) = propce(iel,ipcvis) + idifft(iu)*propce(iel,ipcvst)
+           w1(iel) = viscl(iel) + vcopt_u%idifft*visct(iel)
         enddo
      endif
 
      ! Scalar diffusivity (Default)
-     if (idften(iu).eq.1) then
+     if (iand(vcopt_u%idften, ISOTROPIC_DIFFUSION).ne.0) then
 
         call viscfa &
         !==========
@@ -293,7 +288,7 @@ if (itsqdm.ne.0) then
        viscf  , viscb  )
 
      ! Tensorial diffusion of the velocity (in case of tensorial porosity)
-     else if (idften(iu).eq.6) then
+     else if (iand(vcopt_u%idften, ANISOTROPIC_LEFT_DIFFUSION).ne.0) then
 
         do iel = 1, ncel
           do isou = 1, 3
@@ -305,11 +300,10 @@ if (itsqdm.ne.0) then
         enddo
 
         call vistnv (imvisf, viscce, viscf, viscb)
-        !==========
 
      endif
 
-  ! --- If no dissusion, viscosity is set to 0.
+  ! --- If no diffusion, viscosity is set to 0.
   else
 
      do ifac = 1, nfac
@@ -321,16 +315,7 @@ if (itsqdm.ne.0) then
 
   endif
 
-  if (ivisse.eq.1) then
-
-    call visecv &
-    !==========
- ( propce ,                             &
-   secvif , secvib )
-
-  endif
-
-  idftnp = idften(iu)
+  idftnp = vcopt_u%idften
 
   ! no recontruction
   ircflp = 0;
@@ -344,39 +329,56 @@ if (itsqdm.ne.0) then
 
   icvflb = 0;
 
+  ! The added convective scalar mass flux is:
+  !      (thetap*Y_\face-imasac*Y_\celli)*mf.
+  ! When building the implicit part of the rhs, one
+  ! has to impose 1 on mass accumulation.
+  imasac = 1
+
   call bilscv &
   !==========
-( idtvar , iu     , iconvp , idiffp , nswrgp , imligp , ircflp , &
-  ischcp , isstpp , inc    , imrgra , ivisse ,                   &
-  ippu   , iwarnp , idftnp ,                                     &
-  blencp , epsrgp , climgp , relaxp , thetap ,                   &
-  vela   , vela   ,                                              &
-  coefau , coefbu , cofafu , cofbfu ,                            &
-  flumas , flumab , viscf  , viscb  , secvif , secvib ,          &
-  icvflb , icvfli ,                                              &
+( idtvar , ivarfl(iu)      , iconvp , idiffp , nswrgp , imligp , ircflp , &
+  ischcp , isstpp , inc    , imrgra , ivisse ,                            &
+  iwarnp , idftnp , imasac ,                                              &
+  blencp , epsrgp , climgp , relaxp , thetap ,                            &
+  vela   , vela   ,                                                       &
+  coefau , coefbu , cofafu , cofbfu ,                                     &
+  flumas , flumab , viscf  , viscb  , secvif , secvib ,                   &
+  rvoid  , rvoid  , rvoid  ,                                              &
+  icvflb , icvfli ,                                                       &
   tsexp  )
 
-  if (ncesmp.gt.0) then
-    ! --- Mass sourceterm
-    !     All is explicit for the moment... to be changed when this
-    !      term is tested (see remark at the begining of this file)
-    call catsmv &
-    !==========
- ( ncelet , ncel , ncesmp , iterns , isno2t, thetav(iu),       &
-   icetsm , itypsm(1,iu),                                      &
-   volume , vela , smacel(1,iu) ,smacel(1,ipr) ,               &
-   tsexp  , tsimp , gavinj )
+endif
 
-    do iel = 1, ncel
-      do isou = 1, 3
-        tsexp(isou,iel) = tsexp(isou,iel) + gavinj(isou,iel)
-      enddo
+! End of the test on momentum source terms
+
+! Mass source term
+if (ncesmp.gt.0) then
+
+  ! The momentum balance is used in its conservative form here
+  ! so the mass source term is only composed of gamma*uinj
+  ! => array of previous velocity has to be set to zero
+
+  do iel = 1, ncel
+    do isou = 1,3
+      vel0(isou,iel) = 0.d0
     enddo
+  enddo
 
-  endif
+  call catsmv &
+       !==========
+     ( ncelet , ncel , ncesmp , iterns , isno2t,                   &
+       icetsm , itypsm(1,iu),                                      &
+       cell_f_vol   , vel0 , smacel(1,iu) ,smacel(1,ipr) ,         &
+       tsexp  , tsimp , gavinj )
+
+  do iel = 1, ncel
+    do isou = 1, 3
+      tsexp(isou,iel) = tsexp(isou,iel) + gavinj(isou,iel)
+    enddo
+  enddo
 
 endif
-!     End of the test on momentum source terms
 
 ! --- Volumic forces term (gravity)
 do iel = 1, ncel
@@ -391,16 +393,13 @@ enddo
 
 do iel = 1, ncel
   do isou = 1, 3
-    tsexp(isou,iel) = vela(isou,iel) + dt(iel)*tsexp(isou,iel)
+    tsexp(isou,iel) = dt(iel)*tsexp(isou,iel)
   enddo
 enddo
 
 ! Computation of the flux
 
-! In order to avoid a misfit boundary condition, we impose a homogeneous
-! Neumann condition. Note that it is only useful for gradient
-! reconstruction. The boundary value does not matter since the flux
-! is updated afterwards.
+! volumic flux part based on dt*f^n
 
 ! Initialization of the mass flux
 init   = 1
@@ -409,24 +408,21 @@ inc    = 0
 iccocg = 1
 iflmb0 = 1
 ! Reconstruction is useless here
-nswrgp = 0
-imligp = imligr(ivar)
-iwarnp = iwarni(ivar)
-epsrgp = epsrgr(ivar)
-climgp = climgr(ivar)
-extrap = extrag(ivar)
+nswrgp = 0 ! FIXME
+imligp = vcopt_p%imligr
+iwarnp = vcopt_p%iwarni
+epsrgp = vcopt_p%epsrgr
+climgp = vcopt_p%climgr
+extrap = vcopt_p%extrag
 
 ! Velocity flux (crom, brom not used)
 itypfl = 0
 
-do ifac= 1, nfabor
+! No contribution of f to the boundary mass flux
+do ifac = 1, nfabor
   do isou = 1, 3
     do jsou = 1, 3
-      if (isou.eq.jsou) then
-        coefbv(isou,jsou,ifac) = 1.d0
-      else
-        coefbv(isou,jsou,ifac) = 0.d0
-      endif
+      coefbv(isou,jsou,ifac) = 0.d0
     enddo
   enddo
 enddo
@@ -442,9 +438,24 @@ call inimav                                                      &
   coefau , coefbv ,                                              &
   flumas , flumab )
 
+! volumic flux part based on velocity u^n
+init   = 0
+! take into account Dirichlet velocity boundary conditions
+inc    = 1
+
+call inimav                                                      &
+!==========
+( ivarfl(iu)      , itypfl ,                                     &
+  iflmb0 , init   , inc    , imrgra , nswrgp , imligp ,          &
+  iwarnp ,                                                       &
+  epsrgp , climgp ,                                              &
+  crom, brom,                                                    &
+  vela,                                                          &
+  coefau , coefbu ,                                              &
+  flumas , flumab )
+
 ! Free memory
-deallocate(w1, w8, w9)
-deallocate(w10, w11, w12)
+deallocate(w1)
 deallocate(tsexp)
 deallocate(gavinj)
 deallocate(tsimp)
@@ -452,6 +463,7 @@ deallocate(viscf, viscb)
 if (allocated(secvif)) deallocate(secvif, secvib)
 if (allocated(viscce)) deallocate(viscce)
 deallocate(coefbv)
+deallocate(vel0)
 
 !--------
 ! FORMATS
