@@ -6,7 +6,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -50,10 +50,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#if defined(CS_FPE_TRAP)
-#include <fenv.h>
-#endif
-
 /*----------------------------------------------------------------------------
  * Catalyst and VTK library headers
  *----------------------------------------------------------------------------*/
@@ -68,10 +64,12 @@
 #include <vtkIdTypeArray.h>
 #include <vtkImageData.h>
 #include <vtkInformation.h>
+#include <vtkMPI.h>
 #include <vtkObjectFactory.h>
 #include <vtkPointData.h>
 #include <vtkPolyData.h>
 #include <vtkUnstructuredGrid.h>
+#include <vtkPVConfig.h>
 
 #include <vtkCPDataDescription.h>
 #include <vtkCPInputDataDescription.h>
@@ -110,6 +108,17 @@
 
 #include "fvm_to_catalyst.h"
 
+/*=============================================================================
+ * Local Macro Definitions
+ *============================================================================*/
+
+/* Macro for version test (tests are done on ParaView and not VTK version,
+   as ParaView usually includes its own VTK, and ParaView 5.0 seems to
+   indicate VTK 7.1 just like 5.1, but that version did not contain
+   SetTypedTuple or have SetTupleValue depecated for vtkTypedDataArray) */
+
+#define CS_PV_VERSION  (PARAVIEW_VERSION_MAJOR*10 + PARAVIEW_VERSION_MINOR)
+
 /*----------------------------------------------------------------------------
  * Catalyst field structure
  *----------------------------------------------------------------------------*/
@@ -128,36 +137,33 @@ typedef struct {
 
 typedef struct {
 
-  char                       *name;           /* Writer name */
+  char                       *name;            /* Writer name */
 
-  int                         rank;           /* Rank of current process
-                                                 in communicator */
-  int                         n_ranks;        /* Size of communicator */
+  int                         rank;            /* Rank of current process
+                                                  in communicator */
+  int                         n_ranks;         /* Size of communicator */
 
-  vtkMultiBlockDataSet       *mb;             /* Associated dataset */
+  vtkMultiBlockDataSet       *mb;              /* Associated dataset */
 
   fvm_writer_time_dep_t       time_dependency; /* Mesh time dependency */
 
-  int                         n_fields;       /* Number of fields */
-  fvm_catalyst_field_t      **fields;         /* Array of field helper
-                                                 structures */
+  int                         n_fields;        /* Number of fields */
+  fvm_catalyst_field_t      **fields;          /* Array of field helper
+                                                  structures */
 
-  int                         time_step;      /* Latest time step */
-  double                      time_value;    /* Latest time value */
+  int                         time_step;       /* Latest time step */
+  double                      time_value;      /* Latest time value */
 
-#if defined(HAVE_MPI)
-  MPI_Comm                    comm;           /* Associated MPI communicator */
-#endif
+  vtkCPDataDescription       *datadesc;        /* Data description */
 
-  vtkCPProcessor             *processor;      /* Co processor */
-  vtkCPDataDescription       *datadesc;       /* Data description */
+  char                       *input_name;      /* input name, or NULL for
+                                                  default */
+  bool                        private_comm;    /* Use private communicator */
+  bool                        ensight_names;   /* Use EnSight rules for
+                                                  field names */
 
-  bool                        private_comm;   /* Use private communicator */
-  bool                        ensight_names;  /* Use EnSight rules for
-                                                 field names */
-
-  bool                        modified;       /* Has output been added since
-                                                 las coprocessing ? */
+  bool                        modified;        /* Has output been added since
+                                                  last coprocessing ? */
 
 } fvm_to_catalyst_t;
 
@@ -165,9 +171,16 @@ typedef struct {
  * Static global variables
  *============================================================================*/
 
-#if defined(CS_FPE_TRAP)
-static int    _fenv_save = 0;
-static fenv_t _fenv_old;     /* Old exception mask */
+int _n_writers = 0;
+
+int _n_scripts = 0;                         /* Number of scripts */
+char **_scripts = NULL;                     /* List of scripts */
+
+vtkCPProcessor  *_processor = NULL;         /* Co processor */
+
+#if defined(HAVE_MPI)
+MPI_Comm  _comm = MPI_COMM_NULL;            /* Associated communicator */
+MPI_Comm  _reference_comm = MPI_COMM_NULL;  /* Reference communicator */
 #endif
 
 /*============================================================================
@@ -175,39 +188,286 @@ static fenv_t _fenv_old;     /* Old exception mask */
  *============================================================================*/
 
 /*----------------------------------------------------------------------------
- * Disable floating-point exception handling.
+ * Check if a script is a Catalyst script
  *
- * Uses a counter to handle nested calls.
+ * The script only does cursury checks, so may return false positives
+ * in some cases, but using "incorrect" scripts for Catalyst only
+ * leads to extra warnings, which should be ok pending a more general
+ * file checker.
+ *
+ * parameters:
+ *   path <-- scripts path
+ *
+ * returns:
+ *   1 if script is a Catalyst script, 0 otherwise
  *----------------------------------------------------------------------------*/
 
-static void
-_disable_fe_exceptions(void)
+static int
+_check_script_is_catalyst(const char  *path)
 {
-#if defined(CS_FPE_TRAP)
-  if (_fenv_save == 0) {
-    if (fegetenv(&_fenv_old) == 0)
-      _fenv_save += 1;
-    fedisableexcept(FE_DIVBYZERO | FE_INVALID | FE_OVERFLOW);
+  assert(path != NULL);
+  int retval = 0;
+
+  FILE *fp = fopen(path, "r");
+
+  if (fp == NULL)
+    return retval;
+
+  int checks[] = {false, false, false};
+  int n_checks = 0;
+
+  const char *check_strings[]
+    = {"CreateCoProcessor(",
+       "RequestDataDescription(",
+       "DoCoProcessing("};
+
+  /* Note: we could simply check for "from paraview import coprocessing"
+     in most cases but in cases of non-autogenerated scripts, this might
+     appear in a different form, so we limit ourselves to the main
+     required methods */
+
+  while (true) {
+    char buffer[1024];
+    char *e = buffer+1024;
+
+    char *s = fgets(buffer, 1024, fp);
+    if (s == NULL) break;
+
+    while (*s == ' ' && *s == '\t')  /* skip initial whitespace */
+      s++;
+
+    if (strncmp(s, "def", 3) == 0) {
+      s += 3;
+      if (*s == ' ' || *s == '\t') {
+        while (*s == ' ' && *s == '\t')
+          s++;
+        /* Remove whitespace */
+        size_t l = strlen(s);
+        size_t i, j;
+        for (i = 0, j = 0 ; i < l ; i++) {
+          if (s[i] != ' ' && s[i] != '\t')
+            s[j++] = s[i];
+        }
+
+        for (i = 0; i < 3; i++) {
+          if (strncmp(s, check_strings[i], strlen(check_strings[i])) == 0) {
+            if (checks[i] == false) {
+              checks[i] = true;
+              n_checks += 1;
+            }
+          }
+        }
+
+      }
+    }
+
+    if (n_checks == 3) {
+      retval = 1;
+      break;
+    }
+
+    /* Check for end of line; if not present, continue reading from buffer */
+
+    while (s < e && *s != '\0' && *s != '\n')
+      s++;
+    while (s >= e) {
+      s = fgets(buffer, 1024, fp);
+      if (s == NULL) break;
+      while (s < e && *s != '\0' && *s != '\n')
+        s++;
+    }
   }
-  else
-    _fenv_save += 1;
-#endif
+
+  fclose(fp);
+
+  return retval;
 }
 
 /*----------------------------------------------------------------------------
- * Restore floating-point exception handling.
+ * Add a Catalyst script if not already present.
+ *
+ * parameters:
+ *   path <-- scripts path
+ *
+ * returns:
+ *   id of script in list, or -1 if not valid
+ *----------------------------------------------------------------------------*/
+
+static int
+_add_script(const char         *path)
+{
+  assert(path != NULL);
+
+  int is_catalyst = 0;
+  int rank = 0, n_ranks = 1;
+
+#if defined(HAVE_MPI)
+  if (_comm != MPI_COMM_NULL) {
+    MPI_Comm_rank(_comm, &rank);
+    MPI_Comm_size(_comm, &n_ranks);
+  }
+#endif
+
+  if (rank < 1)
+    is_catalyst = _check_script_is_catalyst(path);
+
+#if defined(HAVE_MPI)
+  if (n_ranks > 1)
+    MPI_Bcast(&is_catalyst, 1, MPI_INT, 0, _comm);
+#endif
+
+  if (is_catalyst < 1)
+    return -1;
+
+  for (int i = 0; i < _n_scripts; i++) {
+    if (strcmp(_scripts[i], path) == 0)
+      return i;
+  }
+
+  int id = _n_scripts;
+  _n_scripts += 1;
+
+  BFT_REALLOC(_scripts, _n_scripts, char *);
+
+  size_t l = strlen(path);
+  BFT_MALLOC(_scripts[id], l+1, char);
+  strncpy(_scripts[id], path, l);
+  _scripts[id][l] = '\0';
+
+  /* Add Catalyst pipeline */
+
+  vtkCPPythonScriptPipeline  *pipeline = vtkCPPythonScriptPipeline::New();
+
+  if (pipeline->Initialize(path) == 0)
+    bft_error(__FILE__, __LINE__, 0,
+              _("Error initializing pipeline from \"%s\":\n\n"), path);
+
+  /* pipeline->SetGhostLevel(1); */
+  _processor->AddPipeline(pipeline);
+  pipeline->Delete();
+
+  return id;
+}
+
+/*----------------------------------------------------------------------------
+ * Add Catalyst scripts from directoty if not already present.
+ *
+ * Currently assumes all Python files in the given directory
+ * are Catalyst scripts.
+ *
+ * parameters:
+ *   dir_path <-- directory path
  *----------------------------------------------------------------------------*/
 
 static void
-_restore_fe_exceptions(void)
+_add_dir_scripts(const char  *dir_path)
 {
-#if defined(CS_FPE_TRAP)
-  if (_fenv_save) {
-    _fenv_save -= 1;
-    if (_fenv_save == 0)
-      fesetenv(&_fenv_old);
+  char **dir_files = cs_file_listdir(dir_path);
+
+  for (int i = 0; dir_files[i] != NULL; i++) {
+    size_t l = strlen(dir_files[i]);
+    /* Filter: Python files only */
+    if (l > 4) {
+      if (strncmp(dir_files[i] + l - 3, ".py", 3) == 0) {
+        char *tmp_name = NULL;
+        BFT_MALLOC(tmp_name,
+                   strlen(dir_path) + 1 + strlen(dir_files[i]) + 1,
+                   char);
+        sprintf(tmp_name, "%s/%s", dir_path, dir_files[i]);
+        _add_script(tmp_name);
+        BFT_FREE(tmp_name);
+      }
+    }
+    BFT_FREE(dir_files[i]);
   }
+
+  BFT_FREE(dir_files);
+}
+
+/*----------------------------------------------------------------------------
+ * Initialize coprocessor.
+ *
+ * parameters:
+ *   private_comm <-- if true, use dedicated communicator
+ *   comm         <-- associated MPI communicator.
+ *----------------------------------------------------------------------------*/
+
+#if defined(HAVE_MPI)
+static void
+_init_coprocessor(bool      private_comm,
+                  MPI_Comm  comm)
+#else
+_init_coprocessor(void)
 #endif
+{
+  if (_processor == NULL)
+    _processor = vtkCPProcessor::New();
+
+  int mpi_flag = 0;
+
+#if defined(HAVE_MPI)
+
+  MPI_Initialized(&mpi_flag);
+
+  if (mpi_flag) {
+
+    if (comm != _reference_comm) {
+
+      if (comm != MPI_COMM_NULL && _reference_comm != MPI_COMM_NULL) {
+        bft_error(__FILE__, __LINE__, 0,
+                  _("All Catalyst writers must use the same MPI communicator"));
+      }
+      else {
+        _reference_comm = comm;
+        if (private_comm && comm != MPI_COMM_NULL)
+          MPI_Comm_dup(comm, &(_comm));
+        else
+          _comm = comm;
+      }
+
+      vtkMPICommunicatorOpaqueComm vtk_comm
+        = vtkMPICommunicatorOpaqueComm(&_comm);
+      _processor->Initialize(vtk_comm);
+
+    }
+
+  }
+
+#endif
+
+  if (!mpi_flag)
+    _processor->Initialize();
+}
+
+/*----------------------------------------------------------------------------
+ * Finalize coprocessor.
+ *
+ * parameters:
+ *   private_comm <-- if true, use dedicated communicator
+ *   comm         <-- associated MPI communicator.
+ *----------------------------------------------------------------------------*/
+
+static void
+_free_coprocessor(void)
+{
+  if (_processor != NULL && _n_writers < 2) {
+
+    _processor->Finalize();
+    _processor->Delete();
+    _processor = NULL;
+
+    for (int i = 0; i < _n_scripts; i++)
+      BFT_FREE(_scripts[i]);
+    BFT_FREE(_scripts);
+
+#if defined(HAVE_MPI)
+    {
+      if (_comm != _reference_comm && _comm != MPI_COMM_NULL)
+        MPI_Comm_free(&_comm);
+    }
+#endif
+
+  }
 }
 
 /*----------------------------------------------------------------------------
@@ -266,7 +526,8 @@ _add_catalyst_mesh(fvm_to_catalyst_t  *writer,
   /* Add a new Catalyst mesh structure */
 
   int id = writer->mb->GetNumberOfBlocks();
-  vtkUnstructuredGrid* ugrid = vtkUnstructuredGrid::New();
+  vtkSmartPointer<vtkUnstructuredGrid> ugrid
+    = vtkSmartPointer<vtkUnstructuredGrid>::New();
 
   writer->mb->SetBlock(id, ugrid);
   writer->mb->GetMetaData(id)->Set(vtkCompositeDataSet::NAME(), mesh->name);
@@ -451,6 +712,9 @@ _get_catalyst_field_id(fvm_to_catalyst_t         *writer,
                        cs_datatype_t              datatype,
                        fvm_writer_var_loc_t       location)
 {
+  CS_UNUSED(dim);
+  CS_UNUSED(datatype);
+
   int i;
 
   for (i = 0; i < writer->n_fields; ++i){
@@ -485,8 +749,6 @@ _get_catalyst_field_id(fvm_to_catalyst_t         *writer,
  *   mesh_id    <-- id of associated mesh in structure.
  *   dim        <-- number of field components.
  *   location   <-- mesh location (cells or vertices).
- *   time_step  <-- number of time step to add
- *   time_value <-- associated time value
  *
  * returns
  *   field id in writer structure
@@ -498,17 +760,17 @@ _add_catalyst_field(fvm_to_catalyst_t         *writer,
                     int                        mesh_id,
                     int                        dim,
                     cs_datatype_t              datatype,
-                    fvm_writer_var_loc_t       location,
-                    const int                  time_step,
-                    const double               time_value)
+                    fvm_writer_var_loc_t       location)
 {
+  CS_UNUSED(datatype);
+
   int f_id = writer->n_fields;
 
   BFT_REALLOC(writer->fields,
               writer->n_fields+ 1,
               fvm_catalyst_field_t *);
 
-  vtkUnstructuredGrid *f =  NULL;
+  vtkUnstructuredGrid *f = NULL;
   vtkDoubleArray *tmp = NULL;
 
   const int dest_dim = (dim == 6) ? 9 : dim;
@@ -524,12 +786,14 @@ _add_catalyst_field(fvm_to_catalyst_t         *writer,
     tmp->SetNumberOfComponents(dest_dim);
 
     if (location == FVM_WRITER_PER_NODE) {
-      f->GetPointData()->AllocateArrays(f->GetNumberOfPoints());
+      tmp->SetNumberOfTuples(f->GetNumberOfPoints());
+      // f->GetPointData()->AllocateArrays(f->GetNumberOfPoints());
       vtkDataSetAttributes::SafeDownCast
         (f->GetPointData())->AddArray(vtkAbstractArray::SafeDownCast(tmp));
     }
     else {
-      f->GetCellData()->AllocateArrays(f->GetNumberOfCells());
+      tmp->SetNumberOfTuples(f->GetNumberOfCells());
+      // f->GetCellData()->AllocateArrays(f->GetNumberOfCells());
       /* force to AddArray, due to protection of SetArray */
       vtkDataSetAttributes::SafeDownCast
         (f->GetCellData())->AddArray(vtkAbstractArray::SafeDownCast(tmp));
@@ -611,12 +875,16 @@ _export_vertex_coords(const fvm_nodal_t        *mesh,
     vtkIdTypeArray *g_vtx_id = vtkIdTypeArray::New();
 
     g_vtx_id->SetNumberOfComponents(1);
-    g_vtx_id->SetName("GlobalNodeId");
+    g_vtx_id->SetName("GlobalNodeIds");
     g_vtx_id->SetNumberOfTuples(n_vertices);
 
     for (i = 0; i < n_vertices; i++) {
       vtkIdType ii = g_vtx_num[i]-1;
+#if CS_PV_VERSION < 51
       g_vtx_id->SetTupleValue(i, &ii);
+#else
+      g_vtx_id->SetTypedTuple(i, &ii);
+#endif
     }
 
     vtk_mesh->GetPointData()->SetGlobalIds(g_vtx_id);
@@ -843,7 +1111,7 @@ _export_field_values_n(const fvm_nodal_t    *mesh,
 
   values = vtkDoubleArray::SafeDownCast
     (f->GetPointData()->GetArray(fieldname))
-    ->WritePointer(0, f->GetNumberOfPoints());
+    ->WritePointer(0, dest_dim*f->GetNumberOfPoints());
 
   fvm_convert_array(dim,
                     0,
@@ -914,7 +1182,7 @@ _export_field_values_e(const fvm_nodal_t         *mesh,
 
   values = vtkDoubleArray::SafeDownCast
     (f->GetCellData()->GetArray(fieldname))
-    ->WritePointer(0, dim*f->GetNumberOfCells());
+    ->WritePointer(0, dest_dim*f->GetNumberOfCells());
 
   assert(values != NULL);
 
@@ -936,7 +1204,7 @@ _export_field_values_e(const fvm_nodal_t         *mesh,
 
     fvm_convert_array(dim,
                       0,
-                      dim,
+                      dest_dim,
                       src_shift,
                       section->n_elements + src_shift,
                       interlace,
@@ -948,25 +1216,27 @@ _export_field_values_e(const fvm_nodal_t         *mesh,
                       field_values,
                       values + start_id);
 
-    /* Special case for symmetric tensors */
-
-    if (dim == 6) {
-      for (cs_lnum_t i = 0; i < mesh->n_vertices; i++) {
-        values[9*i + 8] = values[9*i + 2];
-        values[9*i + 7] = values[9*i + 4];
-        values[9*i + 6] = values[9*i + 5];
-        values[9*i + 4] = values[9*i + 1];
-        values[9*i + 2] = values[9*i + 5];
-        values[9*i + 1] = values[9*i + 3];
-        values[9*i + 5] = values[9*i + 7];
-      }
-    }
-
     start_id += section->n_elements*dest_dim;
     if (n_parent_lists == 0)
       src_shift += section->n_elements;
 
   }
+
+  /* Special case for symmetric tensors */
+
+  if (dim == 6) {
+
+    cs_lnum_t n_elts = f->GetNumberOfCells();
+    for (cs_lnum_t i = 0; i < n_elts; i++) {
+      values[9*i + 8] = values[9*i + 2];
+      values[9*i + 7] = values[9*i + 4];
+      values[9*i + 6] = values[9*i + 5];
+      values[9*i + 4] = values[9*i + 1];
+      values[9*i + 2] = values[9*i + 5];
+      values[9*i + 1] = values[9*i + 3];
+      values[9*i + 5] = values[9*i + 7];
+    }
+  }
 }
 
 /*============================================================================
@@ -982,10 +1252,11 @@ BEGIN_C_DECLS
  *   private_comm        use private MPI communicator (default: false)
  *   names=<fmt>         use same naming rules as <fmt> format
  *                       (default: ensight)
+ *   input_name=<name>   define input name (default: writer name)
  *
  * parameters:
  *   name           <-- base output case name.
- *   options        <-- whitespace separated, lowercase options list
+ *   options        <-- whitespace separaed, lowercase options list
  *   time_dependecy <-- indicates if and how meshes will change with time
  *   comm           <-- associated MPI communicator.
  *
@@ -1008,9 +1279,11 @@ fvm_to_catalyst_init_writer(const char             *name,
                             fvm_writer_time_dep_t   time_dependency)
 #endif
 {
+  CS_UNUSED(path);
+
   fvm_to_catalyst_t  *w = NULL;
 
-  _disable_fe_exceptions();
+  bool private_comm = false;
 
   /* Initialize writer */
 
@@ -1019,6 +1292,8 @@ fvm_to_catalyst_init_writer(const char             *name,
   w->rank = 0;
   w->n_ranks = 1;
 
+  w->time_dependency = time_dependency;
+
   w->mb = vtkMultiBlockDataSet::New();
 
   w->n_fields  = 0;
@@ -1027,8 +1302,8 @@ fvm_to_catalyst_init_writer(const char             *name,
   w->time_step  = -1;
   w->time_value = 0.0;
 
-  w->private_comm = false;
   w->ensight_names = true;
+  w->input_name = NULL;
 
   /* Writer name */
 
@@ -1037,7 +1312,7 @@ fvm_to_catalyst_init_writer(const char             *name,
     strcpy(w->name, name);
   }
   else {
-    const char _name[] = "catalyst_writer";
+    const char _name[] = "catalyst";
     BFT_MALLOC(w->name, strlen(_name) + 1, char);
     strcpy(w->name, _name);
   }
@@ -1056,92 +1331,63 @@ fvm_to_catalyst_init_writer(const char             *name,
       l_opt = i2 - i1;
 
       if ((l_opt == 12) && (strncmp(options + i1, "private_comm", l_opt) == 0))
-        w->private_comm = true;
-      else if ((l_opt == 6) && (strncmp(options + i1, "names=", l_opt) == 0)) {
+        private_comm = true;
+      else if ((l_opt > 6) && (strncmp(options + i1, "names=", 6) == 0)) {
         if ((l_opt == 6+7) && (strncmp(options + i1 + 6, "ensight", 7) == 0))
           w->ensight_names = true;
         else
           w->ensight_names = false;
       }
+      else if ((l_opt > 11) && (strncmp(options + i1, "input_name=", 11) == 0)) {
+        int l = strlen(options + i1 + 11);
+        BFT_MALLOC(w->input_name, l+1, char);
+        strncpy(w->input_name, options + i1 + 11, l);
+        w->input_name[l] = '\0';
+      }
+
+      for (i1 = i2 + 1; i1 < l_tot && options[i1] == ' '; i1++);
 
     }
 
   }
 
+#if CS_PV_VERSION < 55
+  if (w->input_name == NULL) {
+    char n[] = "input";
+    BFT_MALLOC(w->input_name, strlen(n)+1, char);
+    strcpy(w->input_name, n);
+  }
+#endif
+
+  /* Ensure coprocessor is built */
+
+  _n_writers += 1;
+
+#if defined(HAVE_MPI)
+  _init_coprocessor(private_comm, comm);
+#else
+  _init_coprocessor();
+#endif
+
   /* Parallel parameters */
 
 #if defined(HAVE_MPI)
-  if (w->private_comm) {
-    int mpi_flag;
-    MPI_Initialized(&mpi_flag);
-
-    if (mpi_flag && comm != MPI_COMM_NULL) {
-      MPI_Comm_dup(comm, &(w->comm));
-      MPI_Comm_rank(w->comm, &(w->rank));
-      MPI_Comm_size(w->comm, &(w->n_ranks));
-    }
-    else
-      w->comm = MPI_COMM_NULL;
-  }
-  else
-    w->comm = comm;
-#endif /* defined(HAVE_MPI) */
-
-  /* Catalyst pipeline */
-
-  w->processor = vtkCPProcessor::New();
-  w->processor->Initialize();
-
-  vtkCPPythonScriptPipeline  *pipeline = vtkCPPythonScriptPipeline::New();
-
-  /* read the coprocessing Python file */
-
-  char *script_path = NULL;
-
-  int l = strlen(w->name);
-  BFT_MALLOC(script_path, strlen(w->name) + 4, char);
-  strcpy(script_path, w->name);
-  for (int i = 0; i < l; i++) {
-    if (script_path[i] == ' ')
-      script_path[i] = '_';
-  }
-  strcat(script_path, ".py");
-
-  /* check file's existence */
-
-  if (cs_file_isreg(script_path)) {
-
-    if (pipeline->Initialize(script_path) == 0)
-      bft_error(__FILE__, __LINE__, 0,
-                _("Error initializing pipeline from \"%s\":\n\n"), script_path);
-
-    /* pipeline->SetGhostLevel(1); */
-    w->processor->AddPipeline(pipeline);
-    pipeline->Delete();
-
-  }
-  else
-    bft_error(__FILE__, __LINE__, 0,
-              _("Missing Catalyst pipeline file \"%s\":\n\n"), script_path);
-
-  BFT_FREE(script_path);
-
-  w->datadesc = vtkCPDataDescription::New();
-  w->datadesc->AddInput("input");
-
-  w->modified = true;
-
-#if defined(CS_FPE_TRAP) && !defined(HAVE_CATALYST)
-  if (_fenv_set == 0) {
-    if (fegetenv(&_fenv_old) == 0) {
-      feenableexcept(FE_DIVBYZERO | FE_INVALID | FE_OVERFLOW);
-      _fenv_set = 1;
-      /* To revert to initial behavior: fesetenv(&_fenv_old); */
-    }
+  if (_comm != MPI_COMM_NULL) {
+    MPI_Comm_rank(_comm, &(w->rank));
+    MPI_Comm_size(_comm, &(w->n_ranks));
   }
 #endif
 
-  _restore_fe_exceptions();
+  if (_n_scripts < 1)
+    _add_dir_scripts(".");
+
+  w->datadesc = vtkCPDataDescription::New();
+  if (w->input_name != NULL)
+    w->datadesc->AddInput(w->input_name);
+  else
+    w->datadesc->AddInput(w->name);
+
+  w->modified = true;
 
   return w;
 }
@@ -1165,8 +1411,6 @@ fvm_to_catalyst_finalize_writer(void  *this_writer_p)
 
   assert(w != NULL);
 
-  _disable_fe_exceptions();
-
   /* Write output if not done already */
 
   fvm_to_catalyst_flush(this_writer_p);
@@ -1178,8 +1422,10 @@ fvm_to_catalyst_finalize_writer(void  *this_writer_p)
   /* Free vtkUnstructuredGrid and field structures
      (reference counters should go to 0) */
 
-  w->processor->Finalize();
-  w->processor->Delete();
+  _free_coprocessor();
+
+  _n_writers -= 1;
+
   w->datadesc->Delete();
   w->mb->Delete();
 
@@ -1190,19 +1436,10 @@ fvm_to_catalyst_finalize_writer(void  *this_writer_p)
 
   BFT_FREE(w->fields);
 
-#if defined(HAVE_MPI)
-  {
-    if (w->private_comm && w->comm != MPI_COMM_NULL)
-      MPI_Comm_free(&(w->comm));
-  }
-#endif /* defined(HAVE_MPI) */
-
   /* Free fvm_to_catalyst_t structure */
 
   BFT_FREE(w);
 
-  _restore_fe_exceptions();
-
   return NULL;
 }
 
@@ -1225,16 +1462,23 @@ fvm_to_catalyst_set_mesh_time(void    *this_writer_p,
   int _time_step = (time_step > -1) ? time_step : 0;
   double _time_value = (time_value > 0.0) ? time_value : 0.0;
 
-  _disable_fe_exceptions();
-
   if (_time_step > w->time_step) {
+    if (   w->time_dependency == FVM_WRITER_TRANSIENT_CONNECT
+        && _time_step > w->time_step) {
+      for (int i = 0; i < w->n_fields; i++) {
+        w->fields[i]->f = NULL; // delete w->fields[i]->f;
+        BFT_FREE(w->fields[i]);
+      }
+      BFT_FREE(w->fields);
+      w->n_fields = 0;
+      w->mb->Delete();
+      w->mb = vtkMultiBlockDataSet::New();
+    }
     w->time_step = _time_step;
     assert(time_value >= w->time_value);
     w->time_value = _time_value;
     w->datadesc->SetTimeData(w->time_value, w->time_step);
   }
-
-  _restore_fe_exceptions();
 }
 
 /*----------------------------------------------------------------------------
@@ -1255,8 +1499,6 @@ fvm_to_catalyst_export_nodal(void               *this_writer_p,
 
   const int  elt_dim = fvm_nodal_get_max_entity_dim(mesh);
 
-  _disable_fe_exceptions();
-
   /* Initialization */
   /*----------------*/
 
@@ -1320,8 +1562,6 @@ fvm_to_catalyst_export_nodal(void               *this_writer_p,
   } /* End of loop on sections */
 
   w->modified = true;
-
-  _restore_fe_exceptions();
 }
 
 /*----------------------------------------------------------------------------
@@ -1368,8 +1608,6 @@ fvm_to_catalyst_export_field(void                  *this_writer_p,
 
   fvm_to_catalyst_t *w = (fvm_to_catalyst_t *)this_writer_p;
 
-  _disable_fe_exceptions();
-
   /* Initialization */
   /*----------------*/
 
@@ -1425,9 +1663,7 @@ fvm_to_catalyst_export_field(void                  *this_writer_p,
                                    mesh_id,
                                    dimension,
                                    datatype,
-                                   location,
-                                   time_step,
-                                   time_value);
+                                   location);
 
   vtkUnstructuredGrid  *f = w->fields[field_id]->f;
 
@@ -1466,8 +1702,6 @@ fvm_to_catalyst_export_field(void                  *this_writer_p,
   fvm_to_catalyst_set_mesh_time(w, time_step, time_value);
 
   w->modified = true;
-
-  _restore_fe_exceptions();
 }
 
 /*----------------------------------------------------------------------------
@@ -1484,15 +1718,20 @@ fvm_to_catalyst_flush(void  *this_writer_p)
 {
   fvm_to_catalyst_t *w = (fvm_to_catalyst_t *)this_writer_p;
 
-  _disable_fe_exceptions();
+  if (_processor->RequestDataDescription(w->datadesc) != 0 && w->modified) {
+    int n = w->datadesc->GetNumberOfInputDescriptions();
+    if (n == 1)
+      w->datadesc->GetInputDescription(0)->SetGrid(w->mb);
+    else {
+      if (w->input_name != NULL)
+        w->datadesc->GetInputDescriptionByName(w->input_name)->SetGrid(w->mb);
+      else
+        w->datadesc->GetInputDescriptionByName(w->name)->SetGrid(w->mb);
+    }
 
-  if (w->processor->RequestDataDescription(w->datadesc) != 0 && w->modified) {
-    w->datadesc->GetInputDescriptionByName("input")->SetGrid(w->mb);
-    w->processor->CoProcess(w->datadesc);
+    _processor->CoProcess(w->datadesc);
     w->modified = false;
   }
-
-  _restore_fe_exceptions();
 }
 
 /*----------------------------------------------------------------------------*/
