@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -27,6 +27,8 @@ module ppincl
 
   !===========================================================================
 
+  use, intrinsic :: iso_c_binding
+
   use ppppar
   use ppthch
 
@@ -49,12 +51,12 @@ module ppincl
   !----------------------------------------------------------------------------
   !--> TABLEAU INDICATEURS DU CHOIX DE LA PHYSIQUE PARTICULIERE CHOISIE
 
-  !> maximal number of specific physics
+  !> number of specific physics
   integer   nmodmx
-  parameter(nmodmx = 50)
+  parameter(nmodmx = 14)
 
   !> global indicator for speciphic physics
-  !> By default, all the indicators ippmod(i.....) are initialised at -1,
+  !> By default, all the indicators ippmod(i.....) are initialized to -1,
   !> which means that no specific physics is activated.
   !>   - Diffusion flame in the framework of “3 points” rapid complete chemistry:
   !>  indicator ippmod(icod3p)
@@ -106,8 +108,10 @@ module ppincl
   !>      - ippmod(ieljou) = 4 use of complex potential and specific boundary conditions
   !>  for transformers.
   !>      - ippmod(ieljou) =-1 module not activated
-  !>   - Compressible module: indicator ippmod(icompf)
-  !>      - ippmod(icompf) = 0 module activated
+  !>   - compressible flow module: indicator ippmod(icompf)
+  !>      - ippmod(icompf) = 2 module activated: homogeneous two phase model
+  !>      - ippmod(icompf) = 1 module activated: single phase model
+  !>      - ippmod(icompf) = 0 module activated: single phase barotropic model
   !>      - ippmod(icompf) =-1 module not activated
   !>   - atmospheric flow module: indicator ippmod(iatmos)
   !>      - ippmod(iatmos) =-1 module not activated
@@ -115,22 +119,20 @@ module ppincl
   !>      - ippmod(iatmos) = 1 dry atmosphere
   !>      - ippmod(iatmos) = 2 humid atmosphere
 
-  integer, save ::           ippmod(nmodmx)
+  integer(c_int), pointer, save :: ippmod(:)
 
   !> ippmod(iphpar) is a global indicator for the specific physics:
   !>  - 0: no specific physics
   !>  - 1: switch on the specific physics
-  !>  - 2: switch on the specific physics plus radiative transfert
+  !>  - 2: switch on the specific physics plus radiative transfer
   !>       with a parametric file
   integer :: iphpar
 
-  !> pointer for speciphic physics
+  !> pointer for specific physics
   !> - ippmod(icod3p) = 0 adiabatic conditions
   !> - ippmod(icod3p) = 1 permeatic conditions (enthalpy transport)
   !> - ippmod(icod3p) =-1 module not activated
   integer ::  icod3p
-  ! TODO Modeles propres a la combustion gaz ICO...
-  integer ::  icodeq
 
   !> pointer to specify Eddy Break Up pre-mixed flame with indicator ippmod(icoebu)
   !> - ippmod(icoebu) = 0 adiabatic conditions at constant richness
@@ -140,9 +142,6 @@ module ppincl
   !> - ippmod(icoebu) =-1 module not activated
   integer ::  icoebu
 
-  ! TODO Modeles propres a la combustion gaz ICO...
-  integer ::  icobml
-
   !> pointer to specify Libby-Williams pre-mixed flame withy indicator ippmod(icolwc)
   !> - ippmod(icolwc)=0 two peak model with adiabiatic conditions.
   !> - ippmod(icolwc)=1 two peak model with permeatic conditions.
@@ -154,7 +153,7 @@ module ppincl
   integer ::  icolwc
 
   ! TODO Modeles propres a la combustion gaz ICO...
-  integer ::  isoot
+  integer(c_int), pointer, save ::  isoot
 
   !> pointer to specify Joule effect module (Laplace forces not taken into account)
   !> with indicator ippmod(ieljou):
@@ -175,9 +174,6 @@ module ppincl
   !> - ippmod(ielarc) =-1 module not activated
   integer ::  ielarc
 
-  ! TODO Modeles propres aux versions effet Joule et conduction ionique
-  integer ::  ielion
-
   !> pointer to specify Lagrangian modelling of multi-coals and multi-classes pulverised coal
   !> combustion with indicator ippmod(icpl3c).
   !> The number of different coals must be inferior or equal
@@ -199,7 +195,7 @@ module ppincl
   integer ::  iccoal
 
   !> coal with drift (0: without drift (default), 1: with)
-  integer ::  i_coal_drift
+  integer ::  i_comb_drift
 
   !> pointer to specify multi-classes pulverised heavy fuel combustion
   !> with indicator ippmod(icfuel)
@@ -208,8 +204,10 @@ module ppincl
   integer ::  icfuel
 
   !> pointer to specify compressible module with indicator ippmod(icompf)
-  !> - ippmod(icompf) = 0 module activated
-  !> - ippmod(icompf) =-1 module not activated
+  !>      - ippmod(icompf) = 2 module activated: homogeneous two phase model
+  !>      - ippmod(icompf) = 1 module activated: single phase model
+  !>      - ippmod(icompf) = 0 module activated: single phase barotropic model
+  !>      - ippmod(icompf) =-1 module not activated
   integer ::  icompf
 
   !> pointer to specify atmospheric flow module with indicator ippmod(iatmos)
@@ -219,38 +217,56 @@ module ppincl
   !> - ippmod(iatmos) = 2 humid atmosphere
   integer ::  iatmos
 
-  ! TODO : commentaire neutralise
-  ! pointer to specify cooling towers module with indicator ippmod(iaeros)
-  ! - ippmod(iaeros) =-1 module not activated
-  ! - ippmod(iaeros) = 0 no model (NOT functional)
-  ! - ippmod(iaeros) = 1 Poppe's model
-  ! - ippmod(iaeros) = 2 Merkel's model
+  !> pointer to specify cooling towers module with indicator ippmod(iaeros)
+  !> - ippmod(iaeros) =-1 module not activated
+  !> - ippmod(iaeros) >= 0  activated
   integer ::  iaeros
 
-  parameter       (iphpar = 1 , icod3p = 2 , icodeq = 3 ,           &
-                   icoebu = 4 , icobml = 5 , icolwc = 6 ,           &
-                   icpl3c = 7 , icfuel = 8 , ieljou = 9 ,           &
-                   ielarc = 10, ielion = 11, icompf = 12,           &
-                   iatmos = 13, iaeros = 14, iccoal = 15)
+  !> pointer to specify gas mixture module with indicator ippmod(igmix)
+  !> - ippmod(igmix) =-1 module not activated
+  !> - ippmod(igmix) = 0  Air/Helium   gas mixtures
+  !> - ippmod(igmix) = 1  Air/Hydrogen gas mixtures
+  !> - ippmod(igmix) = 2  Air/Steam    gas mixtures
+  !> - ippmod(igmix) = 3  Air/Helium/Steam gas mixtures
+  !> - ippmod(igmix) = 4  Air/Hydrogen/Steam gas mixtures
+
+  integer ::  igmix
+
+  !> pointer to specify richards model
+  !> - ippmod(iricha) =-1 module not activated
+  !> - ippmod(iricha) = 1 module activated
+  integer ::  idarcy
+
+  parameter       (iphpar = 1 , icod3p = 2 ,                        &
+                   icoebu = 3 , icolwc = 4 ,                        &
+                   icpl3c = 5 , iccoal = 6 , icfuel = 7 ,           &
+                   ieljou = 8 , ielarc = 9 , icompf = 10,           &
+                   iatmos = 11, iaeros = 12,                        &
+                   igmix  = 13, idarcy = 14)
+
   !> \}
 
-  !--> NOMBRE DE VARIABLES ALGEBRIQUES OU D'ETAT
-  !    pour la physique particuliere NSALPP
-  !    total NSALTO
+  !--> PARAMETERS ASSOCIATED WITH THE GAS MIXTURE MODELLING
 
-  !> \defgroup number_variables Number of algebraic or state variables
+  !> \defgroup modelling chosen by the user
 
-  !> \addtogroup number_variables
+  !> \addtogroup gas_mixture
   !> \{
 
-  !> number of variables for specific physic
-  integer, save :: nsalpp
-  !> total number of variables
-  integer, save :: nsalto
+  !> Specific condensation modelling
+  !>      if = -1 module not activated
+  !>      if =  0 condensation source terms activated
+  integer, save ::  icondb
 
+  !> Specific condensation modelling
+  !>      if = -1 module not activated
+  !>      if =  0 condensation source terms with metal
+  !>                               structures activate
+  integer, save ::  icondv
 
   !> \}
 
+
   !--> POINTEURS VARIABLES COMBUSTION GAZ
 
   !> \defgroup gaz_combustion Gaz combustion variables pointers
@@ -277,21 +293,21 @@ module ppincl
   integer, save :: icm
 
   ! TODO
-  !> tansported variable
+  !> transported variable
   integer, save :: icp2m
 
   ! TODO
-  !> tansported variable
+  !> transported variable
   integer, save :: ifpcpm
 
   ! TODO
-  !> tansported variable
+  !> transported variable
   integer, save :: iyfm
   ! TODO
-  !> tansported variable
+  !> transported variable
   integer, save :: iyfp2m
   ! TODO
-  !> tansported variable
+  !> transported variable
   integer, save :: icoyfp
 
   ! ---- Variables d'etat
@@ -319,7 +335,7 @@ module ppincl
   !> state variable:  \f$T^4\f$ term, when the radiation modelling is activated
   integer, save :: it4m
 
-  ! --- Pointeurs proprietes (PROPCE)
+  ! pointer for source term in combustion
   ! TODO
   integer, save :: itsc
 
@@ -377,12 +393,8 @@ module ppincl
   !> from the three others)
   integer, save :: if4p2m
 
-  ! TODO absent de la doc utilisateur
-  !> transported variable of countinuous phase (gas mixture)
+  !> f1f2 variance
   integer, save :: ifvp2m
-  ! TODO absent de la doc utilisateur
-  !> transported variable of countinuous phase (gas mixture)
-  integer, save :: if3mc2
 
   !        Phase dispersee (classe de particules)
   !> coke mass fraction related to the class icla
@@ -401,11 +413,20 @@ module ppincl
   !> transported variable of dispersed phase (particle class)
   integer, save :: ixwt(nclcpm)
 
-  !> Pointer to x2*age(particles) and x1*age(gas phase)
-  integer, save :: iagecp_temp(nclcpm)
+  !> Pointer to Np*age(particles)
+  integer, save :: inagecp(nclcpm)
 
-  !> Pointer to x2*age(particles) and x1*age(gas phase)
-  integer, save :: iaggas_temp
+  !> Pointer to age of bulk
+  integer, save :: iage
+
+  !>
+  integer, save :: iv_p_x(nclcpm)
+
+  !>
+  integer, save :: iv_p_y(nclcpm)
+
+  !>
+  integer, save :: iv_p_z(nclcpm)
 
   ! ---- Variables d'etat
   !        Phase continue (melange gazeux)
@@ -450,6 +471,9 @@ module ppincl
   !> coke disappearance rate of the coke burnout of the class \c icla
   integer, save :: igmhet(nclcpm)
 
+  !> Implicite part of the exchanges to the gas by molecular distribution
+  integer, save :: igmtr(nclcpm)
+
   ! TODO absent de la doc utilisateur
   !> State variables of dispersed phase (particles class)
   integer, save :: ighco2(nclcpm)
@@ -464,11 +488,11 @@ module ppincl
   !> State variables of dispersed phase (particles class)
   integer, save :: igmsec(nclcpm)
 
-  ! TODO absent de la doc utilisateur
+  !> Used for bulk balance of Carbon
   integer, save :: ibcarbone
-  ! TODO absent de la doc utilisateur
+  !> Used for bulk balance of Oxygen
   integer, save :: iboxygen
-  ! TODO absent de la doc utilisateur
+  !> Used for bulk balance of Hydrogen
   integer, save :: ibhydrogen
 
   !> \}
@@ -523,85 +547,6 @@ module ppincl
 
   !> \}
 
-  !--> POINTEURS VARIABLES VERSION ELECTRIQUES
-
-  !> \defgroup electrical Electrical version variables
-
-  !> \addtogroup electrical
-  !> \{
-
-  !> dsize of "electrical" vectors
-  integer    ndimve
-  parameter (ndimve = 3)
-
-  !> \defgroup elec_transported Transported variables
-
-  !> \addtogroup elec_transported
-  !> \{
-
-  !> real potential
-  integer, save :: ipotr
-
-  !> imaginary potential
-  integer, save :: ipoti
-
-  !> components of magnetic vector potential
-  integer, save :: ipotva(ndimve)
-
-  !> constituent massic fraction
-  integer, save :: iycoel(ngazgm)
-
-  !> \}
-
-  !> \defgroup elec_properties Physical properties
-
-  !> \addtogroup elec_properties
-  !> \{
-
-  !> volumetric power dissipated by Joule effect (\f$ W/m^3 \f$)
-  integer, save :: iefjou
-
-  !> volumetric electrical charge (\f$ C/m^3 \f$)
-  ! TODO  Charge electrique volumique C/m3                  = IQELEC
-  integer, save :: iqelec
-  !> electromagetic Laplace force (\f$ N/m^2 \f$)
-  ! TODO  Forces electromagnetiques de Laplace en N/m3      = ILAPLA()
-  integer, save :: ilapla(ndimve)
-
-  !> real intensity density (\f$ A/m^2 \f$)
-  ! TODO  Densite de courant electrique reelle A/m2         = IDJR()
-  integer, save :: idjr(ndimve)
-
-  !> imaginary intensity density (\f$ A/m^2 \f$)
-  ! TODO  Densite de courant electrique imaginaire en A/m2  = IDJI()
-  integer, save :: idji(ndimve)
-
-  !> absorption coefficient  (\f$ m^{-1} \f$)
-  ! TODO   ou coeff d'absorption en m-1                     = IDRAD
-  integer, save :: idrad
-
-  !> \}
-
-  !> \}
-
-
-  !> \defgroup ionic_conduction Ionic conduction variables
-
-  !> \addtogroup ionic_conduction
-  !> \{
-
-  ! TODO absent de la doc utilisateur
-  integer   nesiom
-  parameter (nesiom = 10)
-
-  ! TODO absent de la doc utilisateur
-  integer   nespio
-
-  !> transporded variable per species
-  integer, save :: iymion(nesiom)
-
-  !> \}
-
 
   !> \defgroup compressible Compressible module
 
@@ -614,19 +559,35 @@ module ppincl
   !> temperature deduced from the specific total energy
   integer, save :: itempk
 
+  !> \defgroup comp_homogeneous Homogeneous two-phase flow model
+
+  !> \addtogroup comp_homogeneous
+  !> \{
+
+  !> \anchor ifracv
+  !> homogeneous model, volume fraction \f$ \alpha \f$
+  integer, save :: ifracv
+
+  !> \anchor ifracm
+  !> homogeneous model, mass fraction \f$ y \f$
+  integer, save :: ifracm
+
+  !> \anchor ifrace
+  !> homogeneous model, energy fraction \f$ z \f$
+  integer, save :: ifrace
+
+  !> \}
+
   !> \defgroup comp_properties Physical properties
 
   !> \addtogroup comp_properties
   !> \{
 
-  !> additional property
-  integer, save :: icv
-
   !> additional property:
   !>  - 0 indicates that the volume viscosity is constant and equal to
   !> the reference volume viscosity \ref viscv0.
   !>  - 1 indicates that the volume viscosity is variable: its
-  !> variation law must be specified in the user subroutine \ref uscfpv.
+  !> variation law must be specified in the user subroutine \ref cs_user_physical_properties.
   !>
   !> Always useful.
   !> The volume viscosity \f$\kappa\f$ is defined by the formula expressing the stress:
@@ -638,11 +599,6 @@ module ppincl
   !>
   integer, save :: iviscv
 
-  !> indicates the equation of state. Only perfect gas with a constant adiabatic
-  !>  coefficient, \ref ieos=1 is available, but the user can complete the file
-  !> \ref cfther.f90, which is not a user subroutine, to add new equations of state.
-  integer, save :: ieos
-
   !> \}
   !> \}
 
@@ -660,14 +616,10 @@ module ppincl
   !> alias for boundary conditions
   integer, save :: irunh
 
-  ! ---- Proprietes supplementaires par phase
-  !> additional property
-  double precision, save :: cv0
-
   !> reference volume viscosity (noted \f$\kappa\f$ in the equation
   !> expressing \f$\tens{\sigma}\f$ in the paragraph dedicated to \ref iviscv)
   !> always useful, it is the used value, unless the user specifies the volume
-  !> viscosity in the user subroutine \ref uscfpv
+  !> viscosity in the user subroutine \ref cs_user_physical_properties
   double precision, save :: viscv0
 
   !> pressure predicion by an evolution equation
@@ -718,17 +670,40 @@ module ppincl
   !> \addtogroup cooling
   !> \{
 
+  !> Molar mass ratio = Ratio of the molar mass (H2O) over the molar mass (air)
+  double precision :: molmass_rat
+  parameter(molmass_rat = 0.622d0)
+
   !> \defgroup cool_transported  Transported variables
 
   !> \addtogroup cool_transported
   !> \{
 
-  !> pointer do define the air temperature in array isca(itemp4)
-  integer, save :: itemp4
+  !> Pointer to define the mass fraction of the water in array isca(iymw)
+  integer, save :: iymw
 
-  !> pointer do define the air humidity in array isca(ihumid)
+  !> Pointer to define mass fraction of liquid water which is injected in the packing
+  integer, save :: iyml
+
+  !> Pointer to define mass fraction of liquid water which is injected in the rain zones
+  integer, save :: iy_p_l
+
+  !> Pointer to define the temperature of liquide water in rain zone (in fact Y_l.T_l)
+  integer, save :: it_p_l
+
+  !> Pointer to define the temperature (property, deduced from enthalpy)
+  !> of liquid water which is injected in the packing
+  integer, save :: itml
+
+  !> Pointer to define the field enthalpy of liquid water
+  integer, save :: ihml
+
+  !> Pointer to define the air humidity
   integer, save :: ihumid
 
+  !> Pointer to define the vertical velocity of liquid water
+  integer, save :: ivertvel
+
   !> \}
   !> \}
 
@@ -737,22 +712,20 @@ module ppincl
   !> \addtogroup enthalpy
   !> \{
 
-  !> enthalpy, if transported
+  !> enthalpy, if transported or if deduced
   integer, save :: ihm
 
+  !> \anchor srrom
   !> with gas combustion, pulverised coal or the electric module, \ref srrom
   !> is the sub-relaxation coefficient for the density, following the formula:
   !> \f$\rho^{n+1}$\,=\,srrom\,$\rho^n$+(1-srrom)\,$\rho^{n+1}\f$
   !> hence, with a zero value, there is no sub-relaxation.
-  !> With combustion and pulversied coal, \ref srrom is initialised to \ref -grand
-  !> and the user must specify a proper value through the Interface or the
-  !> initialisation subroutines (\ref usd3p1,
-  !> \ref usebu1, \ref uslwc1, \ref uscpi1 or
-  !> \ref uscpl1). With the electric module, \ref srrom is initialised in to 0
-  !> and may be modified by the user in \ref useli1.
-  !> With gas combustion, pulverised coal or electric arcs, \ref ssrom is
+  !> With combustion and pulverized coal, \ref srrom is initialized to
+  !> \ref cstnum::grand "-grand" and the user must specify a proper value through
+  !> the Interface or the initialization subroutine (\ref cs_user_combustion).
+  !> With gas combustion, pulverised coal or electric arcs, \ref srrom is
   !> automatically used after the second time-step. With Joule effect,
-  !> the user decides whether or not it will be used in \ref uselph
+  !> the user decides whether or not it will be used in \ref cs_user_physical_properties
   !> from the coding law giving the density.
   !>
   !> Always useful with gas combustion, pulverized coal or the electric module.
@@ -769,11 +742,13 @@ module ppincl
   !> imposed flow zone indicator
   !> in a way which is similar to the process described in the framework of the EBU module,
   !> the user chooses for every inlet face to impose the mass flow or not
-  !> (\ref iqimp(izone)=1 or 0). If the mass flow is imposed, the user
-  !> must set the air mass flow value \ref qimpat(izone), its direction in
-  !> \ref rcodcl(ifac,iu), \ref rcodcl(ifac,iv) and \ref rcodcl(ifac,iw) and the incoming
-  !> air temperature \ref timpat(izone) in Kelvin. If the velocity is imposed, he has to
-  !> set  \ref rcodcl(ifac,iu), \ref rcodcl(ifac,iv) and \ref rcodcl(ifac,iw).
+  !> (\ref iqimp "iqimp"(izone)=1 or 0). If the mass flow is imposed, the user
+  !> must set the air mass flow value \ref cpincl::qimpat "qimpat"(izone), its direction in
+  !> \ref rcodcl "rcodcl"(ifac,\ref iu), \ref rcodcl "rcodcl"(ifac,\ref iv)
+  !> and \ref rcodcl "rcodcl"(ifac,\ref iw) and the incoming
+  !> air temperature \ref cpincl::timpat "timpat"(izone) in Kelvin.
+  !> If the velocity is imposed, he has to set  \ref rcodcl "rcodcl"(ifac,\ref iu),
+  !> \ref rcodcl "rcodcl"(ifac,\ref iv), and \ref rcodcl "rcodcl"(ifac,\ref iw).
   integer, save ::          iqimp(nozppm)
 
   !> condition type turbulence indicator
@@ -794,13 +769,81 @@ module ppincl
   !> number of boundary zones on current process
   integer, save :: nzfppp
 
-  !> list of bondary zones index
+  !> list of boundary zones index
   integer, save :: ilzppp(nbzppm)
 
   !> \}
 
-  !===========================================================================
+  !=============================================================================
 
   !> \}
 
+  !=============================================================================
+
+  interface
+
+    !---------------------------------------------------------------------------
+
+    !> \cond DOXYGEN_SHOULD_SKIP_THIS
+
+    !---------------------------------------------------------------------------
+
+    ! Interface to C function retrieving pointers to members of the
+    ! global physical model flags
+
+    subroutine cs_f_physical_model_get_pointers(p_ippmod)    &
+      bind(C, name='cs_f_physical_model_get_pointers')
+      use, intrinsic :: iso_c_binding
+      implicit none
+      type(c_ptr), intent(out) :: p_ippmod
+    end subroutine cs_f_physical_model_get_pointers
+
+    !---------------------------------------------------------------------------
+
+    ! Interface to C function retrieving pointers to members of the
+    ! global physical model flags
+
+    subroutine cs_f_combustion_model_get_pointers(p_isoot)    &
+      bind(C, name='cs_f_combustion_model_get_pointers')
+      use, intrinsic :: iso_c_binding
+      implicit none
+      type(c_ptr), intent(out) :: p_isoot
+    end subroutine cs_f_combustion_model_get_pointers
+
+    !---------------------------------------------------------------------------
+
+    !> (DOXYGEN_SHOULD_SKIP_THIS) \endcond
+
+    !---------------------------------------------------------------------------
+
+  end interface
+
+  !=============================================================================
+
+contains
+
+  !=============================================================================
+
+  !> \brief Initialize Fortran physical models properties API.
+  !> This maps Fortran pointers to global C variables.
+
+  subroutine pp_models_init
+
+    use, intrinsic :: iso_c_binding
+    implicit none
+
+    ! Local variables
+
+    type(c_ptr) :: p_ippmod, p_isoot
+
+    call cs_f_physical_model_get_pointers(p_ippmod)
+    call cs_f_combustion_model_get_pointers(p_isoot)
+
+    call c_f_pointer(p_ippmod, ippmod, [nmodmx])
+    call c_f_pointer(p_isoot, isoot)
+
+  end subroutine pp_models_init
+
+  !=============================================================================
+
 end module ppincl
