@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -27,54 +27,46 @@
 !> \file reseps.f90
 !>
 !> \brief This subroutine performs the solving of epsilon in
-!> \f$ R_{ij} - \varepsilon \f$ RANS turbulence model.
+!>        \f$ R_{ij} - \varepsilon \f$ RANS turbulence model.
 !>
 !-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
 ! Arguments
 !______________________________________________________________________________.
-!  mode           name          role                                           !
+!  mode           name          role
 !______________________________________________________________________________!
 !> \param[in]     nvar          total number of variables
 !> \param[in]     nscal         total number of scalars
 !> \param[in]     ncepdp        number of cells with head loss
 !> \param[in]     ncesmp        number of cells with mass source term
-!> \param[in]     ivar          variable number
-!> \param[in]     isou          local variable number (7 here)
-!> \param[in]     ipp           index for writing
 !> \param[in]     icepdc        index of cells with head loss
 !> \param[in]     icetsm        index of cells with mass source term
 !> \param[in]     itypsm        type of mass source term for each variable
-!>                               (see \ref ustsma)
+!>                               (see \ref cs_user_mass_source_terms)
 !> \param[in]     dt            time step (per cell)
-!> \param[in,out] rtp, rtpa     calculated variables at cell centers
-!>                               (at current and previous time steps)
-!> \param[in]     propce        physical properties at cell centers
-!> \param[in]     gradv         tableau de travail pour terme grad
-!>                                 de vitesse     uniqt pour iturb=31
-!> \param[in]     produc        tableau de travail pour production
-!>                              (sans rho volume) uniqt pour iturb=30
-!> \param[in]     gradro        tableau de travail pour grad rom
+!> \param[in]     gradv         work array for the term grad
+!>                               of velocity only for iturb=31
+!> \param[in]     produc        work array for production (without
+!>                               rho volume) only for iturb=30
+!> \param[in]     gradro        work array for \f$ \grad{rom} \f$
 !> \param[in]     ckupdc        work array for the head loss
 !> \param[in]     smacel        value associated to each variable in the mass
-!>                               source terms or mass rate (see \ref ustsma)
-!> \param[in]     viscf         visc*surface/dist aux faces internes
-!> \param[in]     viscb         visc*surface/dist aux faces de bord
-!> \param[in]     tslagr        coupling term for lagrangian
+!>                               source terms or mass rate
+!>                               (see \ref cs_user_mass_source_terms)
+!> \param[in]     viscf         visc*surface/dist at internal faces
+!> \param[in]     viscb         visc*surface/dist at edge faces
 !> \param[in]     smbr          working array
 !> \param[in]     rovsdt        working array
 !_______________________________________________________________________________
 
 subroutine reseps &
  ( nvar   , nscal  , ncepdp , ncesmp ,                            &
-   ivar   , isou   , ipp    ,                                     &
    icepdc , icetsm , itypsm ,                                     &
-   dt     , rtp    , rtpa   , propce ,                            &
+   dt     ,                                                       &
    gradv  , produc , gradro ,                                     &
    ckupdc , smacel ,                                              &
    viscf  , viscb  ,                                              &
-   tslagr ,                                                       &
    smbr   , rovsdt )
 
 !===============================================================================
@@ -96,6 +88,7 @@ use mesh
 use field
 use field_operator
 use cs_f_interfaces
+use cs_c_bindings
 
 !===============================================================================
 
@@ -105,44 +98,45 @@ implicit none
 
 integer          nvar   , nscal
 integer          ncepdp , ncesmp
-integer          ivar   , isou   , ipp
 
 integer          icepdc(ncepdp)
 integer          icetsm(ncesmp), itypsm(ncesmp,nvar)
 
-double precision dt(ncelet), rtp(ncelet,nflown:nvar), rtpa(ncelet,nflown:nvar)
-double precision propce(ncelet,*)
+double precision dt(ncelet)
 double precision produc(6,ncelet), gradv(3, 3, ncelet)
-double precision gradro(ncelet,3)
-double precision ckupdc(ncepdp,6), smacel(ncesmp,nvar)
+double precision gradro(3,ncelet)
+double precision ckupdc(6,ncepdp), smacel(ncesmp,nvar)
 double precision viscf(nfac), viscb(nfabor)
-double precision tslagr(ncelet,*)
 double precision smbr(ncelet), rovsdt(ncelet)
 
 ! Local variables
 
+integer          init
+integer          ivar
 integer          iel
 integer          iiun
-integer          ipcvis, ipcvst, iflmas, iflmab
+integer          iflmas, iflmab
 integer          nswrgp, imligp, iwarnp
-integer          iconvp, idiffp, ndircp, ireslp
-integer          nitmap, nswrsp, ircflp, ischcp, isstpp, iescap
-integer          imgrp , ncymxp, nitmfp
-integer          iptsta
+integer          iconvp, idiffp, ndircp
+integer          nswrsp, ircflp, ischcp, isstpp, iescap
+integer          st_prv_id
 integer          imucpp, idftnp, iswdyp
 integer          icvflb
 integer          ivoid(1)
+integer          key_t_ext_id
+integer          iroext
 double precision blencp, epsilp, epsrgp, climgp, extrap, relaxp
 double precision epsrsp, alpha3
 double precision trprod , trrij
 double precision tseps , kseps , ceps2
 double precision tuexpe, thets , thetv , thetap, thetp1
 double precision prdeps, xttdrb, xttke , xttkmg
+double precision normp
 
 double precision rvoid(1)
 
 double precision, allocatable, dimension(:) :: w1
-double precision, allocatable, dimension(:) :: w7, w9
+double precision, allocatable, dimension(:) :: w7, cprod
 double precision, allocatable, dimension(:) :: dpvar
 double precision, allocatable, dimension(:,:) :: viscce
 double precision, allocatable, dimension(:,:) :: weighf
@@ -151,42 +145,72 @@ double precision, dimension(:), pointer :: imasfl, bmasfl
 double precision, dimension(:), pointer ::  crom, cromo
 double precision, dimension(:), pointer :: coefap, coefbp, cofafp, cofbfp
 double precision, dimension(:,:), pointer :: visten
+double precision, dimension(:), pointer :: cvar_ep, cvara_ep, cvar_al
+double precision, dimension(:), pointer :: cvara_r11, cvara_r22, cvara_r33
+double precision, dimension(:), pointer :: cvara_r12, cvara_r13, cvara_r23
+double precision, dimension(:,:), pointer :: cvara_rij, lagr_st_rij
+double precision, dimension(:), pointer :: viscl, visct, c_st_prv
 
 character(len=80) :: label
 
+type(var_cal_opt) :: vcopt
+
 !===============================================================================
 
 !===============================================================================
-! 1. Initialisation
+! 1. Initialization
 !===============================================================================
 
+! Time extrapolation?
+call field_get_key_id("time_extrapolated", key_t_ext_id)
+
+call field_get_key_int(icrom, key_t_ext_id, iroext)
+
+ivar = iep
+
 ! Allocate work arrays
 allocate(w1(ncelet))
-allocate(w9(ncelet))
+allocate(cprod(ncelet))
 allocate(dpvar(ncelet))
 allocate(viscce(6,ncelet))
 allocate(weighf(2,nfac))
 allocate(weighb(nfabor))
 
-if(iwarni(ivar).ge.1) then
+call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+
+if (vcopt%iwarni.ge.1) then
   call field_get_label(ivarfl(ivar), label)
   write(nfecra,1000) label
 endif
 
 call field_get_val_s(icrom, crom)
-ipcvis = ipproc(iviscl)
-ipcvst = ipproc(ivisct)
-call field_get_key_int(ivarfl(iu), kimasf, iflmas)
-call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
-call field_get_val_s(iflmas, imasfl)
-call field_get_val_s(iflmab, bmasfl)
+call field_get_val_s(iviscl, viscl)
+call field_get_val_s(ivisct, visct)
 
+call field_get_val_s(ivarfl(iep), cvar_ep)
+call field_get_val_prev_s(ivarfl(iep), cvara_ep)
+if (iturb.eq.32) call field_get_val_s(ivarfl(ial), cvar_al)
+if(irijco.eq.1) then
+  call field_get_val_prev_v(ivarfl(irij), cvara_rij)
+else
+  call field_get_val_prev_s(ivarfl(ir11), cvara_r11)
+  call field_get_val_prev_s(ivarfl(ir22), cvara_r22)
+  call field_get_val_prev_s(ivarfl(ir33), cvara_r33)
+  call field_get_val_prev_s(ivarfl(ir12), cvara_r12)
+  call field_get_val_prev_s(ivarfl(ir13), cvara_r13)
+  call field_get_val_prev_s(ivarfl(ir23), cvara_r23)
+endif
 call field_get_coefa_s(ivarfl(ivar), coefap)
 call field_get_coefb_s(ivarfl(ivar), coefbp)
 call field_get_coefaf_s(ivarfl(ivar), cofafp)
 call field_get_coefbf_s(ivarfl(ivar), cofbfp)
 
-! Constante Ce2, qui vaut CE2 pour ITURB=30 et CSSGE2 pour ITRUB=31
+call field_get_key_int(ivarfl(iu), kimasf, iflmas)
+call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
+call field_get_val_s(iflmas, imasfl)
+call field_get_val_s(iflmab, bmasfl)
+
+! Constant Ce2, which worths Ce2 for iturb=30 and CSSGE2 for itrub=31
 if (iturb.eq.30) then
   ceps2 = ce2
 elseif (iturb.eq.31) then
@@ -195,20 +219,22 @@ else
   ceps2 = cebme2
 endif
 
-! S pour Source, V pour Variable
+! S as Source, V as Variable
 thets  = thetst
-thetv  = thetav(ivar )
+thetv  = vcopt%thetav
 
-if (isto2t.gt.0.and.iroext.gt.0) then
+call field_get_key_int(ivarfl(ivar), kstprv, st_prv_id)
+if (st_prv_id.ge.0) then
+  call field_get_val_s(st_prv_id, c_st_prv)
+else
+  c_st_prv=> null()
+endif
+
+if (st_prv_id.ge.0.and.iroext.gt.0) then
   call field_get_val_prev_s(icrom, cromo)
 else
   call field_get_val_s(icrom, cromo)
 endif
-if(isto2t.gt.0) then
-  iptsta = ipproc(itstua)
-else
-  iptsta = 0
-endif
 
 do iel = 1, ncel
   smbr(iel) = 0.d0
@@ -222,30 +248,29 @@ enddo
 !===============================================================================
 
 call cs_user_turbulence_source_terms &
-!===================================
  ( nvar   , nscal  , ncepdp , ncesmp ,                            &
    ivarfl(ivar)    ,                                              &
    icepdc , icetsm , itypsm ,                                     &
    ckupdc , smacel ,                                              &
    smbr   , rovsdt )
 
-!     Si on extrapole les T.S.
-if(isto2t.gt.0) then
+!     If we extrapolate the source terms
+if (st_prv_id.ge.0) then
   do iel = 1, ncel
-!       Sauvegarde pour echange
-    tuexpe = propce(iel,iptsta+isou-1)
-!       Pour la suite et le pas de temps suivant
-    propce(iel,iptsta+isou-1) = smbr(iel)
-!       Second membre du pas de temps precedent
-!       On suppose -ROVSDT > 0 : on implicite
-!          le terme source utilisateur (le reste)
-    smbr(iel) = rovsdt(iel)*rtpa(iel,ivar) - thets*tuexpe
-!       Diagonale
+    !       Save for exchange
+    tuexpe = c_st_prv(iel)
+    !       For the continuation and the next time step
+    c_st_prv(iel) = smbr(iel)
+    !       Second member of previous time step
+    !       We suppose -rovsdt > 0: we implicit
+    !          the user source term (the rest)
+    smbr(iel) = rovsdt(iel)*cvara_ep(iel) - thets*tuexpe
+    !       Diagonal
     rovsdt(iel) = - thetv*rovsdt(iel)
   enddo
 else
   do iel = 1, ncel
-    smbr(iel)   = rovsdt(iel)*rtpa(iel,ivar) + smbr(iel)
+    smbr(iel)   = rovsdt(iel)*cvara_ep(iel) + smbr(iel)
     rovsdt(iel) = max(-rovsdt(iel),zero)
   enddo
 endif
@@ -254,21 +279,29 @@ endif
 ! 3. Lagrangian source terms
 !===============================================================================
 
-!     Ordre 2 non pris en compte
+!     Second order is not taken into account
 if (iilagr.eq.2 .and. ltsdyn.eq.1) then
 
-  do iel = 1, ncel
-    ! Ts sur eps
-    tseps = -0.5d0 * ( tslagr(iel,itsr11)                        &
-                     + tslagr(iel,itsr22)                        &
-                     + tslagr(iel,itsr33) )
-    ! rapport k/eps
-    kseps = 0.5d0 * ( rtpa(iel,ir11)                           &
-                    + rtpa(iel,ir22)                           &
-                    + rtpa(iel,ir33) )                         &
-                    / rtpa(iel,iep)
+  call field_get_val_v_by_name('rij_st_lagr', lagr_st_rij)
 
-    smbr(iel)   = smbr(iel) + ce4 *tseps *rtpa(iel,iep) /kseps
+  do iel = 1, ncel
+    ! Source terms with eps
+    tseps = -0.5d0 * ( lagr_st_rij(1,iel)                        &
+                     + lagr_st_rij(2,iel)                        &
+                     + lagr_st_rij(3,iel))
+    ! quotient k/eps
+    if (irijco.eq.1) then
+      kseps = 0.5d0 * ( cvara_rij(1,iel)                         &
+                      + cvara_rij(2,iel)                         &
+                      + cvara_rij(3,iel) )                       &
+                      / cvara_ep(iel)
+    else
+      kseps = 0.5d0 * ( cvara_r11(iel)                           &
+                      + cvara_r22(iel)                           &
+                      + cvara_r33(iel) )                         &
+                      / cvara_ep(iel)
+    endif
+    smbr(iel)   = smbr(iel) + ce4 *tseps *cvara_ep(iel) /kseps
     rovsdt(iel) = rovsdt(iel) + max( (-ce4*tseps/kseps) , zero)
   enddo
 
@@ -280,24 +313,22 @@ endif
 
 if (ncesmp.gt.0) then
 
-!       Entier egal a 1 (pour navsto : nb de sur-iter)
+  ! Integer equal to 1 (forr navsto: nb of sur-iter)
   iiun = 1
 
-!       On incremente SMBR par -Gamma RTPA et ROVSDT par Gamma (*theta)
+  ! We increment smbr with -Gamma.var_prev. and rovsdt with Gamma
   call catsma &
-  !==========
- ( ncelet , ncel   , ncesmp , iiun   , isto2t , thetv ,           &
+ ( ncelet , ncel   , ncesmp , iiun   , isto2t ,                   &
    icetsm , itypsm(:,ivar)  ,                                     &
-   volume , rtpa(:,ivar)    , smacel(:,ivar)   , smacel(:,ipr) ,  &
+   volume , cvara_ep        , smacel(:,ivar)   , smacel(:,ipr) ,  &
    smbr   , rovsdt , w1 )
 
-!       Si on extrapole les TS on met Gamma Pinj dans PROPCE
-  if(isto2t.gt.0) then
+  ! If we extrapolate the source terms, we put Gamma Pinj in c_st_prv
+  if (st_prv_id.ge.0) then
     do iel = 1, ncel
-      propce(iel,iptsta+isou-1) =                                 &
-      propce(iel,iptsta+isou-1) + w1(iel)
+      c_st_prv(iel) = c_st_prv(iel) + w1(iel)
     enddo
-!       Sinon on le met directement dans SMBR
+  ! Otherwise we put it directly in smbr
   else
     do iel = 1, ncel
       smbr(iel) = smbr(iel) + w1(iel)
@@ -307,12 +338,12 @@ if (ncesmp.gt.0) then
 endif
 
 !===============================================================================
-! 5. Non-stationary term
+! 5. Unsteady term
 !===============================================================================
 
 do iel = 1, ncel
   rovsdt(iel) = rovsdt(iel)                                          &
-              + istat(ivar)*(crom(iel)/dt(iel))*volume(iel)
+              + vcopt%istat*(crom(iel)/dt(iel))*cell_f_vol(iel)
 enddo
 
 !===============================================================================
@@ -320,30 +351,44 @@ enddo
 !    Dissipation (rho*Ce2.epsilon/k*epsilon)
 !===============================================================================
 
-if (isto2t.gt.0) then
+if (st_prv_id.ge.0) then
   thetap = thetv
 else
   thetap = 1.d0
 endif
 
-! ---> Calcul de la trace de la production, suivant qu'on est en
-!     Rij standard ou en SSG (utilisation de PRODUC ou GRDVIT)
+! ---> Calculation the production trace, depending we are in standard
+!     Rij or in SSG (use of produc or grdvit)
 if (iturb.eq.30) then
   do iel = 1, ncel
-    w9(iel) = 0.5d0*(produc(1,iel)+produc(2,iel)+produc(3,iel))
+    cprod(iel) = 0.5d0*(produc(1,iel)+produc(2,iel)+produc(3,iel))
   enddo
 else
-  do iel = 1, ncel
-    w9(iel) = -( rtpa(iel,ir11)*gradv(1, 1, iel) +               &
-                 rtpa(iel,ir12)*gradv(2, 1, iel) +               &
-                 rtpa(iel,ir13)*gradv(3, 1, iel) +               &
-                 rtpa(iel,ir12)*gradv(1, 2, iel) +               &
-                 rtpa(iel,ir22)*gradv(2, 2, iel) +               &
-                 rtpa(iel,ir23)*gradv(3, 2, iel) +               &
-                 rtpa(iel,ir13)*gradv(1, 3, iel) +               &
-                 rtpa(iel,ir23)*gradv(2, 3, iel) +               &
-                 rtpa(iel,ir33)*gradv(3, 3, iel) )
-  enddo
+  if (irijco.eq.1) then
+    do iel = 1, ncel
+      cprod(iel) = -( cvara_rij(1,iel)*gradv(1, 1, iel) +               &
+                   cvara_rij(4,iel)*gradv(2, 1, iel) +               &
+                   cvara_rij(6,iel)*gradv(3, 1, iel) +               &
+                   cvara_rij(4,iel)*gradv(1, 2, iel) +               &
+                   cvara_rij(2,iel)*gradv(2, 2, iel) +               &
+                   cvara_rij(5,iel)*gradv(3, 2, iel) +               &
+                   cvara_rij(6,iel)*gradv(1, 3, iel) +               &
+                   cvara_rij(5,iel)*gradv(2, 3, iel) +               &
+                   cvara_rij(3,iel)*gradv(3, 3, iel) )
+    enddo
+  else
+    do iel = 1, ncel
+      cprod(iel) = -( cvara_r11(iel)*gradv(1, 1, iel) +               &
+                   cvara_r12(iel)*gradv(2, 1, iel) +               &
+                   cvara_r13(iel)*gradv(3, 1, iel) +               &
+                   cvara_r12(iel)*gradv(1, 2, iel) +               &
+                   cvara_r22(iel)*gradv(2, 2, iel) +               &
+                   cvara_r23(iel)*gradv(3, 2, iel) +               &
+                   cvara_r13(iel)*gradv(1, 3, iel) +               &
+                   cvara_r23(iel)*gradv(2, 3, iel) +               &
+                   cvara_r33(iel)*gradv(3, 3, iel) )
+    enddo
+  endif
 endif
 
 
@@ -351,56 +396,65 @@ endif
 if (iturb.eq.32) then
 
   do iel = 1, ncel
-    ! Demi-traces
-    trprod = w9(iel)
-    trrij  = 0.5d0 * (rtpa(iel,ir11) + rtpa(iel,ir22) + rtpa(iel,ir33))
-    ! Calcul de l echelle de temps de Durbin
-    xttke  = trrij/rtpa(iel,iep)
-    xttkmg = xct*sqrt(propce(iel,ipcvis)/crom(iel)           &
-                                        /rtpa(iel,iep))
+    ! Half-traces
+    trprod = cprod(iel)
+    if(irijco.eq.1) then
+      trrij  = 0.5d0 * (cvara_rij(1,iel) + cvara_rij(2,iel) + cvara_rij(3,iel))
+    else
+      trrij  = 0.5d0 * (cvara_r11(iel) + cvara_r22(iel) + cvara_r33(iel))
+    endif
+    ! Calculation of the Durbin time scale
+    xttke  = trrij/cvara_ep(iel)
+    xttkmg = xct*sqrt(viscl(iel)/crom(iel)/cvara_ep(iel))
     xttdrb = max(xttke,xttkmg)
 
-    prdeps = trprod/rtpa(iel,iep)
-    alpha3 = rtp(iel,ial)**3
+    prdeps = trprod/cvara_ep(iel)
+    alpha3 = cvar_al(iel)**3
 
     ! Production (explicit)
     ! Compute of C_eps_1'
-    w1(iel) = cromo(iel)*volume(iel)*                         &
+    w1(iel) = cromo(iel)*cell_f_vol(iel)*                                 &
               ce1*(1.d0+xa1*(1.d0-alpha3)*prdeps)*trprod/xttdrb
 
 
     ! Dissipation (implicit)
-    smbr(iel) = smbr(iel) - crom(iel)*volume(iel)*           &
-                             ceps2*rtpa(iel,iep)/xttdrb
+    smbr(iel) = smbr(iel) - crom(iel)*cell_f_vol(iel)*                    &
+                             ceps2*cvara_ep(iel)/xttdrb
 
     rovsdt(iel) = rovsdt(iel)                                         &
-                + ceps2*crom(iel)*volume(iel)*thetap/xttdrb
+                + ceps2*crom(iel)*cell_f_vol(iel)*thetap/xttdrb
   enddo
 
 ! SSG and LRR
 else
 
   do iel = 1, ncel
-    ! Demi-traces
-    trprod = w9(iel)
-    trrij  = 0.5d0 * (rtpa(iel,ir11) + rtpa(iel,ir22) + rtpa(iel,ir33))
-    xttke  = trrij/rtpa(iel,iep)
-    ! Production (explicit)
-    w1(iel) = cromo(iel)*volume(iel)*ce1/xttke*trprod
+    ! Half-traces
+    trprod = cprod(iel)
+    if(irijco.eq.1) then
+      trrij  = 0.5d0 * (cvara_rij(1,iel) + cvara_rij(2,iel) + cvara_rij(3,iel))
+    else
+      trrij  = 0.5d0 * (cvara_r11(iel) + cvara_r22(iel) + cvara_r33(iel))
+    endif
+    xttke  = trrij/cvara_ep(iel)
+    ! Production (explicit, a part might be implicit)
+    rovsdt(iel) = rovsdt(iel)                                        &
+                + max(- cromo(iel)*cell_f_vol(iel)*ce1*trprod/trrij, 0.d0)
+    w1(iel) = cromo(iel)*cell_f_vol(iel)*ce1/xttke*trprod
 
     ! Dissipation (implicit)
-    smbr(iel) = smbr(iel)                              &
-              - crom(iel)*volume(iel)*ceps2*rtpa(iel,iep)**2/trrij
+    smbr(iel) = smbr(iel)                                            &
+              - crom(iel)*cell_f_vol(iel)*ceps2*cvara_ep(iel)**2/trrij
     rovsdt(iel) = rovsdt(iel)                                        &
-                + ceps2*crom(iel)*volume(iel)/xttke*thetap
+                + ceps2*crom(iel)*cell_f_vol(iel)/xttke*thetap
   enddo
 
 endif
 
 ! Extrapolation of source terms (2nd order in time)
-if (isto2t.gt.0) then
+if (st_prv_id.ge.0) then
   do iel = 1, ncel
-    propce(iel,iptsta+isou-1) = propce(iel,iptsta+isou-1) + w1(iel)
+    c_st_prv(iel) = c_st_prv(iel) + w1(iel)
   enddo
 else
   do iel = 1, ncel
@@ -412,6 +466,7 @@ endif
 ! 7. Buoyancy term
 !===============================================================================
 
+!FIXME use beta ... WARNING
 if (igrari.eq.1) then
 
   ! Allocate a work array
@@ -421,17 +476,21 @@ if (igrari.eq.1) then
     w7(iel) = 0.d0
   enddo
 
-  call rijthe(nscal, ivar, rtpa, gradro, w7)
-  !==========
+  ! Note that w7 is positive.
+  if(irijco.eq.1) then
+    call rijtheps(nscal, gradro, w7)
+  else
+    call rijthe(nscal, ivar, gradro, w7)
+  endif
 
   ! Extrapolation of source terms (2nd order in time)
-  if (isto2t.gt.0) then
+  if (st_prv_id.ge.0) then
     do iel = 1, ncel
-      propce(iel,iptsta+isou-1) = propce(iel,iptsta+isou-1) + w7(iel)
+      c_st_prv(iel) = c_st_prv(iel) + w7(iel) * cell_f_vol(iel)
     enddo
   else
     do iel = 1, ncel
-      smbr(iel) = smbr(iel) + w7(iel)
+      smbr(iel) = smbr(iel) + w7(iel) * cell_f_vol(iel)
     enddo
   endif
 
@@ -445,23 +504,22 @@ endif
 !===============================================================================
 
 ! Symmetric tensor diffusivity (GGDH)
-if (idften(ivar).eq.6) then
+if (iand(vcopt%idften, ANISOTROPIC_DIFFUSION).ne.0) then
 
   call field_get_val_v(ivsten, visten)
 
   do iel = 1, ncel
-    viscce(1,iel) = visten(1,iel)/sigmae + propce(iel,ipcvis)
-    viscce(2,iel) = visten(2,iel)/sigmae + propce(iel,ipcvis)
-    viscce(3,iel) = visten(3,iel)/sigmae + propce(iel,ipcvis)
+    viscce(1,iel) = visten(1,iel)/sigmae + viscl(iel)
+    viscce(2,iel) = visten(2,iel)/sigmae + viscl(iel)
+    viscce(3,iel) = visten(3,iel)/sigmae + viscl(iel)
     viscce(4,iel) = visten(4,iel)/sigmae
     viscce(5,iel) = visten(5,iel)/sigmae
     viscce(6,iel) = visten(6,iel)/sigmae
   enddo
 
-  iwarnp = iwarni(ivar)
+  iwarnp = vcopt%iwarni
 
   call vitens &
-  !==========
  ( viscce , iwarnp ,             &
    weighf , weighb ,             &
    viscf  , viscb  )
@@ -470,11 +528,10 @@ if (idften(ivar).eq.6) then
 else
 
   do iel = 1, ncel
-    w1(iel) = propce(iel,ipcvis) + idifft(ivar)*propce(iel,ipcvst)/sigmae
+    w1(iel) = viscl(iel) + vcopt%idifft*visct(iel)/sigmae
   enddo
 
   call viscfa                    &
-  !==========
  ( imvisf ,                      &
    w1     ,                      &
    viscf  , viscb  )
@@ -485,62 +542,58 @@ endif
 ! 9. Solving
 !===============================================================================
 
-if (isto2t.gt.0) then
+if (st_prv_id.ge.0) then
   thetp1 = 1.d0 + thets
   do iel = 1, ncel
-    smbr(iel) = smbr(iel) + thetp1*propce(iel,iptsta+isou-1)
+    smbr(iel) = smbr(iel) + thetp1*c_st_prv(iel)
   enddo
 endif
 
-iconvp = iconv (ivar)
-idiffp = idiff (ivar)
-ndircp = ndircl(ivar)
-ireslp = iresol(ivar)
-nitmap = nitmax(ivar)
-nswrsp = nswrsm(ivar)
-nswrgp = nswrgr(ivar)
-imligp = imligr(ivar)
-ircflp = ircflu(ivar)
-ischcp = ischcv(ivar)
-isstpp = isstpc(ivar)
+iconvp = vcopt%iconv
+idiffp = vcopt%idiff
+ndircp = vcopt%ndircl
+nswrsp = vcopt%nswrsm
+nswrgp = vcopt%nswrgr
+imligp = vcopt%imligr
+ircflp = vcopt%ircflu
+ischcp = vcopt%ischcv
+isstpp = vcopt%isstpc
 iescap = 0
 imucpp = 0
-idftnp = idften(ivar)
-iswdyp = iswdyn(ivar)
-imgrp  = imgr  (ivar)
-ncymxp = ncymax(ivar)
-nitmfp = nitmgf(ivar)
-iwarnp = iwarni(ivar)
-blencp = blencv(ivar)
-epsilp = epsilo(ivar)
-epsrsp = epsrsm(ivar)
-epsrgp = epsrgr(ivar)
-climgp = climgr(ivar)
-extrap = extrag(ivar)
-relaxp = relaxv(ivar)
+idftnp = vcopt%idften
+iswdyp = vcopt%iswdyn
+iwarnp = vcopt%iwarni
+blencp = vcopt%blencv
+epsilp = vcopt%epsilo
+epsrsp = vcopt%epsrsm
+epsrgp = vcopt%epsrgr
+climgp = vcopt%climgr
+extrap = vcopt%extrag
+relaxp = vcopt%relaxv
 ! all boundary convective flux with upwind
 icvflb = 0
+normp = -1.d0
+init   = 1
 
 call codits &
-!==========
- ( idtvar , ivar   , iconvp , idiffp , ireslp , ndircp , nitmap , &
+ ( idtvar , init   , ivarfl(ivar)    , iconvp , idiffp , ndircp , &
    imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,          &
-   imgrp  , ncymxp , nitmfp , ipp    , iwarnp ,                   &
+   iwarnp , normp  ,                                              &
    blencp , epsilp , epsrsp , epsrgp , climgp , extrap ,          &
    relaxp , thetv  ,                                              &
-   rtpa(1,ivar)    , rtpa(1,ivar)    ,                            &
+   cvara_ep        , cvara_ep        ,                            &
    coefap , coefbp , cofafp , cofbfp ,                            &
    imasfl , bmasfl ,                                              &
-   viscf  , viscb  , viscce  , viscf  , viscb  , viscce  ,        &
+   viscf  , viscb  , viscf , viscb  , viscce  ,                   &
    weighf , weighb ,                                              &
    icvflb , ivoid  ,                                              &
-   rovsdt , smbr   , rtp(1,ivar)     , dpvar  ,                   &
+   rovsdt , smbr   , cvar_ep         , dpvar  ,                   &
    rvoid  , rvoid  )
 
 ! Free memory
 deallocate(w1)
-deallocate(w9)
+deallocate(cprod)
 deallocate(viscce)
 deallocate(weighf, weighb)
 
@@ -550,11 +603,11 @@ deallocate(weighf, weighb)
 
 #if defined(_CS_LANG_FR)
 
- 1000 format(/,'           RESOLUTION POUR LA VARIABLE ',A8,/)
+ 1000 format(/,'           Resolution de la variable ',A8,/)
 
 #else
 
- 1000 format(/,'           SOLVING VARIABLE ',A8           ,/)
+ 1000 format(/,'           Solving variable ',A8           ,/)
 
 #endif
 
