@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -29,20 +29,19 @@
 !> \brief This subroutine performs the solving of the Reynolds stress components
 !> in \f$ R_{ij} - \varepsilon \f$ RANS (LRR) turbulence model.
 !>
-!> Remark:
+!> \remark
 !> - isou=1 for \f$ R_{11} \f$
 !> - isou=2 for \f$ R_{22} \f$
 !> - isou=3 for \f$ R_{33} \f$
 !> - isou=4 for \f$ R_{12} \f$
 !> - isou=5 for \f$ R_{13} \f$
 !> - isou=6 for \f$ R_{23} \f$
-!>
 !-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
 ! Arguments
 !______________________________________________________________________________.
-!  mode           name          role                                           !
+!  mode           name          role
 !______________________________________________________________________________!
 !> \param[in]     nvar          total number of variables
 !> \param[in]     nscal         total number of scalars
@@ -50,41 +49,34 @@
 !> \param[in]     ncesmp        number of cells with mass source term
 !> \param[in]     ivar          variable number
 !> \param[in]     isou          local variable number (7 here)
-!> \param[in]     ipp           index for writing
 !> \param[in]     icepdc        index of cells with head loss
 !> \param[in]     icetsm        index of cells with mass source term
 !> \param[in]     itypsm        type of mass source term for each variable
-!>                               (see \ref ustsma)
+!>                               (see \ref cs_user_mass_source_terms)
 !> \param[in]     dt            time step (per cell)
-!> \param[in,out] rtp, rtpa     calculated variables at cell centers
-!>                               (at current and previous time steps)
-!> \param[in]     propce        physical properties at cell centers
-!> \param[in]     grdvit        tableau de travail pour terme grad
-!>                                 de vitesse     uniqt pour iturb=31
-!> \param[in]     produc        tableau de travail pour production
-!> \param[in]     gradro        tableau de travail pour grad rom
-!>                              (sans rho volume) uniqt pour iturb=30
+!> \param[in]     produc        work array for production
+!> \param[in]     gradro        work array for grad rom
+!>                              (without rho volume) only for iturb=30
 !> \param[in]     ckupdc        work array for the head loss
 !> \param[in]     smacel        value associated to each variable in the mass
-!>                               source terms or mass rate (see \ref ustsma)
-!> \param[in]     viscf         visc*surface/dist aux faces internes
-!> \param[in]     viscb         visc*surface/dist aux faces de bord
-!> \param[in]     tslagr        coupling term for lagrangian
-!> \param[in]     tslage        explicit source terms for the Lagrangian module
+!>                              source terms or mass rate (see
+!>                              \ref cs_user_mass_source_terms)
+!> \param[in]     viscf         visc*surface/dist at internal faces
+!> \param[in]     viscb         visc*surface/dist at edge faces
 !> \param[in]     tslagi        implicit source terms for the Lagrangian module
 !> \param[in]     smbr          working array
 !> \param[in]     rovsdt        working array
-!_______________________________________________________________________________
+!______________________________________________________________________________!
 
 subroutine resrij &
  ( nvar   , nscal  , ncepdp , ncesmp ,                            &
-   ivar   , isou   , ipp    ,                                     &
+   ivar   , isou   ,                                              &
    icepdc , icetsm , itypsm ,                                     &
-   dt     , rtp    , rtpa   , propce ,                            &
+   dt     ,                                                       &
    produc , gradro ,                                              &
    ckupdc , smacel ,                                              &
    viscf  , viscb  ,                                              &
-   tslage , tslagi ,                                              &
+   tslagi ,                                                       &
    smbr   , rovsdt )
 
 !===============================================================================
@@ -105,7 +97,9 @@ use lagran
 use mesh
 use field
 use cs_f_interfaces
+use rotation
 use turbomachinery
+use cs_c_bindings
 
 !===============================================================================
 
@@ -115,35 +109,36 @@ implicit none
 
 integer          nvar   , nscal
 integer          ncepdp , ncesmp
-integer          ivar   , isou   , ipp
+integer          ivar   , isou
 
 integer          icepdc(ncepdp)
 integer          icetsm(ncesmp), itypsm(ncesmp,nvar)
 
-double precision dt(ncelet), rtp(ncelet,nflown:nvar), rtpa(ncelet,nflown:nvar)
-double precision propce(ncelet,*)
+double precision dt(ncelet)
 double precision produc(6,ncelet)
-double precision gradro(ncelet,3)
-double precision ckupdc(ncepdp,6), smacel(ncesmp,nvar)
+double precision gradro(3,ncelet)
+double precision ckupdc(6,ncepdp), smacel(ncesmp,nvar)
 double precision viscf(nfac), viscb(nfabor)
-double precision tslage(ncelet),tslagi(ncelet)
+double precision tslagi(ncelet)
 double precision smbr(ncelet), rovsdt(ncelet)
 
 ! Local variables
 
 integer          iel
-integer          ii    , jj    , kk    , iiun
-integer          ipcvis, iflmas, iflmab
+integer          ii    , jj    , kk    , iiun, comp_id
+integer          iflmas, iflmab
 integer          nswrgp, imligp, iwarnp
-integer          iconvp, idiffp, ndircp, ireslp
-integer          nitmap, nswrsp, ircflp, ischcp, isstpp, iescap
-integer          imgrp , ncymxp, nitmfp
-integer          iptsta
+integer          iconvp, idiffp, ndircp
+integer          nswrsp, ircflp, ischcp, isstpp, iescap
+integer          st_prv_id
 integer          isoluc
 integer          imucpp, idftnp, iswdyp
-integer          indrey(3,3)
+integer          ivar_r(3,3)
 integer          icvflb
+integer          init
 integer          ivoid(1)
+integer          key_t_ext_id
+integer          iroext
 
 double precision blencp, epsilp, epsrgp, climgp, extrap, relaxp
 double precision epsrsp
@@ -152,6 +147,7 @@ double precision tuexpr, thets , thetv , thetp1
 double precision d1s3  , d2s3
 double precision ccorio, matrot(3,3)
 double precision rctse
+double precision normp
 
 double precision rvoid(1)
 
@@ -165,7 +161,14 @@ double precision, allocatable, dimension(:) :: weighb
 double precision, dimension(:), pointer :: imasfl, bmasfl
 double precision, dimension(:), pointer :: crom, cromo
 double precision, dimension(:), pointer :: coefap, coefbp, cofafp, cofbfp
-double precision, dimension(:,:), pointer :: visten
+double precision, dimension(:,:), pointer :: visten, lagr_st_rij
+double precision, dimension(:), pointer :: cvara_ep
+double precision, dimension(:), pointer :: cvara_r11, cvara_r22, cvara_r33
+double precision, dimension(:), pointer :: cvar_var, cvara_var
+double precision, dimension(:), pointer :: cvara_rik, cvara_rjk
+double precision, dimension(:), pointer :: viscl, c_st_prv
+
+type(var_cal_opt) :: vcopt
 
 !===============================================================================
 
@@ -173,6 +176,11 @@ double precision, dimension(:,:), pointer :: visten
 ! 1. Initialization
 !===============================================================================
 
+! Time extrapolation?
+call field_get_key_id("time_extrapolated", key_t_ext_id)
+
+call field_get_key_int(icrom, key_t_ext_id, iroext)
+
 ! Allocate work arrays
 allocate(w1(ncelet))
 allocate(w7(ncelet), w8(ncelet))
@@ -181,43 +189,57 @@ allocate(viscce(6,ncelet))
 allocate(weighf(2,nfac))
 allocate(weighb(nfabor))
 
-if (iwarni(ivar).ge.1) then
+call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+
+if (vcopt%iwarni.ge.1) then
   call field_get_label(ivarfl(ivar), label)
   write(nfecra,1000) label
 endif
 
 call field_get_val_s(icrom, crom)
-ipcvis = ipproc(iviscl)
+call field_get_val_s(iviscl, viscl)
 call field_get_key_int(ivarfl(iu), kimasf, iflmas)
 call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
 call field_get_val_s(iflmas, imasfl)
 call field_get_val_s(iflmab, bmasfl)
 
+call field_get_val_prev_s(ivarfl(iep), cvara_ep)
+
+call field_get_val_prev_s(ivarfl(ir11), cvara_r11)
+call field_get_val_prev_s(ivarfl(ir22), cvara_r22)
+call field_get_val_prev_s(ivarfl(ir33), cvara_r33)
+
+call field_get_val_s(ivarfl(ivar), cvar_var)
+call field_get_val_prev_s(ivarfl(ivar), cvara_var)
+
 call field_get_coefa_s(ivarfl(ivar), coefap)
 call field_get_coefb_s(ivarfl(ivar), coefbp)
 call field_get_coefaf_s(ivarfl(ivar), cofafp)
 call field_get_coefbf_s(ivarfl(ivar), cofbfp)
 
 deltij = 1.0d0
-if(isou.gt.3) then
+if (isou.gt.3) then
   deltij = 0.0d0
 endif
 d1s3 = 1.d0/3.d0
 d2s3 = 2.d0/3.d0
 
-!     S pour Source, V pour Variable
+!     S as Source, V as Variable
 thets  = thetst
-thetv  = thetav(ivar )
+thetv  = vcopt%thetav
 
-if (isto2t.gt.0.and.iroext.gt.0) then
+call field_get_key_int(ivarfl(ivar), kstprv, st_prv_id)
+if (st_prv_id.ge.0) then
+  call field_get_val_s(st_prv_id, c_st_prv)
+else
+  c_st_prv=> null()
+endif
+
+if (st_prv_id.ge.0.and.iroext.gt.0) then
   call field_get_val_prev_s(icrom, cromo)
 else
   call field_get_val_s(icrom, cromo)
 endif
-iptsta = 0
-if(isto2t.gt.0) then
-  iptsta = ipproc(itstua)
-endif
 
 do iel = 1, ncel
   smbr(iel) = 0.d0
@@ -242,30 +264,29 @@ endif
 !===============================================================================
 
 call cs_user_turbulence_source_terms &
-!===================================
  ( nvar   , nscal  , ncepdp , ncesmp ,                            &
    ivarfl(ivar)    ,                                              &
    icepdc , icetsm , itypsm ,                                     &
    ckupdc , smacel ,                                              &
    smbr   , rovsdt )
 
-!     Si on extrapole les T.S.
-if(isto2t.gt.0) then
+!     If we extrapolate the source terms
+if (st_prv_id.ge.0) then
   do iel = 1, ncel
-!       Sauvegarde pour echange
-    tuexpr = propce(iel,iptsta+isou-1)
-!       Pour la suite et le pas de temps suivant
-    propce(iel,iptsta+isou-1) = smbr(iel)
-!       Second membre du pas de temps precedent
-!       On suppose -ROVSDT > 0 : on implicite
-!          le terme source utilisateur (le reste)
-    smbr(iel) = rovsdt(iel)*rtpa(iel,ivar)  - thets*tuexpr
-!       Diagonale
+!       Save for exchange
+    tuexpr = c_st_prv(iel)
+!       For continuation and next time step
+    c_st_prv(iel) = smbr(iel)
+!       Second member of previous time step
+!       We suppose -rovsdt > 0: we implicite
+!          the user source term (the rest)
+    smbr(iel) = rovsdt(iel)*cvara_var(iel)  - thets*tuexpr
+!       Diagonal
     rovsdt(iel) = - thetv*rovsdt(iel)
   enddo
 else
   do iel = 1, ncel
-    smbr(iel)   = rovsdt(iel)*rtpa(iel,ivar) + smbr(iel)
+    smbr(iel)   = rovsdt(iel)*cvara_var(iel) + smbr(iel)
     rovsdt(iel) = max(-rovsdt(iel),zero)
   enddo
 endif
@@ -274,10 +295,17 @@ endif
 ! 3. Lagrangian source terms
 !===============================================================================
 
-!     Ordre 2 non pris en compte
+!     Second order is not taken into account
 if (iilagr.eq.2 .and. ltsdyn.eq.1) then
+  call field_get_val_v_by_name('rij_st_lagr', lagr_st_rij)
+  comp_id = isou
+  if (isou .eq. 5) then
+    comp_id = 6
+  else if (isou .eq.6) then
+    comp_id = 5
+  endif
   do iel = 1,ncel
-    smbr(iel)   = smbr(iel)   + tslage(iel)
+    smbr(iel)   = smbr(iel)   + lagr_st_rij(comp_id,iel)
     rovsdt(iel) = rovsdt(iel) + max(-tslagi(iel),zero)
   enddo
 endif
@@ -288,24 +316,22 @@ endif
 
 if (ncesmp.gt.0) then
 
-!       Entier egal a 1 (pour navsto : nb de sur-iter)
+!       Integer equal to 1 (for navsto: nb of sur-iter)
   iiun = 1
 
-!       On incremente SMBR par -Gamma RTPA et ROVSDT par Gamma (*theta)
+!       We increment smbr with -Gamma.var_prev. and rovsdt with Gamma
   call catsma &
-  !==========
- ( ncelet , ncel   , ncesmp , iiun   , isto2t , thetv  ,          &
+ ( ncelet , ncel   , ncesmp , iiun   , isto2t ,                   &
    icetsm , itypsm(:,ivar)  ,                                     &
-   volume , rtpa(:,ivar)    , smacel(:,ivar)   , smacel(:,ipr) ,  &
+   cell_f_vol , cvara_var   , smacel(:,ivar)   , smacel(:,ipr) ,  &
    smbr   ,  rovsdt , w1 )
 
-!       Si on extrapole les TS on met Gamma Pinj dans PROPCE
-  if(isto2t.gt.0) then
+!       If we extrapolate the source terms we put Gamma Pinj in c_st_prv
+  if (st_prv_id.ge.0) then
     do iel = 1, ncel
-      propce(iel,iptsta+isou-1) =                                 &
-      propce(iel,iptsta+isou-1) + w1(iel)
+      c_st_prv(iel) = c_st_prv(iel) + w1(iel)
     enddo
-!       Sinon on le met directement dans SMBR
+!       Otherwise we put it directly in smbr
   else
     do iel = 1, ncel
       smbr(iel) = smbr(iel) + w1(iel)
@@ -315,12 +341,12 @@ if (ncesmp.gt.0) then
 endif
 
 !===============================================================================
-! 5. Non-stationary term
+! 5. Unsteady term
 !===============================================================================
 
 do iel=1,ncel
   rovsdt(iel) = rovsdt(iel)                                          &
-              + istat(ivar)*(crom(iel)/dt(iel))*volume(iel)
+              + vcopt%istat*(crom(iel)/dt(iel))*cell_f_vol(iel)
 enddo
 
 
@@ -328,115 +354,113 @@ enddo
 ! 6. Production, Pressure-Strain correlation, dissipation
 !===============================================================================
 
-! ---> Calcul de k pour la suite du sous-programme
-!       on utilise un tableau de travail puisqu'il y en a...
+! ---> Calculation of k for the sub-routine continuation
+!       we use a work array
 do iel = 1, ncel
-  w8(iel) = 0.5d0 * (rtpa(iel,ir11) + rtpa(iel,ir22) + rtpa(iel,ir33))
+  w8(iel) = 0.5d0 * (cvara_r11(iel) + cvara_r22(iel) + cvara_r33(iel))
 enddo
 
-! ---> Terme source
+! ---> Source term
 
-!      (1-CRIJ2) Pij (pour toutes les composantes de Rij)
+!      (1-CRIJ2) Pij (for all components of Rij)
 
 !      DELTAIJ*(2/3.CRIJ2.P+2/3.CRIJ1.EPSILON)
-!                    (termes diagonaux pour R11, R22 et R33)
+!                    (diagonal terms for R11, R22 et R33)
 
 !      -DELTAIJ*2/3*EPSILON
 
-!     Si on extrapole les TS
-!       On modifie la partie implicite :
-!         Dans PHI1, on ne prendra que RHO CRIJ1 EPSILON/K et non pas
-!                                  RHO CRIJ1 EPSILON/K (1-2/3 DELTAIJ)
-!         Cela permet de conserver k^n au lieu de (R11^(n+1)+R22^n+R33^n)
-!         Ce choix est discutable. C'est la solution ISOLUC = 1
-!       Si on veut tout prendre en implicite (comme c'est fait
-!         en ordre 1 std), c'est la solution ISOLUC = 2
-!       -> a tester plus precisement si necessaire
+!     If we extrapolate the source terms
+!     We modify the implicit part:
+!     In PHI1, we will only take rho CRIJ1 epsilon/k and not
+!                                rho CRIJ1 epsilon/k (1-2/3 DELTAIJ)
+!     It allow to keep  k^n instead of (R11^(n+1)+R22^n+R33^n)
+!     This choice is questionable. It is the solution isoluc = 1
+!     If we want to take all as implicit (like it is done in
+!     standard first order), it is the solution isoluc = 2
+!     -> to  be tested more precisely if necessary
 
 
-!     Si on extrapole les TS
-if (isto2t.gt.0) then
+!     If we extrapolate the source terms
+if (st_prv_id.ge.0) then
 
   isoluc = 1
 
   do iel = 1, ncel
 
-!     Demi-traces de Prod et R
+    !     Half-traces of Prod and R
     trprod = 0.5d0*(produc(1,iel)+produc(2,iel)+produc(3,iel))
     trrij  = w8(iel)
 
-!     Calcul de Prod+Phi1+Phi2-Eps
-!       = rhoPij-C1rho eps/k(Rij-2/3k dij)-C2rho(Pij-1/3Pkk dij)-2/3rho eps dij
-!       Dans PROPCE :
-!       = rhoPij-C1rho eps/k(   -2/3k dij)-C2rho(Pij-1/3Pkk dij)-2/3rho eps dij
-!       = rho{2/3dij[C2 Pkk/2+(C1-1)eps)]+(1-C2)Pij           }
-    propce(iel,iptsta+isou-1) = propce(iel,iptsta+isou-1)         &
-                          + cromo(iel) * volume(iel)              &
+    !     Calculation of Prod+Phi1+Phi2-Eps
+    !       = rhoPij-C1rho eps/k(Rij-2/3k dij)-C2rho(Pij-1/3Pkk dij)-2/3rho eps dij
+    !       In c_st_prv:
+    !       = rhoPij-C1rho eps/k(   -2/3k dij)-C2rho(Pij-1/3Pkk dij)-2/3rho eps dij
+    !       = rho{2/3dij[C2 Pkk/2+(C1-1)eps)]+(1-C2)Pij           }
+    c_st_prv(iel) = c_st_prv(iel) + cromo(iel) * cell_f_vol(iel)  &
       *(   deltij*d2s3*                                           &
            (  crij2*trprod                                        &
-            +(crij1-1.d0)* rtpa(iel,iep)  )                     &
+            +(crij1-1.d0)* cvara_ep(iel)  )                       &
          +(1.0d0-crij2)*produc(isou,iel)               )
-!       Dans SMBR
-!       =       -C1rho eps/k(Rij         )
-!       = rho{                                     -C1eps/kRij}
-    smbr(iel) = smbr(iel) + crom(iel) * volume(iel)      &
-      *( -crij1*rtpa(iel,iep)/trrij * rtpa(iel,ivar)  )
+    !       In smbr
+    !       =       -C1rho eps/k(Rij         )
+    !       = rho{                                     -C1eps/kRij}
+    smbr(iel) = smbr(iel) + crom(iel) * cell_f_vol(iel)               &
+      *( -crij1*cvara_ep(iel)/trrij * cvara_var(iel) )
 
-!     Calcul de la partie implicite issue de Phi1
-!       = C1rho eps/k(1        )
-    rovsdt(iel) = rovsdt(iel) + crom(iel) * volume(iel)  &
-                            *crij1*rtpa(iel,iep)/trrij*thetv
+    !     Calculation of the implicit part coming from Phil
+    !       = C1rho eps/k(1        )
+    rovsdt(iel) = rovsdt(iel) + crom(iel) * cell_f_vol(iel)           &
+                            *crij1*cvara_ep(iel)/trrij*thetv
 
   enddo
 
-!     Si on veut impliciter un bout de -C1rho eps/k(   -2/3k dij)
-  if(isoluc.eq.2) then
+  !     If we want to implicit a part of -C1rho eps/k(   -2/3k dij)
+  if (isoluc.eq.2) then
 
     do iel = 1, ncel
 
       trrij  = w8(iel)
 
-!    On enleve a CROMO
-!       =       -C1rho eps/k(   -1/3Rij dij)
-      propce(iel,iptsta+isou-1) = propce(iel,iptsta+isou-1)       &
-                          - cromo(iel) * volume(iel)              &
-      *(deltij*d1s3*crij1*rtpa(iel,iep)/trrij * rtpa(iel,ivar))
-!    On ajoute a SMBR (avec CROM)
-!       =       -C1rho eps/k(   -1/3Rij dij)
+     !    We remove of cromo
+     !       =       -C1rho eps/k(   -1/3Rij dij)
+      c_st_prv(iel) = c_st_prv(iel) - cromo(iel) * cell_f_vol(iel)    &
+      *(deltij*d1s3*crij1*cvara_ep(iel)/trrij * cvara_var(iel))
+      !    We add to smbr (with crom)
+      !       =       -C1rho eps/k(   -1/3Rij dij)
       smbr(iel)                 = smbr(iel)                       &
-                          + crom(iel) * volume(iel)      &
-      *(deltij*d1s3*crij1*rtpa(iel,iep)/trrij * rtpa(iel,ivar))
-!    On ajoute a ROVSDT (avec CROM)
-!       =        C1rho eps/k(   -1/3    dij)
-      rovsdt(iel) = rovsdt(iel) + crom(iel) * volume(iel)&
-      *(deltij*d1s3*crij1*rtpa(iel,iep)/trrij                 )
+                          + crom(iel) * cell_f_vol(iel)           &
+      *(deltij*d1s3*crij1*cvara_ep(iel)/trrij * cvara_var(iel))
+      !    We add to rovsdt (woth crom)
+      !       =        C1rho eps/k(   -1/3    dij)
+      rovsdt(iel) = rovsdt(iel) + crom(iel) * cell_f_vol(iel)         &
+      *(deltij*d1s3*crij1*cvara_ep(iel)/trrij                 )
     enddo
 
   endif
 
-! Si on n'extrapole pas les termes sources
+! If we do not extrapolate the source terms
 else
 
   do iel = 1, ncel
 
-!     Demi-traces de Prod et R
+    !     Half-traces of Prod and R
     trprod = 0.5d0*(produc(1,iel)+produc(2,iel)+produc(3,iel))
     trrij  = w8(iel)
 
-!     Calcul de Prod+Phi1+Phi2-Eps
-!       = rhoPij-C1rho eps/k(Rij-2/3k dij)-C2rho(Pij-1/3Pkk dij)-2/3rho eps dij
-!       = rho{2/3dij[C2 Pkk/2+(C1-1)eps)]+(1-C2)Pij-C1eps/kRij}
-    smbr(iel) = smbr(iel) + crom(iel) * volume(iel)      &
+    !     Calculation of Prod+Phi1+Phi2-Eps
+    !       = rhoPij-C1rho eps/k(Rij-2/3k dij)-C2rho(Pij-1/3Pkk dij)-2/3rho eps dij
+    !       = rho{2/3dij[C2 Pkk/2+(C1-1)eps)]+(1-C2)Pij-C1eps/kRij}
+    smbr(iel) = smbr(iel) + crom(iel) * cell_f_vol(iel)           &
       *(   deltij*d2s3*                                           &
            (  crij2*trprod                                        &
-            +(crij1-1.d0)* rtpa(iel,iep)  )                     &
+            +(crij1-1.d0)* cvara_ep(iel)  )                           &
          +(1.0d0-crij2)*produc(isou,iel)                          &
-         -crij1*rtpa(iel,iep)/trrij * rtpa(iel,ivar)  )
+         -crij1*cvara_ep(iel)/trrij * cvara_var(iel)  )
 
-!     Calcul de la partie implicite issue de Phi1
-!       = C1rho eps/k(1-1/3 dij)
-    rovsdt(iel) = rovsdt(iel) + crom(iel) * volume(iel)  &
-         *(1.d0-d1s3*deltij)*crij1*rtpa(iel,iep)/trrij
+    !     Calculation of the implicit part coming from Phi1
+    !       = C1rho eps/k(1-1/3 dij)
+    rovsdt(iel) = rovsdt(iel) + crom(iel) * cell_f_vol(iel)           &
+         *(1.d0-d1s3*deltij)*crij1*cvara_ep(iel)/trrij
   enddo
 
 endif
@@ -451,68 +475,61 @@ if (icorio.eq.1 .or. iturbo.eq.1) then
     w7(iel) = 0.d0
   enddo
 
-  ! Rotation matrix: dual antisymmetric matrix of the rotation vector omega
-  matrot(1,2) = -rotax(3)
-  matrot(1,3) =  rotax(2)
-  matrot(2,3) = -rotax(1)
+  ! Index connectivity (i,j) <-> ivar
+  ivar_r(1,1) = ir11
+  ivar_r(2,2) = ir22
+  ivar_r(3,3) = ir33
+  ivar_r(1,2) = ir12
+  ivar_r(1,3) = ir13
+  ivar_r(2,3) = ir23
+  ivar_r(2,1) = ivar_r(1,2)
+  ivar_r(3,1) = ivar_r(1,3)
+  ivar_r(3,2) = ivar_r(2,3)
 
-  do ii = 1, 3
-    matrot(ii,ii) = 0.d0
-    do jj = ii+1, 3
-      matrot(jj,ii) = -matrot(ii,jj)
-    enddo
-  enddo
-
-  ! Index Connectivity
-  indrey(1,1) = ir11
-  indrey(2,2) = ir22
-  indrey(3,3) = ir33
-  indrey(1,2) = ir12
-  indrey(1,3) = ir13
-  indrey(2,3) = ir23
-  indrey(2,1) = indrey(1,2)
-  indrey(3,1) = indrey(1,3)
-  indrey(3,2) = indrey(2,3)
-
-  if (isou.eq.1) then
+  if (ivar.eq.ir11) then
     ii = 1
     jj = 1
-  elseif (isou.eq.2) then
+  else if (ivar.eq.ir22) then
     ii = 2
     jj = 2
-  elseif (isou.eq.3) then
+  else if (ivar.eq.ir33) then
     ii = 3
     jj = 3
-  elseif (isou.eq.4) then
+  else if (ivar.eq.ir12) then
     ii = 1
     jj = 2
-  elseif (isou.eq.5) then
+  else if (ivar.eq.ir13) then
     ii = 1
     jj = 3
-  elseif (isou.eq.6) then
+  else if (ivar.eq.ir23) then
     ii = 2
     jj = 3
-  endif
+  end if
 
   ! Compute Gij: (i,j) component of the Coriolis production
-  do iel = 1, ncel
-    do kk = 1, 3
-      w7(iel) = w7(iel) - ccorio*( matrot(ii,kk)*rtpa(iel,indrey(jj,kk)) &
-                               + matrot(jj,kk)*rtpa(iel,indrey(ii,kk)) )
+  do kk = 1, 3
+    call field_get_val_prev_s(ivarfl(ivar_r(ii,kk)), cvara_rik)
+    call field_get_val_prev_s(ivarfl(ivar_r(jj,kk)), cvara_rjk)
+
+    do iel = 1, ncel
+      call coriolis_t(irotce(iel), 1.d0, matrot)
+
+      w7(iel) = w7(iel) - ccorio*(  matrot(ii,kk)*cvara_rjk(iel) &
+                                  + matrot(jj,kk)*cvara_rik(iel) )
     enddo
   enddo
 
   ! Coriolis contribution in the Phi1 term: (1-C2/2)Gij
   if (icorio.eq.1) then
     do iel = 1, ncel
-      w7(iel) = crom(iel)*volume(iel)*(1.d0 - 0.5d0*crij2)*w7(iel)
+      w7(iel) = crom(iel)*cell_f_vol(iel)*(1.d0 - 0.5d0*crij2)*w7(iel)
     enddo
   endif
 
   ! If source terms are extrapolated
-  if (isto2t.gt.0) then
+  if (st_prv_id.ge.0) then
     do iel = 1, ncel
-      propce(iel,iptsta+isou-1) = propce(iel,iptsta+isou-1) + w7(iel)
+      c_st_prv(iel) = c_st_prv(iel) + w7(iel)
     enddo
   ! Otherwise, directly in smbr
   else
@@ -533,15 +550,14 @@ if (irijec.eq.1) then
     w7(iel) = 0.d0
   enddo
 
-  call rijech(isou, rtpa, produc, w7)
-  !==========
+  call rijech(isou, produc, w7)
 
-  ! Si on extrapole les T.S. : PROPCE
-  if(isto2t.gt.0) then
+  ! If we extrapolate the source terms: c_st_prv
+  if (st_prv_id.ge.0) then
     do iel = 1, ncel
-       propce(iel,iptsta+isou-1) = propce(iel,iptsta+isou-1) + w7(iel)
+       c_st_prv(iel) = c_st_prv(iel) + w7(iel)
      enddo
-  ! Sinon SMBR
+  ! Otherwise smbr
   else
     do iel = 1, ncel
       smbr(iel) = smbr(iel) + w7(iel)
@@ -561,14 +577,12 @@ if (igrari.eq.1) then
     w7(iel) = 0.d0
   enddo
 
-  call rijthe(nscal, ivar, rtpa, gradro, w7)
-  !==========
+  call rijthe(nscal, ivar, gradro, w7)
 
   ! If source terms are extrapolated
-  if (isto2t.gt.0) then
+  if (st_prv_id.ge.0) then
     do iel = 1, ncel
-      propce(iel,iptsta+isou-1) =                                  &
-      propce(iel,iptsta+isou-1) + w7(iel)
+      c_st_prv(iel) = c_st_prv(iel) + w7(iel)
     enddo
   else
     do iel = 1, ncel
@@ -583,23 +597,22 @@ endif
 !===============================================================================
 
 ! Symmetric tensor diffusivity (GGDH)
-if (idften(ivar).eq.6) then
+if (iand(vcopt%idften, ANISOTROPIC_DIFFUSION).ne.0) then
 
   call field_get_val_v(ivsten, visten)
 
   do iel = 1, ncel
-    viscce(1,iel) = visten(1,iel) + propce(iel,ipcvis)
-    viscce(2,iel) = visten(2,iel) + propce(iel,ipcvis)
-    viscce(3,iel) = visten(3,iel) + propce(iel,ipcvis)
+    viscce(1,iel) = visten(1,iel) + viscl(iel)
+    viscce(2,iel) = visten(2,iel) + viscl(iel)
+    viscce(3,iel) = visten(3,iel) + viscl(iel)
     viscce(4,iel) = visten(4,iel)
     viscce(5,iel) = visten(5,iel)
     viscce(6,iel) = visten(6,iel)
   enddo
 
-  iwarnp = iwarni(ivar)
+  iwarnp = vcopt%iwarni
 
   call vitens &
-  !==========
  ( viscce , iwarnp ,             &
    weighf , weighb ,             &
    viscf  , viscb  )
@@ -608,13 +621,12 @@ if (idften(ivar).eq.6) then
 else
 
   do iel = 1, ncel
-    trrij = 0.5d0 * (rtpa(iel,ir11) + rtpa(iel,ir22) + rtpa(iel,ir33))
-    rctse = crom(iel) * csrij * trrij**2 / rtpa(iel,iep)
-    w1(iel) = propce(iel,ipcvis) + idifft(ivar)*rctse
+    trrij = 0.5d0 * (cvara_r11(iel) + cvara_r22(iel) + cvara_r33(iel))
+    rctse = crom(iel) * csrij * trrij**2 / cvara_ep(iel)
+    w1(iel) = viscl(iel) + vcopt%idifft*rctse
   enddo
 
   call viscfa                    &
-  !==========
  ( imvisf ,                      &
    w1     ,                      &
    viscf  , viscb  )
@@ -625,57 +637,53 @@ endif
 ! 10. Solving
 !===============================================================================
 
-if (isto2t.gt.0) then
+if (st_prv_id.ge.0) then
   thetp1 = 1.d0 + thets
   do iel = 1, ncel
-    smbr(iel) = smbr(iel) + thetp1*propce(iel,iptsta+isou-1)
+    smbr(iel) = smbr(iel) + thetp1*c_st_prv(iel)
   enddo
 endif
 
-iconvp = iconv (ivar)
-idiffp = idiff (ivar)
-ndircp = ndircl(ivar)
-ireslp = iresol(ivar)
-nitmap = nitmax(ivar)
-nswrsp = nswrsm(ivar)
-nswrgp = nswrgr(ivar)
-imligp = imligr(ivar)
-ircflp = ircflu(ivar)
-ischcp = ischcv(ivar)
-isstpp = isstpc(ivar)
+iconvp = vcopt%iconv
+idiffp = vcopt%idiff
+ndircp = vcopt%ndircl
+nswrsp = vcopt%nswrsm
+nswrgp = vcopt%nswrgr
+imligp = vcopt%imligr
+ircflp = vcopt%ircflu
+ischcp = vcopt%ischcv
+isstpp = vcopt%isstpc
 iescap = 0
 imucpp = 0
-idftnp = idften(ivar)
-iswdyp = iswdyn(ivar)
-imgrp  = imgr  (ivar)
-ncymxp = ncymax(ivar)
-nitmfp = nitmgf(ivar)
-iwarnp = iwarni(ivar)
-blencp = blencv(ivar)
-epsilp = epsilo(ivar)
-epsrsp = epsrsm(ivar)
-epsrgp = epsrgr(ivar)
-climgp = climgr(ivar)
-extrap = extrag(ivar)
-relaxp = relaxv(ivar)
+idftnp = vcopt%idften
+iswdyp = vcopt%iswdyn
+iwarnp = vcopt%iwarni
+blencp = vcopt%blencv
+epsilp = vcopt%epsilo
+epsrsp = vcopt%epsrsm
+epsrgp = vcopt%epsrgr
+climgp = vcopt%climgr
+extrap = vcopt%extrag
+relaxp = vcopt%relaxv
 ! all boundary convective flux with upwind
 icvflb = 0
+normp = -1.d0
+init   = 1
 
 call codits &
-!==========
- ( idtvar , ivar   , iconvp , idiffp , ireslp , ndircp , nitmap , &
+ ( idtvar , init   , ivarfl(ivar)    , iconvp , idiffp , ndircp , &
    imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,          &
-   imgrp  , ncymxp , nitmfp , ipp    , iwarnp ,                   &
+   iwarnp , normp  ,                                              &
    blencp , epsilp , epsrsp , epsrgp , climgp , extrap ,          &
    relaxp , thetv  ,                                              &
-   rtpa(1,ivar)    , rtpa(1,ivar)    ,                            &
+   cvara_var       , cvara_var       ,                            &
    coefap , coefbp , cofafp , cofbfp ,                            &
    imasfl , bmasfl ,                                              &
-   viscf  , viscb  , viscce , viscf  , viscb  , viscce ,          &
+   viscf  , viscb  , viscf  , viscb  , viscce ,                   &
    weighf , weighb ,                                              &
    icvflb , ivoid  ,                                              &
-   rovsdt , smbr   , rtp(1,ivar)     , dpvar  ,                   &
+   rovsdt , smbr   , cvar_var        , dpvar  ,                   &
    rvoid  , rvoid  )
 
 ! Free memory
@@ -691,11 +699,11 @@ deallocate(weighf, weighb)
 
 #if defined(_CS_LANG_FR)
 
- 1000 format(/,'           RESOLUTION POUR LA VARIABLE ',A8,/)
+ 1000 format(/,'           Resolution pour la variable ',A8,/)
 
 #else
 
- 1000 format(/,'           SOLVING VARIABLE ',A8           ,/)
+ 1000 format(/,'           Solving variable ',A8           ,/)
 
 #endif
 
