@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -36,13 +36,12 @@
 !> - isou=4 for \f$ R_{12} \f$
 !> - isou=5 for \f$ R_{13} \f$
 !> - isou=6 for \f$ R_{23} \f$
-!>
 !-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
 ! Arguments
 !______________________________________________________________________________.
-!  mode           name          role                                           !
+!  mode           name          role
 !______________________________________________________________________________!
 !> \param[in]     nvar          total number of variables
 !> \param[in]     nscal         total number of scalars
@@ -50,27 +49,21 @@
 !> \param[in]     ncesmp        number of cells with mass source term
 !> \param[in]     ivar          variable number
 !> \param[in]     isou          local variable number (7 here)
-!> \param[in]     ipp           index for writing
 !> \param[in]     icepdc        index of cells with head loss
 !> \param[in]     icetsm        index of cells with mass source term
 !> \param[in]     itypsm        type of mass source term for each variable
-!>                               (see \ref ustsma)
+!>                               (see \ref cs_user_mass_source_terms)
 !> \param[in]     dt            time step (per cell)
-!> \param[in,out] rtp, rtpa     calculated variables at cell centers
-!>                               (at current and previous time steps)
-!> \param[in]     propce        physical properties at cell centers
-!> \param[in]     gradv         tableau de travail pour terme grad
-!>                                 de vitesse     uniqt pour iturb=31
-!> \param[in]     produc        tableau de travail pour production
-!> \param[in]     gradro        tableau de travail pour grad rom
-!>                              (sans rho volume) uniqt pour iturb=30
+!> \param[in]     gradv         work array for the velocity grad term
+!>                                 only for iturb=31
+!> \param[in]     gradro        work array for grad rom
+!>                              (without rho volume) only for iturb=30
 !> \param[in]     ckupdc        work array for the head loss
 !> \param[in]     smacel        value associated to each variable in the mass
-!>                               source terms or mass rate (see \ref ustsma)
-!> \param[in]     viscf         visc*surface/dist aux faces internes
-!> \param[in]     viscb         visc*surface/dist aux faces de bord
-!> \param[in]     tslagr        coupling term for lagrangian
-!> \param[in]     tslage        explicit source terms for the Lagrangian module
+!>                               source terms or mass rate
+!>                               (see \ref cs_user_mass_source_terms)
+!> \param[in]     viscf         visc*surface/dist at internal faces
+!> \param[in]     viscb         visc*surface/dist at edge faces
 !> \param[in]     tslagi        implicit source terms for the Lagrangian module
 !> \param[in]     smbr          working array
 !> \param[in]     rovsdt        working array
@@ -78,13 +71,13 @@
 
 subroutine resssg &
  ( nvar   , nscal  , ncepdp , ncesmp ,                            &
-   ivar   , isou   , ipp    ,                                     &
+   ivar   , isou   ,                                              &
    icepdc , icetsm , itypsm ,                                     &
-   dt     , rtp    , rtpa   , propce ,                            &
+   dt     ,                                                       &
    gradv  , gradro ,                                              &
    ckupdc , smacel ,                                              &
    viscf  , viscb  ,                                              &
-   tslage , tslagi ,                                              &
+   tslagi ,                                                       &
    smbr   , rovsdt )
 
 !===============================================================================
@@ -106,7 +99,9 @@ use mesh
 use field
 use field_operator
 use cs_f_interfaces
+use rotation
 use turbomachinery
+use cs_c_bindings
 
 !===============================================================================
 
@@ -116,35 +111,36 @@ implicit none
 
 integer          nvar   , nscal
 integer          ncepdp , ncesmp
-integer          ivar   , isou   , ipp
+integer          ivar   , isou
 
 integer          icepdc(ncepdp)
 integer          icetsm(ncesmp), itypsm(ncesmp,nvar)
 
-double precision dt(ncelet), rtp(ncelet,nflown:nvar), rtpa(ncelet,nflown:nvar)
-double precision propce(ncelet,*)
+double precision dt(ncelet)
 double precision gradv(3, 3, ncelet)
-double precision gradro(ncelet,3)
-double precision ckupdc(ncepdp,6), smacel(ncesmp,nvar)
+double precision gradro(3,ncelet)
+double precision ckupdc(6,ncepdp), smacel(ncesmp,nvar)
 double precision viscf(nfac), viscb(nfabor)
-double precision tslage(ncelet),tslagi(ncelet)
+double precision tslagi(ncelet)
 double precision smbr(ncelet), rovsdt(ncelet)
 
 ! Local variables
 
 integer          iel
 integer          ii    , jj    , kk    , iiun  , iii   , jjj
-integer          ipcvis, iflmas, iflmab
+integer          iflmas, iflmab
 integer          nswrgp, imligp, iwarnp
-integer          iconvp, idiffp, ndircp, ireslp
-integer          nitmap, nswrsp, ircflp, ischcp, isstpp, iescap
-integer          imgrp , ncymxp, nitmfp
-integer          iptsta
+integer          iconvp, idiffp, ndircp
+integer          nswrsp, ircflp, ischcp, isstpp, iescap
+integer          st_prv_id, comp_id
 integer          iprev , inc, iccocg, ll
 integer          imucpp, idftnp, iswdyp
-integer          indrey(3,3)
+integer          ivar_r(3,3)
 integer          icvflb
+integer          init
 integer          ivoid(1)
+integer          key_t_ext_id
+integer          iroext
 
 double precision blencp, epsilp, epsrgp, climgp, extrap, relaxp
 double precision epsrsp
@@ -159,6 +155,7 @@ double precision pij, phiij1, phiij2, epsij
 double precision phiijw, epsijw
 double precision ccorio
 double precision rctse
+double precision normp
 
 double precision rvoid(1)
 
@@ -173,7 +170,19 @@ double precision, allocatable, dimension(:) :: weighb
 double precision, dimension(:), pointer :: imasfl, bmasfl
 double precision, dimension(:), pointer :: crom, cromo
 double precision, dimension(:), pointer :: coefap, coefbp, cofafp, cofbfp
-double precision, dimension(:,:), pointer :: visten
+double precision, dimension(:,:), pointer :: visten, lagr_st_rij
+double precision, dimension(:), pointer :: cvara_ep, cvar_al
+double precision, dimension(:), pointer :: cvara_r11, cvara_r22, cvara_r33
+double precision, dimension(:), pointer :: cvara_r12, cvara_r13, cvara_r23
+double precision, dimension(:), pointer :: cvar_var, cvara_var
+double precision, dimension(:), pointer :: viscl, c_st_prv, visct
+
+type pmapper_double_r1
+   double precision, dimension(:),  pointer :: p !< rank 1 array pointer
+end type pmapper_double_r1
+type(pmapper_double_r1), allocatable, dimension(:,:) :: cvara_r
+
+type(var_cal_opt) :: vcopt
 
 !===============================================================================
 
@@ -181,6 +190,11 @@ double precision, dimension(:,:), pointer :: visten
 ! 1. Initialization
 !===============================================================================
 
+! Time extrapolation?
+call field_get_key_id("time_extrapolated", key_t_ext_id)
+
+call field_get_key_int(icrom, key_t_ext_id, iroext)
+
 ! Allocate work arrays
 allocate(w1(ncelet), w2(ncelet))
 allocate(dpvar(ncelet))
@@ -192,13 +206,30 @@ allocate(weighb(nfabor))
 iii = 0
 jjj = 0
 
-if (iwarni(ivar).ge.1) then
+call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+
+if (vcopt%iwarni.ge.1) then
   call field_get_label(ivarfl(ivar), label)
   write(nfecra,1000) label
 endif
 
 call field_get_val_s(icrom, crom)
-ipcvis = ipproc(iviscl)
+call field_get_val_s(iviscl, viscl)
+call field_get_val_s(ivisct, visct)
+
+call field_get_val_prev_s(ivarfl(iep), cvara_ep)
+if (iturb.ne.31) call field_get_val_s(ivarfl(ial), cvar_al)
+
+call field_get_val_prev_s(ivarfl(ir11), cvara_r11)
+call field_get_val_prev_s(ivarfl(ir22), cvara_r22)
+call field_get_val_prev_s(ivarfl(ir33), cvara_r33)
+call field_get_val_prev_s(ivarfl(ir12), cvara_r12)
+call field_get_val_prev_s(ivarfl(ir13), cvara_r13)
+call field_get_val_prev_s(ivarfl(ir23), cvara_r23)
+
+call field_get_val_s(ivarfl(ivar), cvar_var)
+call field_get_val_prev_s(ivarfl(ivar), cvara_var)
+
 call field_get_key_int(ivarfl(iu), kimasf, iflmas)
 call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
 call field_get_val_s(iflmas, imasfl)
@@ -209,23 +240,26 @@ d1s3   = 1.d0/3.d0
 d2s3   = 2.d0/3.d0
 
 deltij = 1.0d0
-if(isou.gt.3) then
+if (isou.gt.3) then
   deltij = 0.0d0
 endif
 
-!     S pour Source, V pour Variable
+!     S as Source, V as Variable
 thets  = thetst
-thetv  = thetav(ivar )
+thetv  = vcopt%thetav
 
-if (isto2t.gt.0.and.iroext.gt.0) then
+call field_get_key_int(ivarfl(ivar), kstprv, st_prv_id)
+if (st_prv_id .ge. 0) then
+  call field_get_val_s(st_prv_id, c_st_prv)
+else
+  c_st_prv=> null()
+endif
+
+if (st_prv_id.ge.0.and.iroext.gt.0) then
   call field_get_val_prev_s(icrom, cromo)
 else
   call field_get_val_s(icrom, cromo)
 endif
-iptsta = 0
-if (isto2t.gt.0) then
-  iptsta = ipproc(itstua)
-endif
 
 do iel = 1, ncel
   smbr(iel) = 0.d0
@@ -234,15 +268,36 @@ do iel = 1, ncel
   rovsdt(iel) = 0.d0
 enddo
 
-! Coefficient of the "Coriolis-type" term
-if (icorio.eq.1) then
-  ! Relative velocity formulation
-  ccorio = 2.d0
-elseif (iturbo.eq.1) then
-  ! Mixed relative/absolute velocity formulation
-  ccorio = 1.d0
-else
-  ccorio = 0.d0
+if (icorio.eq.1 .or. iturbo.eq.1) then
+
+  ! Index connectivity (i,j) -> ivar
+  ivar_r(1,1) = ir11
+  ivar_r(2,2) = ir22
+  ivar_r(3,3) = ir33
+  ivar_r(1,2) = ir12
+  ivar_r(1,3) = ir13
+  ivar_r(2,3) = ir23
+  ivar_r(2,1) = ivar_r(1,2)
+  ivar_r(3,1) = ivar_r(1,3)
+  ivar_r(3,2) = ivar_r(2,3)
+
+  ! Build the interleaved Reynolds tensor
+  allocate(cvara_r(3,3))
+  do ii = 1, 3
+    do kk = 1, 3
+      call field_get_val_prev_s(ivarfl(ivar_r(ii,kk)), cvara_r(ii,kk)%p)
+    enddo
+  enddo
+
+  ! Coefficient of the "Coriolis-type" term
+  if (icorio.eq.1) then
+    ! Relative velocity formulation
+    ccorio = 2.d0
+  elseif (iturbo.eq.1) then
+    ! Mixed relative/absolute velocity formulation
+    ccorio = 1.d0
+  endif
+
 endif
 
 !===============================================================================
@@ -250,30 +305,29 @@ endif
 !===============================================================================
 
 call cs_user_turbulence_source_terms &
-!===================================
  ( nvar   , nscal  , ncepdp , ncesmp ,                            &
    ivarfl(ivar)    ,                                              &
    icepdc , icetsm , itypsm ,                                     &
    ckupdc , smacel ,                                              &
    smbr   , rovsdt )
 
-!     Si on extrapole les T.S.
-if(isto2t.gt.0) then
+!     If we extrapolate the source terms
+if (st_prv_id.ge.0) then
   do iel = 1, ncel
-!       Sauvegarde pour echange
-    tuexpr = propce(iel,iptsta+isou-1)
-!       Pour la suite et le pas de temps suivant
-    propce(iel,iptsta+isou-1) = smbr(iel)
-!       Second membre du pas de temps precedent
-!       On suppose -ROVSDT > 0 : on implicite
-!          le terme source utilisateur (le reste)
-    smbr(iel) = rovsdt(iel)*rtpa(iel,ivar)  - thets*tuexpr
-!       Diagonale
+    !       Save for exchange
+    tuexpr = c_st_prv(iel)
+    !       For continuation and the next time step
+    c_st_prv(iel) = smbr(iel)
+    !       Second member of the previous time step
+    !       We suppose -rovsdt > 0: we implicite
+    !          the user source term (the rest)
+    smbr(iel) = rovsdt(iel)*cvara_var(iel)  - thets*tuexpr
+    !       Diagonal
     rovsdt(iel) = - thetv*rovsdt(iel)
   enddo
 else
   do iel = 1, ncel
-    smbr(iel)   = rovsdt(iel)*rtpa(iel,ivar) + smbr(iel)
+    smbr(iel)   = rovsdt(iel)*cvara_var(iel) + smbr(iel)
     rovsdt(iel) = max(-rovsdt(iel),zero)
   enddo
 endif
@@ -282,10 +336,17 @@ endif
 ! 3. Lagrangian source terms
 !===============================================================================
 
-!     Ordre 2 non pris en compte
+ !     2nd order is not taken into account
  if (iilagr.eq.2 .and. ltsdyn.eq.1) then
+   call field_get_val_v_by_name('rij_st_lagr', lagr_st_rij)
+   comp_id = isou
+   if (isou .eq. 5) then
+     comp_id = 6
+   else if (isou .eq.6) then
+     comp_id = 5
+   endif
    do iel = 1,ncel
-     smbr(iel)   = smbr(iel)   + tslage(iel)
+     smbr(iel)   = smbr(iel)   + lagr_st_rij(comp_id,iel)
      rovsdt(iel) = rovsdt(iel) + max(-tslagi(iel),zero)
    enddo
  endif
@@ -296,24 +357,22 @@ endif
 
 if (ncesmp.gt.0) then
 
-!       Entier egal a 1 (pour navsto : nb de sur-iter)
+  !       Integer equal to 1 (for navsto: nb of sur-iter)
   iiun = 1
 
-! On incremente SMBR par -Gamma RTPA et ROVSDT par Gamma (*theta)
+  ! We increment smbr with -Gamma.var_prev and rovsdr with Gamma
   call catsma &
-  !==========
- ( ncelet , ncel   , ncesmp , iiun   , isto2t , thetv  ,          &
+ ( ncelet , ncel   , ncesmp , iiun   , isto2t ,                   &
    icetsm , itypsm(:,ivar)  ,                                     &
-   volume , rtpa(:,ivar)    , smacel(:,ivar)   , smacel(:,ipr) ,  &
+   cell_f_vol , cvara_var       , smacel(:,ivar)   , smacel(:,ipr) ,  &
    smbr   ,  rovsdt , w1 )
 
-!       Si on extrapole les TS on met Gamma Pinj dans PROPCE
-  if(isto2t.gt.0) then
+  ! If we extrapolate the source terms we put Gamma Pinj in the previous st
+  if (st_prv_id.ge.0) then
     do iel = 1, ncel
-      propce(iel,iptsta+isou-1) =                                 &
-      propce(iel,iptsta+isou-1) + w1(iel)
+      c_st_prv(iel) = c_st_prv(iel) + w1(iel)
     enddo
-!       Sinon on le met directement dans SMBR
+  ! Otherwise we put it directly in the RHS
   else
     do iel = 1, ncel
       smbr(iel) = smbr(iel) + w1(iel)
@@ -323,14 +382,14 @@ if (ncesmp.gt.0) then
 endif
 
 !===============================================================================
-! 5. Non-stationary term
+! 5. Unsteady term
 !===============================================================================
 
-! ---> Ajout dans la diagonale de la matrice
+! ---> Added in the matrix diagonal
 
-do iel=1,ncel
+do iel = 1, ncel
   rovsdt(iel) = rovsdt(iel)                                       &
-            + istat(ivar)*(crom(iel)/dt(iel))*volume(iel)
+            + vcopt%istat*(crom(iel)/dt(iel))*cell_f_vol(iel)
 enddo
 
 
@@ -338,31 +397,31 @@ enddo
 ! 6. Production, Pressure-Strain correlation, dissipation, Coriolis
 !===============================================================================
 
-! ---> Terme source
+! ---> Source term
 !     -rho*epsilon*( Cs1*aij + Cs2*(aikajk -1/3*aijaij*deltaij))
 !     -Cr1*P*aij + Cr2*rho*k*sij - Cr3*rho*k*sij*sqrt(aijaij)
 !     +Cr4*rho*k(aik*sjk+ajk*sik-2/3*akl*skl*deltaij)
 !     +Cr5*rho*k*(aik*rjk + ajk*rik)
 !     -2/3*epsilon*deltaij
 
-if(isou.eq.1)then
+if (isou.eq.1)then
   iii = 1
   jjj = 1
-elseif(isou.eq.2)then
+elseif (isou.eq.2)then
   iii = 2
   jjj = 2
-elseif(isou.eq.3)then
+elseif (isou.eq.3)then
   iii = 3
   jjj = 3
-elseif(isou.eq.4)then
+elseif (isou.eq.4)then
   iii = 1
   jjj = 2
-elseif(isou.eq.5)then
-  iii = 1
-  jjj = 3
-elseif(isou.eq.6)then
+elseif (isou.eq.5)then
   iii = 2
   jjj = 3
+elseif (isou.eq.6)then
+  iii = 1
+  jjj = 3
 endif
 
 ! EBRSM
@@ -380,39 +439,6 @@ if (iturb.eq.32) then
 
 endif
 
-if (icorio.eq.1 .or. iturbo.eq.1) then
-
-  ! Compute the rotation matrix (dual matrix of the rotation vector)
-  matrot(1,2) = -rotax(3)
-  matrot(1,3) =  rotax(2)
-  matrot(2,3) = -rotax(1)
-
-  do ii = 1, 3
-    matrot(ii,ii) = 0.d0
-    do jj = ii+1, 3
-      matrot(jj,ii) = -matrot(ii,jj)
-    enddo
-  enddo
-
-else
-  do ii = 1, 3
-    do jj = 1, 3
-      matrot(ii,jj) = 0.d0
-    enddo
-  enddo
-endif
-
-! Index of the Reynolds stress variables in rtpa array
-indrey(1,1) = ir11
-indrey(2,2) = ir22
-indrey(3,3) = ir33
-indrey(1,2) = ir12
-indrey(1,3) = ir13
-indrey(2,3) = ir23
-indrey(2,1) = indrey(1,2)
-indrey(3,1) = indrey(1,3)
-indrey(3,2) = indrey(2,3)
-
 do iel=1,ncel
 
   ! EBRSM
@@ -435,43 +461,47 @@ do iel=1,ncel
   endif
 
   ! Pij
-  xprod(1,1) = -2.0d0*(rtpa(iel,ir11)*gradv(1, 1, iel) +         &
-                       rtpa(iel,ir12)*gradv(2, 1, iel) +         &
-                       rtpa(iel,ir13)*gradv(3, 1, iel) )
-  xprod(1,2) = -(      rtpa(iel,ir11)*gradv(1, 2, iel) +         &
-                       rtpa(iel,ir12)*gradv(2, 2, iel) +         &
-                       rtpa(iel,ir13)*gradv(3, 2, iel) )         &
-               -(      rtpa(iel,ir12)*gradv(1, 1, iel) +         &
-                       rtpa(iel,ir22)*gradv(2, 1, iel) +         &
-                       rtpa(iel,ir23)*gradv(3, 1, iel) )
-  xprod(1,3) = -(      rtpa(iel,ir11)*gradv(1, 3, iel) +         &
-                       rtpa(iel,ir12)*gradv(2, 3, iel) +         &
-                       rtpa(iel,ir13)*gradv(3, 3, iel) )         &
-               -(      rtpa(iel,ir13)*gradv(1, 1, iel) +         &
-                       rtpa(iel,ir23)*gradv(2, 1, iel) +         &
-                       rtpa(iel,ir33)*gradv(3, 1, iel) )
-  xprod(2,2) = -2.0d0*(rtpa(iel,ir12)*gradv(1, 2, iel) +         &
-                       rtpa(iel,ir22)*gradv(2, 2, iel) +         &
-                       rtpa(iel,ir23)*gradv(3, 2, iel) )
-  xprod(2,3) = -(      rtpa(iel,ir12)*gradv(1, 3, iel) +         &
-                       rtpa(iel,ir22)*gradv(2, 3, iel) +         &
-                       rtpa(iel,ir23)*gradv(3, 3, iel) )         &
-               -(      rtpa(iel,ir13)*gradv(1, 2, iel) +         &
-                       rtpa(iel,ir23)*gradv(2, 2, iel) +         &
-                       rtpa(iel,ir33)*gradv(3, 2, iel) )
-  xprod(3,3) = -2.0d0*(rtpa(iel,ir13)*gradv(1, 3, iel) +         &
-                       rtpa(iel,ir23)*gradv(2, 3, iel) +         &
-                       rtpa(iel,ir33)*gradv(3, 3, iel) )
+  xprod(1,1) = -2.0d0*(cvara_r11(iel)*gradv(1, 1, iel) +         &
+                       cvara_r12(iel)*gradv(2, 1, iel) +         &
+                       cvara_r13(iel)*gradv(3, 1, iel) )
+  xprod(1,2) = -(      cvara_r11(iel)*gradv(1, 2, iel) +         &
+                       cvara_r12(iel)*gradv(2, 2, iel) +         &
+                       cvara_r13(iel)*gradv(3, 2, iel) )         &
+               -(      cvara_r12(iel)*gradv(1, 1, iel) +         &
+                       cvara_r22(iel)*gradv(2, 1, iel) +         &
+                       cvara_r23(iel)*gradv(3, 1, iel) )
+  xprod(1,3) = -(      cvara_r11(iel)*gradv(1, 3, iel) +         &
+                       cvara_r12(iel)*gradv(2, 3, iel) +         &
+                       cvara_r13(iel)*gradv(3, 3, iel) )         &
+               -(      cvara_r13(iel)*gradv(1, 1, iel) +         &
+                       cvara_r23(iel)*gradv(2, 1, iel) +         &
+                       cvara_r33(iel)*gradv(3, 1, iel) )
+  xprod(2,2) = -2.0d0*(cvara_r12(iel)*gradv(1, 2, iel) +         &
+                       cvara_r22(iel)*gradv(2, 2, iel) +         &
+                       cvara_r23(iel)*gradv(3, 2, iel) )
+  xprod(2,3) = -(      cvara_r12(iel)*gradv(1, 3, iel) +         &
+                       cvara_r22(iel)*gradv(2, 3, iel) +         &
+                       cvara_r23(iel)*gradv(3, 3, iel) )         &
+               -(      cvara_r13(iel)*gradv(1, 2, iel) +         &
+                       cvara_r23(iel)*gradv(2, 2, iel) +         &
+                       cvara_r33(iel)*gradv(3, 2, iel) )
+  xprod(3,3) = -2.0d0*(cvara_r13(iel)*gradv(1, 3, iel) +         &
+                       cvara_r23(iel)*gradv(2, 3, iel) +         &
+                       cvara_r33(iel)*gradv(3, 3, iel) )
 
   ! Rotating frame of reference => "Coriolis production" term
+
   if (icorio.eq.1 .or. iturbo.eq.1) then
+
+    call coriolis_t(irotce(iel), 1.d0, matrot)
+
     if (irotce(iel).gt.0) then
       do ii = 1, 3
         do jj = ii, 3
           do kk = 1, 3
             xprod(ii,jj) = xprod(ii,jj)                             &
-                 - ccorio*( matrot(ii,kk)*rtpa(iel,indrey(jj,kk))   &
-                 + matrot(jj,kk)*rtpa(iel,indrey(ii,kk)) )
+                     - ccorio*( matrot(ii,kk)*cvara_r(jj,kk)%p(iel) &
+                     + matrot(jj,kk)*cvara_r(ii,kk)%p(iel) )
           enddo
         enddo
       enddo
@@ -483,20 +513,20 @@ do iel=1,ncel
   xprod(3,2) = xprod(2,3)
 
   trprod = d1s2 * (xprod(1,1) + xprod(2,2) + xprod(3,3) )
-  trrij  = d1s2 * (rtpa(iel,ir11) + rtpa(iel,ir22) + rtpa(iel,ir33))
-!-----> aII = aijaij
+  trrij  = d1s2 * (cvara_r11(iel) + cvara_r22(iel) + cvara_r33(iel))
+  !-----> aII = aijaij
   aii    = 0.d0
   aklskl = 0.d0
   aiksjk = 0.d0
   aikrjk = 0.d0
   aikakj = 0.d0
   ! aij
-  xaniso(1,1) = rtpa(iel,ir11)/trrij - d2s3
-  xaniso(2,2) = rtpa(iel,ir22)/trrij - d2s3
-  xaniso(3,3) = rtpa(iel,ir33)/trrij - d2s3
-  xaniso(1,2) = rtpa(iel,ir12)/trrij
-  xaniso(1,3) = rtpa(iel,ir13)/trrij
-  xaniso(2,3) = rtpa(iel,ir23)/trrij
+  xaniso(1,1) = cvara_r11(iel)/trrij - d2s3
+  xaniso(2,2) = cvara_r22(iel)/trrij - d2s3
+  xaniso(3,3) = cvara_r33(iel)/trrij - d2s3
+  xaniso(1,2) = cvara_r12(iel)/trrij
+  xaniso(1,3) = cvara_r13(iel)/trrij
+  xaniso(2,3) = cvara_r23(iel)/trrij
   xaniso(2,1) = xaniso(1,2)
   xaniso(3,1) = xaniso(1,3)
   xaniso(3,2) = xaniso(2,3)
@@ -550,54 +580,53 @@ do iel=1,ncel
     aikakj = aikakj + xaniso(iii,kk)*xaniso(kk,jjj)
   enddo
 
-!     Si on extrapole les TS (rarissime), on met tout dans PROPCE.
-!     On n'implicite pas le terme en Cs1*aij ni le terme en Cr1*P*aij.
-!     Sinon, on met tout dans SMBR et on peut impliciter Cs1*aij
-!     et Cr1*P*aij. Ici on stocke le second membre et le terme implicite
-!     dans W1 et W2, pour eviter d'avoir un test IF(ISTO2T.GT.0)
-!     dans la boucle NCEL
-!     Dans le terme en W1, qui a vocation a etre extrapole, on utilise
-!     naturellement CROMO.
-!     L'implicitation des deux termes pourrait se faire aussi en cas
-!     d'extrapolation, en isolant ces deux termes et les mettant dans
-!     SMBR et pas PROPCE et en utilisant IPCROM ... a modifier si le
-!     besoin s'en fait vraiment sentir           !
+  !     If we extrapolate the source terms (rarely), we put all in the previous ST..
+  !     We do not implicit the term with Cs1*aij neither the term with Cr1*P*aij.
+  !     Otherwise, we put all in smbr and we can implicit Cs1*aij
+  !     and Cr1*P*aij. Here we store the second member and the implicit term
+  !     in W1 and W2, to avoid the test(ST_PRV_ID.GE.0)
+  !     in the ncel loop
+  !     In the term with W1, which is dedicated to be extrapolated, we use
+  !     cromo.
+  !     The implicitation of the two terms can also be done in the case of
+  !     extrapolation, by isolating those two terms and by putting it in
+  !     the RHS but not in the prev. ST and by using ipcrom .... to be modified if needed
 
   if (iturb.eq.31) then
 
     pij = xprod(iii,jjj)
-    phiij1 = -rtpa(iel,iep)* &
+    phiij1 = -cvara_ep(iel)* &
        (cssgs1*xaniso(iii,jjj)+cssgs2*(aikakj-d1s3*deltij*aii))
     phiij2 = - cssgr1*trprod*xaniso(iii,jjj)                             &
            +   trrij*xstrai(iii,jjj)*(cssgr2-cssgr3*sqrt(aii))           &
            +   cssgr4*trrij*(aiksjk-d2s3*deltij*aklskl)                  &
            +   cssgr5*trrij* aikrjk
-    epsij = -d2s3*rtpa(iel,iep)*deltij
+    epsij = -d2s3*cvara_ep(iel)*deltij
 
-    w1(iel) = cromo(iel)*volume(iel)*(pij+phiij1+phiij2+epsij)
+    w1(iel) = cromo(iel)*cell_f_vol(iel)*(pij+phiij1+phiij2+epsij)
 
-    w2(iel) = volume(iel)/trrij*crom(iel)*(                &
-           cssgs1*rtpa(iel,iep) + cssgr1*max(trprod,0.d0) )
+    w2(iel) = cell_f_vol(iel)/trrij*crom(iel)*(                              &
+           cssgs1*cvara_ep(iel) + cssgr1*max(trprod,0.d0) )
 
   ! EBRSM
   else
 
-    xrij(1,1) = rtpa(iel,ir11)
-    xrij(2,2) = rtpa(iel,ir22)
-    xrij(3,3) = rtpa(iel,ir33)
-    xrij(1,2) = rtpa(iel,ir12)
-    xrij(1,3) = rtpa(iel,ir13)
-    xrij(2,3) = rtpa(iel,ir23)
+    xrij(1,1) = cvara_r11(iel)
+    xrij(2,2) = cvara_r22(iel)
+    xrij(3,3) = cvara_r33(iel)
+    xrij(1,2) = cvara_r12(iel)
+    xrij(1,3) = cvara_r13(iel)
+    xrij(2,3) = cvara_r23(iel)
     xrij(2,1) = xrij(1,2)
     xrij(3,1) = xrij(1,3)
     xrij(3,2) = xrij(2,3)
 
     ! Compute the explicit term
 
-    ! Calcul des termes de proches parois et quasi-homgene de phi et
-    ! epsilon
+    ! Calculation of the terms near the walls and et almost homogeneous
+    ! of phi and epsilon
 
-    ! Calcul du terme de proche paroi \Phi_{ij}^w --> W3
+    ! Calculation of the term near the wall \f$ \Phi_{ij}^w \f$ --> W3
     phiijw = 0.d0
     xnnd = d1s2*( xnal(iii)*xnal(jjj) + deltij )
     do kk = 1, 3
@@ -607,56 +636,60 @@ do iel=1,ncel
         phiijw = phiijw - xrij(kk,ll)*xnal(kk)*xnal(ll)*xnnd
       enddo
     enddo
-    phiijw = -5.d0*rtpa(iel,iep)/trrij * phiijw
+    phiijw = -5.d0*cvara_ep(iel)/trrij * phiijw
 
-    ! Calcul du terme quasi-homogene \Phi_{ij}^h --> W4
-    phiij1 = -rtpa(iel,iep)*cebms1*xaniso(iii,jjj)
+    ! Calculation of the almost homogeneous term \f$ \phi_{ij}^h \f$ --> W4
+    phiij1 = -cvara_ep(iel)*cebms1*xaniso(iii,jjj)
     phiij2 = -cebmr1*trprod*xaniso(iii,jjj)                       &
                +trrij*xstrai(iii,jjj)*(cebmr2-cebmr3*sqrt(aii))   &
                +cebmr4*trrij   *(aiksjk-d2s3*deltij*aklskl)       &
                +cebmr5*trrij   * aikrjk
 
-    ! Calcul de \e_{ij}^w --> W5 (Rotta model)
+    ! Calculation of \f $\e_{ij}^w \f$ --> W5 (Rotta model)
     ! Rij/k*epsilon
-    epsijw =  xrij(iii,jjj)/trrij   *rtpa(iel,iep)
+    epsijw =  xrij(iii,jjj)/trrij   *cvara_ep(iel)
 
     ! Calcul de \e_{ij}^h --> W6
-    epsij =  d2s3*rtpa(iel,iep)*deltij
+    epsij =  d2s3*cvara_ep(iel)*deltij
 
     ! Calcul du terme source explicite de l'equation des Rij
-    !   [ P_{ij} + (1-\alpha^3)\Phi_{ij}^w + \alpha^3\Phi_{ij}^h
-    !            - (1-\alpha^3)\e_{ij}^w   - \alpha^3\e_{ij}^h  ] --> W1
-    alpha3 = rtp(iel,ial)**3
+    !  \f[ P_{ij} + (1-\alpha^3)\Phi_{ij}^w + \alpha^3\Phi_{ij}^h
+    !            - (1-\alpha^3)\e_{ij}^w   - \alpha^3\e_{ij}^h  ]\f$ --> W1
+    alpha3 = cvar_al(iel)**3
 
-    w1(iel) = volume(iel)*crom(iel)*(                    &
+    w1(iel) = cell_f_vol(iel)*crom(iel)*(                             &
                xprod(iii,jjj)                                     &
             + (1.d0-alpha3)*phiijw + alpha3*(phiij1+phiij2)       &
             - (1.d0-alpha3)*epsijw - alpha3*epsij)
 
     !  Implicite term
 
-    ! le terme ci-dessous correspond a la partie implicitee du SSG
-    ! dans le cadre de la ponderation elliptique, il est multiplie par
-    ! \alpha^3
-    w2(iel) = volume(iel)*crom(iel)*(                    &
-              cebms1*rtpa(iel,iep)/trrij*alpha3                   &
+    ! The term below corresponds to the implicit part of SSG
+    ! in the context of elliptical weighting, it is multiplied by
+    ! \f$ \alpha^3 \f$
+    w2(iel) = cell_f_vol(iel)*crom(iel)*(                             &
+              cebms1*cvara_ep(iel)/trrij*alpha3                       &
              +cebmr1*max(trprod/trrij,0.d0)*alpha3                &
-    ! Implicitation de epsijw
-    ! (le facteur 5 apparait lorsqu'on fait Phi_{ij}^w - epsijw)
-            + 5.d0 * (1.d0-alpha3)*rtpa(iel,iep)/trrij            &
-            +        (1.d0-alpha3)*rtpa(iel,iep)/trrij)
+    ! Implicitation of epsijw
+    ! (the factor 5 appears when we calculate \f$ Phi_{ij}^w - epsijw\f$)
+            + 5.d0 * (1.d0-alpha3)*cvara_ep(iel)/trrij                &
+            +        (1.d0-alpha3)*cvara_ep(iel)/trrij)
   endif
 
 enddo
 
+if (icorio.eq.1 .or. iturbo.eq.1) then
+  deallocate(cvara_r)
+endif
+
 if (iturb.eq.32) then
   deallocate(grad)
 endif
 
-if (isto2t.gt.0) then
+if (st_prv_id.ge.0) then
 
   do iel = 1, ncel
-    propce(iel,iptsta+isou-1) = propce(iel,iptsta+isou-1) + w1(iel)
+    c_st_prv(iel) = c_st_prv(iel) + w1(iel)
   enddo
 
 else
@@ -681,15 +714,14 @@ if (igrari.eq.1) then
     w7(iel) = 0.d0
   enddo
 
-  call rijthe(nscal, ivar, rtpa, gradro, w7)
-  !==========
+  call rijthe(nscal, ivar, gradro, w7)
 
-  ! Si on extrapole les T.S. : PROPCE
-  if(isto2t.gt.0) then
+  ! If we extrapolate the source terms: previous ST
+  if (st_prv_id.ge.0) then
     do iel = 1, ncel
-      propce(iel,iptsta+isou-1) = propce(iel,iptsta+isou-1) + w7(iel)
+      c_st_prv(iel) = c_st_prv(iel) + w7(iel)
     enddo
-  ! Sinon SMBR
+  ! Otherwise smbr
   else
     do iel = 1, ncel
       smbr(iel) = smbr(iel) + w7(iel)
@@ -706,23 +738,22 @@ endif
 !===============================================================================
 
 ! Symmetric tensor diffusivity (GGDH)
-if (idften(ivar).eq.6) then
+if (iand(vcopt%idften, ANISOTROPIC_DIFFUSION).ne.0) then
 
   call field_get_val_v(ivsten, visten)
 
   do iel = 1, ncel
-    viscce(1,iel) = visten(1,iel) + propce(iel,ipcvis)
-    viscce(2,iel) = visten(2,iel) + propce(iel,ipcvis)
-    viscce(3,iel) = visten(3,iel) + propce(iel,ipcvis)
+    viscce(1,iel) = visten(1,iel) + viscl(iel)
+    viscce(2,iel) = visten(2,iel) + viscl(iel)
+    viscce(3,iel) = visten(3,iel) + viscl(iel)
     viscce(4,iel) = visten(4,iel)
     viscce(5,iel) = visten(5,iel)
     viscce(6,iel) = visten(6,iel)
   enddo
 
-  iwarnp = iwarni(ivar)
+  iwarnp = vcopt%iwarni
 
   call vitens &
-  !==========
  ( viscce , iwarnp ,             &
    weighf , weighb ,             &
    viscf  , viscb  )
@@ -731,13 +762,11 @@ if (idften(ivar).eq.6) then
 else
 
   do iel = 1, ncel
-    trrij = 0.5d0 * (rtpa(iel,ir11) + rtpa(iel,ir22) + rtpa(iel,ir33))
-    rctse = crom(iel) * csrij * trrij**2 / rtpa(iel,iep)
-    w1(iel) = propce(iel,ipcvis) + idifft(ivar)*rctse
+    rctse = csrij * visct(iel) / cmu
+    w1(iel) = viscl(iel) + vcopt%idifft*rctse
   enddo
 
   call viscfa                    &
-  !==========
  ( imvisf ,                      &
    w1     ,                      &
    viscf  , viscb  )
@@ -748,41 +777,38 @@ endif
 ! 9. Solving
 !===============================================================================
 
-if (isto2t.gt.0) then
+if (st_prv_id.ge.0) then
   thetp1 = 1.d0 + thets
   do iel = 1, ncel
-    smbr(iel) = smbr(iel) + thetp1*propce(iel,iptsta+isou-1)
+    smbr(iel) = smbr(iel) + thetp1*c_st_prv(iel)
   enddo
 endif
 
-iconvp = iconv (ivar)
-idiffp = idiff (ivar)
-ireslp = iresol(ivar)
-ndircp = ndircl(ivar)
-nitmap = nitmax(ivar)
-nswrsp = nswrsm(ivar)
-nswrgp = nswrgr(ivar)
-imligp = imligr(ivar)
-ircflp = ircflu(ivar)
-ischcp = ischcv(ivar)
-isstpp = isstpc(ivar)
+iconvp = vcopt%iconv
+idiffp = vcopt%idiff
+ndircp = vcopt%ndircl
+nswrsp = vcopt%nswrsm
+nswrgp = vcopt%nswrgr
+imligp = vcopt%imligr
+ircflp = vcopt%ircflu
+ischcp = vcopt%ischcv
+isstpp = vcopt%isstpc
 iescap = 0
 imucpp = 0
-idftnp = idften(ivar)
-iswdyp = iswdyn(ivar)
-imgrp  = imgr  (ivar)
-ncymxp = ncymax(ivar)
-nitmfp = nitmgf(ivar)
-iwarnp = iwarni(ivar)
-blencp = blencv(ivar)
-epsilp = epsilo(ivar)
-epsrsp = epsrsm(ivar)
-epsrgp = epsrgr(ivar)
-climgp = climgr(ivar)
-extrap = extrag(ivar)
-relaxp = relaxv(ivar)
+idftnp = vcopt%idften
+iswdyp = vcopt%iswdyn
+iwarnp = vcopt%iwarni
+blencp = vcopt%blencv
+epsilp = vcopt%epsilo
+epsrsp = vcopt%epsrsm
+epsrgp = vcopt%epsrgr
+climgp = vcopt%climgr
+extrap = vcopt%extrag
+relaxp = vcopt%relaxv
 ! all boundary convective flux with upwind
 icvflb = 0
+normp = -1.d0
+init   = 1
 
 call field_get_coefa_s(ivarfl(ivar), coefap)
 call field_get_coefb_s(ivarfl(ivar), coefbp)
@@ -790,20 +816,19 @@ call field_get_coefaf_s(ivarfl(ivar), cofafp)
 call field_get_coefbf_s(ivarfl(ivar), cofbfp)
 
 call codits &
-!==========
- ( idtvar , ivar   , iconvp , idiffp , ireslp , ndircp , nitmap , &
+ ( idtvar , init   , ivarfl(ivar)    , iconvp , idiffp , ndircp , &
    imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,          &
-   imgrp  , ncymxp , nitmfp , ipp    , iwarnp ,                   &
+   iwarnp , normp  ,                                              &
    blencp , epsilp , epsrsp , epsrgp , climgp , extrap ,          &
    relaxp , thetv  ,                                              &
-   rtpa(1,ivar)    , rtpa(1,ivar)    ,                            &
+   cvara_var       , cvara_var       ,                            &
    coefap , coefbp , cofafp , cofbfp ,                            &
    imasfl , bmasfl ,                                              &
-   viscf  , viscb  , viscce , viscf  , viscb  , viscce ,          &
+   viscf  , viscb  , viscf  , viscb  , viscce ,                   &
    weighf , weighb ,                                              &
    icvflb , ivoid  ,                                              &
-   rovsdt , smbr   , rtp(1,ivar)     , dpvar  ,                   &
+   rovsdt , smbr   , cvar_var        , dpvar  ,                   &
    rvoid  , rvoid  )
 
 ! Free memory
@@ -818,11 +843,11 @@ deallocate(weighf, weighb)
 
 #if defined(_CS_LANG_FR)
 
- 1000 format(/,'           RESOLUTION POUR LA VARIABLE ',A8,/)
+ 1000 format(/,'           Resolution pour la variable ',A8,/)
 
 #else
 
- 1000 format(/,'           SOLVING VARIABLE ',A8           ,/)
+ 1000 format(/,'           Solving variable ',A8           ,/)
 
 #endif
 
