@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -20,52 +20,46 @@
 
 !-------------------------------------------------------------------------------
 
-subroutine turbsa &
-!================
-
- ( nvar   , nscal  , ncepdp , ncesmp ,                            &
-   icepdc , icetsm , itypsm ,                                     &
-   dt     , rtp    , rtpa   , propce ,                            &
-   ckupdc , smacel ,                                              &
-   itypfb )
 
 !===============================================================================
 ! Purpose:
 ! --------
-
-! Solving op the equation of nusa, which is the scalar quantity defined by
-! the Spalart-Allmaras model for 1 time-step.
+!> \file turbsa.f90
+!> \brief Solving op the equation of \f$ \tilde{\nu} \f$, which is the scalar
+!>        quantity defined by the Spalart-Allmaras model for 1 time-step.
+!>
+!> Please refer to the
+!> <a href="../../theory.pdf#spalart"><b>Spalart-Allmaras model</b></a>
+!> section of the theory guide for more informations.
+!-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
 ! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-! nvar             ! i  ! <-- ! total number of variables                      !
-! nscal            ! i  ! <-- ! total number of scalars                        !
-! ncepdp           ! i  ! <-- ! number of cells with head loss                 !
-! ncesmp           ! i  ! <-- ! number of cells with mass source term          !
-! icepdc(ncelet    ! te ! <-- ! numero des ncepdp cellules avec pdc            !
-! icetsm(ncesmp    ! te ! <-- ! numero des cellules a source de masse          !
-! itypsm           ! te ! <-- ! type de source de masse pour les               !
-! (ncesmp,nvar)    !    !     !  variables (cf. ustsma)                        !
-! dt(ncelet)       ! ra ! <-- ! time step (per cell)                           !
-! rtp, rtpa        ! ra ! <-- ! calculated variables at cell centers           !
-!  (ncelet, *)     !    !     !  (at current and previous time steps)          !
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
-! tslagr           ! tr ! <-- ! terme de couplage retour du                    !
-!(ncelet,*)        !    !     !     lagrangien                                 !
-! ckupdc           ! tr ! <-- ! tableau de travail pour pdc                    !
-!  (ncepdp,6)      !    !     !                                                !
-! smacel           ! tr ! <-- ! valeur des variables associee a la             !
-! (ncesmp,*   )    !    !     !  source de masse                               !
-!                  !    !     !  pour ivar=ipr, smacel=flux de masse           !
-!__________________!____!_____!________________________________________________!
+!______________________________________________________________________________.
+!  mode           name          role
+!______________________________________________________________________________!
+!> \param[in]     nvar          total number of variables
+!> \param[in]     nscal         total number of scalars
+!> \param[in]     ncepdp        number of cells with head loss
+!> \param[in]     ncesmp        number of cells with mass source term
+!> \param[in]     icepdc        number of ncepdp cells with head losses
+!> \param[in]     icetsm        number of cells with mass source
+!> \param[in]     itypsm        type of mass source for the variables
+!>                               (cf. cs_user_mass_source_terms)
+!> \param[in]     dt            time step (per cell)
+!> \param[in]     ckupdc        work array for head losses
+!> \param[in]     smacel        value of variables associated to the
+!                               mass source
+!                               for ivar = ipr, smacel = mass flux
+!> \param[in]     itypfb        boundary face types
+!______________________________________________________________________________!
 
-!     Type: i (integer), r (real), s (string), a (array), l (logical),
-!           and composite types (ex: ra real array)
-!     mode: <-- input, --> output, <-> modifies data, --- work array
-!===============================================================================
+subroutine turbsa &
+ ( nvar   , nscal  , ncepdp , ncesmp ,                            &
+   icepdc , icetsm , itypsm ,                                     &
+   dt     ,                                                       &
+   ckupdc , smacel ,                                              &
+   itypfb )
 
 !===============================================================================
 ! Module files
@@ -81,7 +75,7 @@ use mesh
 use field
 use field_operator
 use parall
-use pointe, only: dispar
+use cs_c_bindings
 
 !===============================================================================
 
@@ -98,24 +92,26 @@ integer          itypfb(nfabor)
 integer          icvflb
 integer          ivoid(1)
 
-double precision dt(ncelet), rtp(ncelet,nflown:nvar), rtpa(ncelet,nflown:nvar)
-double precision propce(ncelet,*)
-double precision ckupdc(ncepdp,6), smacel(ncesmp,nvar)
+double precision dt(ncelet)
+double precision ckupdc(6,ncepdp), smacel(ncesmp,nvar)
 
 ! Local variables
 
 integer          iel   , ifac  , inc   , iccocg, iprev, ivar
 integer          iiun
 integer          nswrgp, imligp
-integer          iconvp, idiffp, ndircp, ireslp
-integer          nitmap, nswrsp, ircflp, ischcp, isstpp, iescap
-integer          imgrp , ncymxp, nitmfp
-integer          ipcvst, ipcvis, iflmas, iflmab
-integer          iwarnp, ipp
-integer          iptsta
-integer          ipcvto, ipcvlo
+integer          iconvp, idiffp, ndircp
+integer          nswrsp, ircflp, ischcp, isstpp, iescap
+integer          iflmas, iflmab
+integer          iwarnp
+integer          istprv
 integer          ipatrg
 integer          imucpp, idftnp, iswdyp
+integer          f_id
+integer          init
+integer          key_t_ext_id
+integer          iroext
+integer          iviext
 
 double precision romvsd
 double precision visct , rom
@@ -126,7 +122,8 @@ double precision tuexpn
 double precision cofbnu
 double precision chi  , chi3, taussa, nusa, distbf, fw, fv1, fv2
 double precision gsa , rsa , dsigma, cv13
-double precision surfn, nu0, dsa0, hssa, omega, sbar, cst2, cst3
+double precision nu0, dsa0, hssa, omega, sbar, cst2, cst3
+double precision normp
 
 double precision rvoid(1)
 
@@ -142,7 +139,13 @@ double precision, allocatable, dimension(:) :: dpvar
 double precision, allocatable, dimension(:) :: csab1r, rotfct
 double precision, dimension(:), pointer :: imasfl, bmasfl
 double precision, dimension(:), pointer :: brom, crom, cromo
+double precision, dimension(:), pointer :: cvar_nusa, cvara_nusa
+double precision, dimension(:), pointer :: viscl, cvisct
 double precision, dimension(:), pointer :: coefap, coefbp, cofafp, cofbfp
+double precision, dimension(:), pointer :: c_st_nusa_p
+double precision, dimension(:), pointer :: w_dist
+
+type(var_cal_opt) :: vcopt_nusa
 
 !===============================================================================
 
@@ -150,6 +153,9 @@ double precision, dimension(:), pointer :: coefap, coefbp, cofafp, cofbfp
 ! 1. Initialization
 !===============================================================================
 
+! Time extrapolation?
+call field_get_key_id("time_extrapolated", key_t_ext_id)
+
 ! Allocate temporary arrays for the turbulence resolution
 allocate(viscf(nfac), viscb(nfabor))
 allocate(tsimp(ncelet))
@@ -163,40 +169,46 @@ allocate(tsexp(ncelet))
 allocate(dpvar(ncelet))
 
 call field_get_val_s(icrom, crom)
-ipcvst = ipproc(ivisct)
-ipcvis = ipproc(iviscl)
+call field_get_val_s(ivisct, cvisct)
+call field_get_val_s(iviscl, viscl)
 call field_get_val_s(ibrom, brom)
+call field_get_val_s(ivarfl(inusa), cvar_nusa)
+call field_get_val_prev_s(ivarfl(inusa), cvara_nusa)
 
 call field_get_key_int(ivarfl(iu), kimasf, iflmas)
 call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
 call field_get_val_s(iflmas, imasfl)
 call field_get_val_s(iflmab, bmasfl)
 
+call field_get_id("wall_distance", f_id)
+call field_get_val_s(f_id, w_dist)
+
 ivar   = inusa
-thetv  = thetav(ivar)
+
+call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt_nusa)
+
+thetv  = vcopt_nusa%thetav
+
+call field_get_key_int(ivarfl(inusa), kstprv, istprv)
+if (istprv.ge.0) then
+  call field_get_val_s(istprv, c_st_nusa_p)
+  istprv = 1
+endif
 
 call field_get_val_s(icrom, cromo)
-ipcvto = ipcvst
-ipcvlo = ipcvis
 
-if(isto2t.gt.0) then
+if (istprv.ge.0) then
+  call field_get_key_int(icrom, key_t_ext_id, iroext)
   if (iroext.gt.0) then
     call field_get_val_prev_s(icrom, cromo)
   endif
-  if(iviext.gt.0) then
-    ipcvto = ipproc(ivista)
-    ipcvlo = ipproc(ivisla)
+  call field_get_key_int(ivisct, key_t_ext_id, iviext)
+  if (iviext.gt.0) then
+    call field_get_val_prev_s(ivisct, cvisct)
   endif
 endif
 
-! If source terms are extrapolated
-if(isto2t.gt.0) then
-  iptsta = ipproc(itstua)
-else
-  iptsta = 0
-endif
-
-if(iwarni(inusa).ge.1) then
+if (vcopt_nusa%iwarni.ge.1) then
   write(nfecra,1000)
 endif
 
@@ -244,6 +256,8 @@ allocate(grad(3,ncelet))
 
 ! Compute the gradient of nusa
 
+iccocg = 1
+
 call field_gradient_scalar(ivarfl(inusa), iprev, imrgra, inc,       &
                            iccocg,                                  &
                            grad)
@@ -268,7 +282,8 @@ deallocate(grad)
 !      stored in rhssa
 !===============================================================================
 
-! Herebelow, we only handle  the case where all the walls have the same roughness
+! Herebelow, we only handle  the case where all the walls have
+! the same roughness
 ! To extend it, we should be able to link every fluid cell to a boundary face
 ! (and then give it the appropriate roughness value)
 
@@ -289,9 +304,9 @@ do ifac = 1, nfabor
   if (ipatrg.ne.0) exit
 enddo
 
-if(irangp.ge.0) then
+if (irangp.ge.0) then
   call parcpt(ipatrg)
-  if(ipatrg.ne.0) then
+  if (ipatrg.ne.0) then
     call parsom(dsa0)
     dsa0=dsa0/ipatrg
   endif
@@ -307,8 +322,7 @@ if (irccor.eq.1) then
   allocate(rotfct(ncel))
 
   ! Compute the rotation function (w1 array not used)
-  call rotcor(dt, rtpa, rotfct, w1)
-  !==========
+  call rotcor(dt, rotfct, w1)
 
   do iel = 1, ncel
     csab1r(iel) = csab1*rotfct(iel)
@@ -327,17 +341,17 @@ endif
 !                                 visct is visct^n
 do iel = 1, ncel
 
-  visct = propce(iel,ipcvto)
+  visct = cvisct(iel)
   rom   = cromo(iel)
-  ! Kinematic viscosity
-  nu0   = propce(iel,ipcvis)/rom
+  ! kinematic viscosity
+  nu0   = viscl(iel)/rom
   ! We have to know if there is any rough wall
-  distbf= dispar(iel)
+  distbf= w_dist(iel)
   ! viscosity of SA
-  nusa  = rtpa(iel,inusa)
+  nusa  = cvara_nusa(iel)
   chi   = nusa/nu0
   ! If we have a rough wall
-  if(ipatrg.ne.0) then
+  if (ipatrg.ne.0) then
     distbf = distbf + dsa0
     chi  = chi + 0.5d0* hssa/distbf
   endif
@@ -370,7 +384,7 @@ do iel = 1, ncel
      dsigma * csab2*trgrdn(iel)+csab1r(iel)*taussa*nusa-csaw1*fw*(nusa/distbf)**2)
 
   ! Implicitation of the negative source terms of the SA equation.
-  ! NB : this term could be negative, and if so, then we explicit it.
+  ! NB: this term could be negative, and if so, then we explicit it.
   tinssa(iel) = (max(csaw1*fw*nusa/distbf**2-csab1r(iel)*taussa,0.d0)         &
                       )*rom*volume(iel)
 
@@ -393,7 +407,6 @@ do iel = 1, ncel
 enddo
 
 call cs_user_turbulence_source_terms &
-!===================================
  ( nvar   , nscal  , ncepdp , ncesmp ,                            &
    ivarfl(inusa)   ,                                              &
    icepdc , icetsm , itypsm ,                                     &
@@ -402,27 +415,27 @@ call cs_user_turbulence_source_terms &
 
 !===============================================================================
 ! 6. User source terms and d/dt(rho) and div(rho u) are taken into account
-
 !      stored in rhssa
 !===============================================================================
 
 ! If source terms are extrapolated
-if (isto2t.gt.0) then
+if (istprv.ge.0) then
 
   do iel = 1, ncel
 
      ! Ts^(n-1) (Term User EXPlicit Nusa)
-     tuexpn =propce(iel,iptsta)
+     tuexpn =c_st_nusa_p(iel)
 
     ! The explicit user source terms are stored for the next time step
-    ! On stoque les TS explicites du temps n (TS model + TS utilisateur)
-    propce(iel,iptsta) = rhssa(iel) + tsexp(iel)
+    ! We store the explicit source terms of time n (model source terms + user
+    ! source terms)
+    c_st_nusa_p(iel) = rhssa(iel) + tsexp(iel)
 
 
     ! --- Extrapolated explicit source terms
     rhssa(iel) = - thetst*tuexpn
 
-    rhssa(iel) = tsimp(iel)*rtpa(iel,inusa) + rhssa(iel)
+    rhssa(iel) = tsimp(iel)*cvara_nusa(iel) + rhssa(iel)
 
     ! --- Implicit user source terms
     ! Here it is assumed that -tsimp > 0. That is why it is implicited
@@ -433,7 +446,7 @@ if (isto2t.gt.0) then
 ! If source terms are not extrapolated, then they are directly added to the RHS
 else
   do iel = 1, ncel
-    rhssa(iel) = rhssa(iel) + tsimp(iel)*rtpa(iel,inusa) + tsexp(iel)
+    rhssa(iel) = rhssa(iel) + tsimp(iel)*cvara_nusa(iel) + tsexp(iel)
 
     ! --- Implicit user source terms
     tinssa(iel) = tinssa(iel) + max(-tsimp(iel),zero)
@@ -448,7 +461,7 @@ do iel = 1, ncel
 
   ! tinssa already contains the negativ implicited source term
   tinssa(iel) = tinssa(iel)                                        &
-               +istat(inusa)*romvsd
+               +vcopt_nusa%istat*romvsd
 enddo
 
 
@@ -461,7 +474,7 @@ enddo
 !===============================================================================
 ! 8. Explicit mass source terms
 
-!    Gamma*RTPAi is stored in w1
+!    Gamma.var_prev is stored in w1
 !===============================================================================
 
 if (ncesmp.gt.0) then
@@ -470,22 +483,21 @@ if (ncesmp.gt.0) then
   iiun = 1
 
   ! --- Explicit and Implicit part
-  !     -Gamma RTPA is added to the RHS and Gamma*theta to tinssa
+  !     -Gamma.var_prev is added to the RHS and Gamma to tinssa
   ivar = inusa
 
   call catsma &
-  !==========
  ( ncelet , ncel   , ncesmp , iiun   ,                            &
-   isto2t , thetv  ,                                              &
+   isto2t ,                                                       &
    icetsm , itypsm(1,ivar) ,                                      &
-   volume , rtpa(1,ivar) , smacel(1,ivar) , smacel(1,ipr) ,       &
+   volume , cvara_nusa     , smacel(1,ivar) , smacel(1,ipr) ,     &
    rhssa  , tinssa , w1 )
 
-  ! --- Explicit part: Gamma*RTPAi
-  !     (if we extrapolate source terms, Gamma*RTPAi is stored in propce)
-  if(isto2t.gt.0) then
+  ! --- Explicit part: Gamma Pinj
+  !     (if we extrapolate source terms, Gamma.var_prev is stored in prev. TS)
+  if (istprv.ge.0) then
     do iel = 1, ncel
-      propce(iel,iptsta) = propce(iel,iptsta) + w1(iel)
+      c_st_nusa_p(iel) = c_st_nusa_p(iel) + w1(iel)
     enddo
   else
     do iel = 1, ncel
@@ -496,10 +508,10 @@ if (ncesmp.gt.0) then
 endif
 
 ! Finalization of the extrapolated explicit source terms
-if(isto2t.gt.0) then
+if (istprv.ge.0) then
   thetp1 = 1.d0 + thetst
   do iel = 1, ncel
-    rhssa(iel) = rhssa(iel) + thetp1    * propce(iel,iptsta)
+    rhssa(iel) = rhssa(iel) + thetp1 * c_st_nusa_p(iel)
   enddo
 endif
 
@@ -514,60 +526,23 @@ call field_get_coefb_s(ivarfl(ivar), coefbp)
 call field_get_coefaf_s(ivarfl(ivar), cofafp)
 call field_get_coefbf_s(ivarfl(ivar), cofbfp)
 
-ipp    = ipprtp(ivar)
-
 ! Face viscosity
 
-if (idiff(ivar).ge.1) then
+if (vcopt_nusa%idiff.ge.1) then
 
   do iel = 1, ncel
     rom = crom(iel)
 
     ! diffusibility: 1/sigma*(mu_laminaire+ rho*nusa)
-    w1(iel) = dsigma *( propce(iel,ipcvis)                        &
-                        + idifft(ivar)*rtpa(iel,inusa)*rom )
+    w1(iel) = dsigma *( viscl(iel)                                &
+                        + vcopt_nusa%idifft*cvara_nusa(iel)*rom )
   enddo
 
   call viscfa                                                     &
-  !==========
  ( imvisf ,                                                       &
    w1     ,                                                       &
    viscf  , viscb  )
 
-  ! Be carefull with the walls:
-  !  If we have a smooth wall then nusa is zero at the wall
-  !  If we have a rough wall then nusa_wall*(1- IprF/d0)=Vipr
-
-  do ifac = 1, nfabor
-
-    iel   = ifabor(ifac)
-    surfn = surfbn(ifac)
-
-    ! Smooth wall
-    if (itypfb(ifac).eq.iparoi) then
-      viscb(ifac) = dsigma * propce(iel,ipcvis)*surfn/distb(ifac)
-
-    ! Rough wall
-    elseif (itypfb(ifac).eq.iparug) then
-
-      rom = crom(iel)
-
-      ! dsa0 is recomputed in case of many different roughness
-      cofbnu = coefbp(ifac)
-
-      ! Roughness of the wall
-      dsa0   = distb(ifac) *cofbnu/(1.d0-cofbnu)
-      hssa   = exp(8.5d0*xkappa)*dsa0
-
-      ! For rough walls: nusa_F*(IprF/d0+1) = nusa_Ipr
-      viscb(ifac) = dsigma * ( propce(iel,ipcvis)                    &
-                   + idifft(ivar)*rtpa(iel,inusa)*rom                &
-                   * dsa0/(distb(ifac)+dsa0)            )*surfn/distb(ifac)
-
-    endif
-
-  enddo
-
 else
 
   do ifac = 1, nfac
@@ -581,59 +556,54 @@ endif
 
 ! --- Solving
 
-iconvp = iconv (ivar)
-idiffp = idiff (ivar)
-ireslp = iresol(ivar)
-ndircp = ndircl(ivar)
-nitmap = nitmax(ivar)
-nswrsp = nswrsm(ivar)
-nswrgp = nswrgr(ivar)
-imligp = imligr(ivar)
-ircflp = ircflu(ivar)
-ischcp = ischcv(ivar)
-isstpp = isstpc(ivar)
+iconvp = vcopt_nusa%iconv
+idiffp = vcopt_nusa%idiff
+ndircp = vcopt_nusa%ndircl
+nswrsp = vcopt_nusa%nswrsm
+nswrgp = vcopt_nusa%nswrgr
+imligp = vcopt_nusa%imligr
+ircflp = vcopt_nusa%ircflu
+ischcp = vcopt_nusa%ischcv
+isstpp = vcopt_nusa%isstpc
 iescap = 0
 imucpp = 0
-idftnp = idften(ivar)
-iswdyp = iswdyn(ivar)
-imgrp  = imgr  (ivar)
-ncymxp = ncymax(ivar)
-nitmfp = nitmgf(ivar)
-iwarnp = iwarni(ivar)
-blencp = blencv(ivar)
-epsilp = epsilo(ivar)
-epsrsp = epsrsm(ivar)
-epsrgp = epsrgr(ivar)
-climgp = climgr(ivar)
-extrap = extrag(ivar)
-relaxp = relaxv(ivar)
-thetap = thetav(ivar)
+idftnp = vcopt_nusa%idften
+iswdyp = vcopt_nusa%iswdyn
+iwarnp = vcopt_nusa%iwarni
+blencp = vcopt_nusa%blencv
+epsilp = vcopt_nusa%epsilo
+epsrsp = vcopt_nusa%epsrsm
+epsrgp = vcopt_nusa%epsrgr
+climgp = vcopt_nusa%climgr
+extrap = vcopt_nusa%extrag
+relaxp = vcopt_nusa%relaxv
+thetap = vcopt_nusa%thetav
 ! all boundary convective flux with upwind
 icvflb = 0
+normp = -1.d0
+init   = 1
 
 call codits &
-!==========
- ( idtvar , ivar   , iconvp , idiffp , ireslp , ndircp , nitmap , &
+ ( idtvar , init   , ivarfl(ivar)    , iconvp , idiffp , ndircp , &
    imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,          &
-   imgrp  , ncymxp , nitmfp , ipp    , iwarnp ,                   &
+   iwarnp , normp  ,                                              &
    blencp , epsilp , epsrsp , epsrgp , climgp , extrap ,          &
    relaxp , thetap ,                                              &
-   rtpa(1,ivar)    , rtpa(1,ivar)    ,                            &
+   cvara_nusa      , cvara_nusa      ,                            &
    coefap , coefbp , cofafp , cofbfp ,                            &
    imasfl , bmasfl ,                                              &
-   viscf  , viscb  , rvoid  , viscf  , viscb  , rvoid  ,          &
+   viscf  , viscb  , viscf  , viscb  , rvoid  ,                   &
    rvoid  , rvoid  ,                                              &
    icvflb , ivoid  ,                                              &
-   tinssa , rhssa  , rtp(1,ivar)     , dpvar ,                    &
+   tinssa , rhssa  , cvar_nusa       , dpvar ,                    &
    rvoid  , rvoid  )
 
 !===============================================================================
 ! 10. Clipping
 !===============================================================================
 
-call clipsa(ncelet, ncel, nvar, rtp)
-!==========
+call clipsa(ncel)
 
 ! Free memory
 deallocate(viscf, viscb)
@@ -652,12 +622,12 @@ deallocate(dpvar)
 #if defined(_CS_LANG_FR)
 
  1000 format(/,                                                   &
-'   ** RESOLUTION DE SPALART-ALLMARAS                         ',/,&
+'   ** Resolution de Spalart-Allmaras                         ',/,&
 '      ------------------------------------                   ',/)
 #else
 
  1000 format(/,                                                   &
-'   ** SOLVING SPALART-ALLMARAS      '                         ,/,&
+'   ** Solving Spalart-Allmaras      '                         ,/,&
 '      ------------------------------'                         ,/)
 #endif
 
